/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileUploadException;

import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.ViewContextPerspective;

/**
 * The ContextImage servlet serves the image generated by ViewContextPerspective to the client.
 * 
 * @author sschaffe
 * @see at.srfg.ikewiki.layout.perspectives.ViewContextPerspective
 *
 */
public class ContextImage extends HttpServlet {

	private Main main = null;
	/**
	 * 
	 */
	public ContextImage() {
		super();
		// TODO Auto-generated constructor stub
	}

	
	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		super.init();
		try {
			main = Main.getInstance();
		} catch(DBException ex) {
			throw new ServletException("error initialising Main class",ex);
		}
	}

	
	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		if(request.getCharacterEncoding() == null) {
			request.setCharacterEncoding("UTF-8");		
		}

		try {
			RequestData parameters = main.parseRequestData(request);
		
			String title_prefix = "";

			String action = parameters.get("action");
			if(action == null) {
				action = "view";
			}


			String req_title = parameters.get("title");
			
			String article_ns, article_title;
			if(req_title == null) {
		 		article_ns    = "";
		 		article_title = "FrontPage";
		 	} else {
		 		String[] components = req_title.split(":");
		 		if(components.length == 2) {
			 		article_ns    = components[0];
			 		article_title = components[1];
			 	} else if(components.length == 1) {
			 		article_ns    = "";
			 		article_title = components[0];
			 	} else {
			 		article_ns    = "";
			 		article_title = "";			 		
			 	}
		 	}

			ViewContextPerspective cv = main.getViewContextPerspective(article_ns,article_title);
			byte[] content = cv.getImage(); 

			response.setContentType("image/png");
			response.setContentLength(content.length);
			response.getOutputStream().write(content);
			response.getOutputStream().flush();
			
		} catch(DBException ex) {
			throw new ServletException("Error retrieving resource",ex);		
		} catch(FileUploadException ex) {
			throw new ServletException("Error parsing parameters",ex);
		}
	}

	
	
}
