/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.WikiImage;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * @author sschaffe
 *
 */
public class ImageDownload extends HttpServlet {

	private static Log log = LogFactory.getLog(ImageDownload.class);
	
	private Main main = null;

	/**
	 * 
	 */
	public ImageDownload() {
		super();
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		super.init();
		try {
			main = Main.getInstance();
		} catch(DBException ex) {
			throw new ServletException("error initialising Main class",ex);
		}
	}




	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			RequestData parameters = main.parseRequestData(request);
		
			String title = parameters.get("title");
						
			
			if(title != null) {
				WikiResource res = main.getResourceByQTitle(title);
				
				
				WikiImage img = main.getStorageBackend().getImageByNamespaceTitle(res.getNamespace().getPrefix(),res.getTitle());
				
				response.setContentType(img.getMimeType());
				response.setContentLength(img.getBytes().length);
				response.getOutputStream().write(img.getBytes());
			}
			
			
		} catch(DBException ex) {
			log.error("error while connecing with database",ex);
			throw new ServletException("Error retrieving resource",ex);		
		} catch(FileUploadException ex) {
			log.error("error while parsing parameters",ex);
			throw new ServletException("Error parsing parameters",ex);
		}
	}


}
