/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki;

import java.io.UnsupportedEncodingException;
import java.lang.ref.SoftReference;
import java.net.URISyntaxException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
import org.apache.commons.fileupload.DiskFileUpload;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import at.srfg.ikewiki.actions.Action;
import at.srfg.ikewiki.actions.ActionRegistry;
import at.srfg.ikewiki.data.Article;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.User;
import at.srfg.ikewiki.data.WikiLink;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.db.DBStore;
import at.srfg.ikewiki.db.NamespaceNotFoundException;
import at.srfg.ikewiki.db.RDFStore;
import at.srfg.ikewiki.db.ResourceChangeListener;
import at.srfg.ikewiki.db.StorageBackend;
import at.srfg.ikewiki.editing.templates.AnnotationPropertyTemplate;
import at.srfg.ikewiki.editing.templates.ClassTemplate;
import at.srfg.ikewiki.editing.templates.DatatypePropertyTemplate;
import at.srfg.ikewiki.editing.templates.ObjectPropertyTemplate;
import at.srfg.ikewiki.editing.templates.ResourceTemplate;
import at.srfg.ikewiki.editing.templates.Template;
import at.srfg.ikewiki.layout.ViewItemFactory;
import at.srfg.ikewiki.layout.perspectives.EditContentPerspective;
import at.srfg.ikewiki.layout.perspectives.EditFormPerspective;
import at.srfg.ikewiki.layout.perspectives.EditMetadataPerspective;
import at.srfg.ikewiki.layout.perspectives.EditMozilePerspective;
import at.srfg.ikewiki.layout.perspectives.EditReferencesPerspective;
import at.srfg.ikewiki.layout.perspectives.EditTypesPerspective;
import at.srfg.ikewiki.layout.perspectives.ErrorPerspective;
import at.srfg.ikewiki.layout.perspectives.LoginPerspective;
import at.srfg.ikewiki.layout.perspectives.NamespacePerspective;
import at.srfg.ikewiki.layout.perspectives.NewClassPerspective;
import at.srfg.ikewiki.layout.perspectives.NewPropertyPerspective;
import at.srfg.ikewiki.layout.perspectives.NewResourcePerspective;
import at.srfg.ikewiki.layout.perspectives.Perspective;
import at.srfg.ikewiki.layout.perspectives.RecentChangesPerspective;
import at.srfg.ikewiki.layout.perspectives.RedirectPerspective;
import at.srfg.ikewiki.layout.perspectives.RelatedPerspective;
import at.srfg.ikewiki.layout.perspectives.SearchPerspective;
import at.srfg.ikewiki.layout.perspectives.SiteMapPerspective;
import at.srfg.ikewiki.layout.perspectives.SparqlPerspective;
import at.srfg.ikewiki.layout.perspectives.TooltipPerspective;
import at.srfg.ikewiki.layout.perspectives.VariablesPerspective;
import at.srfg.ikewiki.layout.perspectives.ViewArticlePerspective;
import at.srfg.ikewiki.layout.perspectives.ViewContextPerspective;
import at.srfg.ikewiki.layout.perspectives.ViewHistoryPerspective;
import at.srfg.ikewiki.layout.perspectives.XMLExchangePerspective;
import at.srfg.ikewiki.layout.portlet.Portlet;
import at.srfg.ikewiki.layout.portlet.PortletItem;
import at.srfg.ikewiki.layout.views.View;
import at.srfg.ikewiki.layout.views.ViewRegistry;
import at.srfg.ikewiki.permission.PermissionManager;
import at.srfg.ikewiki.render.RenderingPipeline;

/**
 * The Main class provides functionality common to all pages. Main is a singleton, so one instance
 * per application should suffice. 
 * 
 * @author sschaffe
 *
 */
public class Main implements ResourceChangeListener {

	//public final static String BASE_URI = "http://ikewiki.srfg.at/Internal#";
	
//	// TODO: these should really be database lookups instead
//	public final static Namespace NS_INTERNAL = new Namespace("","http://ikewiki.srfg.at/Internal#");
//	public final static Namespace NS_USER = new Namespace("User","http://ikewiki.srfg.at/User#");
//	public final static Namespace NS_IMAGE = new Namespace("Image","http://ikewiki.srfg.at/Image#");
	
	
	private static Log log = LogFactory.getLog(Main.class);

	
	
	
	/**
	 * Portlets to be displayed as boxes in the page
	 */
	private Vector<Portlet> portlets;
	
//	static {
		
		
		
//		navigationPortlet.addPortletItem(new PortletItem("n-mainpage","Main Page","index.jsp"));;
//		//navigationPortlet.addPortletItem(new PortletItem("n-sitemap","Site Map","index.jsp?title=Special:SiteMap"));;
//		//navigationPortlet.addPortletItem(new PortletItem("n-recentchanges","Recent Changes","index.jsp?title=Special:RecentChanges"));;
//		navigationPortlet.addPortletItem(new PortletItem("n-related","Related Pages","index.jsp?title={title}&amp;action=related"));;
//		navigationPortlet.addPortletItem(new PortletItem("n-flush","Flush Caches","index.jsp?action=flush"));;
//		//navigationPortlet.addPortletItem(new PortletItem("n-contact","Contact","index.jsp?title=Special:Contact"));;
//		navigationPortlet.addPortletItem(new PortletItem("n-help","Help","index.jsp?title=help:Contents"));;
		
//		portlets = new Vector<Portlet>();
//		portlets.add(navigationPortlet);
		
//		editPortlet = new Portlet("p-edit","edit");
//		editPortlet.addPortletItem(new PortletItem("n-newresource","New Resource","index.jsp?action=new_resource"));;
//		editPortlet.addPortletItem(new PortletItem("n-newresource","New Class","index.jsp?action=new_class"));;
//		editPortlet.addPortletItem(new PortletItem("n-newresource","New Property","index.jsp?action=new_property"));;
//		editPortlet.addPortletItem(new PortletItem("n-newimage","New Image","index.jsp?action=upload_image"));;
//		editPortlet.addPortletItem(new PortletItem("n-users","Manage Users","index.jsp?action=edit_users"));;
//		editPortlet.addPortletItem(new PortletItem("n-roles","Manage Roles","index.jsp?action=edit_roles"));;
//		editPortlet.addPortletItem(new PortletItem("n-namespace","Manage Namespaces","index.jsp?action=namespaces"));;
//		//editPortlet.addPortletItem(new PortletItem("n-variables","Manage Variables","index.jsp?action=variables"));;
//		portlets.add(editPortlet);
		
		
//		rdfPortlet = new Portlet("p-rdf","tools");
//		rdfPortlet.addPortletItem(new PortletItem("r-exchange","XML Exchange","exchange.jsp?title={title}"));
//		rdfPortlet.addPortletItem(new PortletItem("r-upload","Upload RDF/XML","index.jsp?action=upload_rdf"));
//		rdfPortlet.addPortletItem(new PortletItem("r-sparql","SPARQL Query","index.jsp?action=search_sparql"));
//		rdfPortlet.addPortletItem(new PortletItem("r-download","Download RDF/XML","RDFDownload?format=RDF/XML"));
//		rdfPortlet.addPortletItem(new PortletItem("r-downloadn3","Download RDF/N3","RDFDownload?format=N3"));
//		rdfPortlet.addPortletItem(new PortletItem("r-downloadturtle","Download TURTLE","RDFDownload?format=TURTLE"));
//		rdfPortlet.addPortletItem(new PortletItem("r-downloadturtle","Download N-TRIPLE","RDFDownload?format=N-TRIPLE"));
//		
//		portlets.add(rdfPortlet);
//	}
	
	/**
	 * Actions for a page
	 */
	private static Vector<String> actions;
	
	static {
		actions = new Vector<String>();
		actions.add("view");
		actions.add("context");
		actions.add("edit");
		actions.add("edit_content");
		//actions.add("edit_form");
		actions.add("edit_metadata");
		actions.add("edit_types");
		actions.add("edit_references");
		actions.add("discussion");
		actions.add("history");		
	}

	
	private static Hashtable<String,String> special_to_actions;
	
	static {
		special_to_actions = new Hashtable<String,String>();
		special_to_actions.put("EditNamespaces","namespaces");
		special_to_actions.put("Login","login");
	}
	
	
	private Map<String,Template> templates;
	
	
	
	private StorageBackend database;

	// we cache article views because rendering is expensive
	private HashMap<String,SoftReference<ViewArticlePerspective>> av_cache;
	private HashMap<String,SoftReference<TooltipPerspective>> tt_cache;

	// caches for resource lookups
	private HashMap<String,SoftReference<WikiResource>> resource_qtitle_cache, resource_uri_cache;

	
	// a view registry for view plugins
	private ViewRegistry view_registry;
	
	// an action handler for all actions
	private ActionRegistry action_registry;
	
	
	private Configuration config;
	
	
	private RenderingPipeline renderer;
	
	private PermissionManager permission_manager;
	
	private Main() throws DBException {
		init();
	}
	
	
	
	private void init() throws DBException {
		// initialise base configuration from config file
		try {
			config = new PropertiesConfiguration("config.properties");
			((PropertiesConfiguration)config).setReloadingStrategy(new FileChangedReloadingStrategy());
			log.info("loaded configuration file config.properties");
		} catch(ConfigurationException ex) {
			log.error("error while loading configuration file, starting with empty configuration",ex);
			config = new PropertiesConfiguration();
		}

		
		// set log level for the whole application
		Logger main_logger = Logger.getLogger("at.srfg.ikewiki");
		main_logger.setLevel(Level.toLevel(config.getString("logging.baselevel"),Level.DEBUG));
		
		Logger db_logger = Logger.getLogger("at.srfg.ikewiki.db");
		db_logger.setLevel(Level.DEBUG);
		
		
		
		try {
			database = new StorageBackend(this);
			database.init();

			database.addResourceChangeListener(this);
		} catch(DBException ex) {			
			// TODO: sensible error handling
			log.error("error while opening storage backend",ex);
			throw ex;
		}
		av_cache = new HashMap<String,SoftReference<ViewArticlePerspective>>();
		tt_cache = new HashMap<String,SoftReference<TooltipPerspective>>();

		view_registry = new ViewRegistry(this);
			
		action_registry = new ActionRegistry(this);
		
		resource_qtitle_cache = new HashMap<String,SoftReference<WikiResource>>();
		resource_uri_cache = new HashMap<String,SoftReference<WikiResource>>();

		initTemplates();
		
		renderer = new RenderingPipeline(this);
		
		permission_manager = new PermissionManager(this);
		
		initPortlets();
	}
	

	
	/**
	 * Reinitialise components that can be reinitalised.
	 *
	 */
	public synchronized void reInit() throws DBException {
		this.database.close();
		init();
		this.database.reIndex();
	}

	private void initTemplates() {
		templates = new LinkedHashMap<String,Template>();
		addTemplate(new ResourceTemplate(this));
		addTemplate(new ClassTemplate(this));
		addTemplate(new ObjectPropertyTemplate(this));
		addTemplate(new AnnotationPropertyTemplate(this));
		addTemplate(new DatatypePropertyTemplate(this));
	}
	
	
	/**
	 * Initialise portlet boxes.
	 *
	 */
	private void initPortlets() {
		portlets = new Vector<Portlet>();

		Portlet navigationPortlet, editPortlet, rdfPortlet;

		// navigation portlet
		navigationPortlet = new Portlet(this.getPermissionManager(),"p-navigation","Navigation");
		
		navigationPortlet.addPortletItem(new PortletItem("Main Page",getAction("view")));
		navigationPortlet.addPortletItem(new PortletItem("Related Pages",getAction("related")));;
		navigationPortlet.addPortletItem(new PortletItem("Recent Changes",getAction("recent")));;
		navigationPortlet.addPortletItem(new PortletItem("Flush Caches",getAction("flush")));;
		navigationPortlet.addPortletItem(new PortletItem("Help",getAction("view"), Collections.singletonMap("title","help:Contents")));
		
		portlets.add(navigationPortlet);

		// edit portlet
		editPortlet = new Portlet(this.getPermissionManager(),"p-edit","edit");
		editPortlet.addPortletItem(new PortletItem("New Resource",getAction("new_resource")));
		editPortlet.addPortletItem(new PortletItem("New Class",getAction("new_class")));
		editPortlet.addPortletItem(new PortletItem("New Property",getAction("new_property")));
		editPortlet.addPortletItem(new PortletItem("New Image",getAction("upload_image")));
		editPortlet.addPortletItem(new PortletItem("Manage Users",getAction("edit_users")));
		editPortlet.addPortletItem(new PortletItem("Manage Roles",getAction("edit_roles")));
		editPortlet.addPortletItem(new PortletItem("Manage Namespaces",getAction("namespaces")));
		//editPortlet.addPortletItem(new PortletItem("Manage Variables",getAction("variables")));
		portlets.add(editPortlet);

		// RDF portlet
		rdfPortlet = new Portlet(this.getPermissionManager(),"p-rdf","tools");
		rdfPortlet.addPortletItem(new PortletItem("r-exchange","XML Exchange","exchange.jsp?title={title}"));
		rdfPortlet.addPortletItem(new PortletItem("Upload RDF/XML",getAction("upload_rdf")));
		rdfPortlet.addPortletItem(new PortletItem("SPARQL Query",getAction("search_sparql")));
		rdfPortlet.addPortletItem(new PortletItem("r-download","Download RDF/XML","RDFDownload?format=RDF/XML"));
		rdfPortlet.addPortletItem(new PortletItem("r-downloadn3","Download RDF/N3","RDFDownload?format=N3"));
		rdfPortlet.addPortletItem(new PortletItem("r-downloadturtle","Download TURTLE","RDFDownload?format=TURTLE"));
		rdfPortlet.addPortletItem(new PortletItem("r-downloadturtle","Download N-TRIPLE","RDFDownload?format=N-TRIPLE"));
		
		portlets.add(rdfPortlet);

	}
	
	private void addTemplate(Template t) {
		templates.put(t.getName(),t);
	}
			
	
	
	private static Main instance = null;
	
	private ViewItemFactory viewItemFactory = null;
	private String viewItemFactoryType = "at.srfg.ikewiki.layout.viewitem.BasicViewItemFactory";
	

	/**
	 * Main is a singleton; getInstance() returns the single instance.
	 * @return
	 */
	public static synchronized Main getInstance() throws DBException {
		if(instance == null) {
			instance = new Main();			
		}
		return instance;
	}
		
	
	public Configuration getConfig() {
		return config;
	}

	/**
	 * Retrieve article identified by namespace prefix and title from the database and 
	 * create an ViewArticlePerspective instance. This method always retrieves the latest revision
	 * of an article.
	 * <p>
	 * Corresponds to the parameter action=view
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new ViewArticlePerspective instance for the article
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public ViewArticlePerspective getViewArticlePerspective(String namespace, String title) throws DBException {
		synchronized(av_cache) {
			SoftReference<ViewArticlePerspective> ref = av_cache.get(namespace+title);
			if(ref != null && ref.get() != null) {
				log.debug("cache hit for article view "+namespace+":"+title);
				return ref.get();
			} else {
				ViewArticlePerspective a = new ViewArticlePerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));;
				av_cache.put(namespace+title,new SoftReference<ViewArticlePerspective>(a));
				log.debug("cache miss for article view "+namespace+":"+title);
				return a; 
			}
		}
	}


	/**
	 * Retrieve a specific revision of an article identified by namespace prefix and title
	 * from the database and create an ViewArticlePerspective instance. The combination of revision number
	 * and article namespace/title must exist, otherwise a DBException is thrown.
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @param revision revision number to retrieve
	 * @return a new ViewArticlePerspective instance for the article revision
	 * @throws DBException in case the article cannot be retrieved with this revision number.
	 */
	public ViewArticlePerspective getViewArticlePerspective(String namespace, String title, long revision) throws DBException {
		return new ViewArticlePerspective(this,getStorageBackend().getArticleByRevisionId(namespace,title,revision));
	}

	
	
	/**
	 * Retrieve article identified by namespace prefix and title from the database and 
	 * create an ViewArticlePerspective instance. This method always retrieves the latest revision
	 * of an article.
	 * <p>
	 * Corresponds to the parameter action=view
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new ViewArticlePerspective instance for the article
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public EditFormPerspective getEditFormPerspective(String namespace, String title) throws DBException {
		return new EditFormPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
	}

	
	/**
	 * Retrieve article identified by namespace prefix and title from the database and 
	 * create an ViewArticlePerspective instance. This method always retrieves the latest revision
	 * of an article.
	 * <p>
	 * Corresponds to the parameter action=view
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new ViewArticlePerspective instance for the article
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public TooltipPerspective getTooltipPerspective(User user, String namespace, String title) throws DBException {
		if(user == null)
			user = getPermissionManager().getAnonymousUser();
		
		WikiResource r = getResourceByQTitle(namespace+":"+title);
		
		if(getPermissionManager().hasPermission(user,r,getViewAction())) {
			synchronized(tt_cache) {
				SoftReference<TooltipPerspective> ref = tt_cache.get(namespace+title);
				if(ref != null && ref.get() != null) {
					log.debug("cache hit for article view "+namespace+":"+title);
					return ref.get();
				} else {
					TooltipPerspective a = new TooltipPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));;
					tt_cache.put(namespace+title,new SoftReference<TooltipPerspective>(a));
					log.debug("cache miss for tooltip "+namespace+":"+title);
					return a; 
				}
			}
		} else {
			return null;
		}
	}
	
	public Action getViewAction() {
		return action_registry.getAction("view");
	}
	
	/**
	 * Retrieve article identified by namespace prefix and title from the database and 
	 * create an XMLExchangePerspective instance. This method always retrieves the latest revision
	 * of an article.
	 * <p>
	 * Corresponds to the parameter action=view
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new ViewArticlePerspective instance for the article
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public XMLExchangePerspective getXMLExchangePerspective(String namespace, String title) throws DBException {
		XMLExchangePerspective a = new XMLExchangePerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));;
		return a;
	}

	
	/**
	 * Retrieve a resource identified by namespace prefix and title from database and create a 
	 * context view for it.
	 * 
	 * @param namespace namespace prefix of the resource
	 * @param title title of the resource
	 * @return a new ViewContextPerspective instance for the resource
	 * @throws DBException in case the resource cannot be retrieved from the database
	 * @see at.srfg.ikewiki.layout.perspectives.ViewContextPerspective
	 */
	public ViewContextPerspective getViewContextPerspective(String namespace, String title) throws DBException {
		return new ViewContextPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
	}

	/**
	 * Return the revision history of an article identified by namespace prefix and title from the
	 * database and create a ViewHistoryPerspective instance.
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new ViewHistoryPerspective instance
	 * @throws DBException in case the article cannot be retrieved from database
	 * @see at.srfg.ikewiki.layout.perspectives.ViewHistoryPerspective
	 */
	public ViewHistoryPerspective getViewHistoryPerspective(String namespace, String title) throws DBException {
		return new ViewHistoryPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
	}
	
	/**
	 * Retrieve an article identified by namespace prefix and title from the database and return an EditContentPerspective instance
	 * suitable for editing the article in structured text (without rendering).
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new EditContentPerspective instance
	 * @throws DBException in case the article cannot be retrieved from the database
	 * @see at.srfg.ikewiki.layout.perspectives.EditContentPerspective
	 */
	public EditContentPerspective getEditContentPerspective(String namespace, String title) throws DBException {
		// check whether article already exists; if yes, return EditContentPerspective on article, otherwise create new empty article
		if(getDBStore().existsArticleByNamespaceTitle(namespace,title)) {		
			return new EditContentPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
		} else {
			try {
				Namespace ns = getDBStore().getNamespaceByPrefix(namespace);
				return new EditContentPerspective(this,new Article(ns,title,"initial version","replace text by content",Locale.getDefault().getLanguage(),ns.constructURI(title).toString(),new Date()));
			} catch(URISyntaxException ex) {
				log.error("the URI for empty article content is not correct; please change Main.java",ex);
				return null;
			}
		}
	}	

	/**
	 * Retrieve article identified by namespace prefix and title from the database and 
	 * create an ViewArticlePerspective instance. This method always retrieves the latest revision
	 * of an article.
	 * <p>
	 * Corresponds to the parameter action=view
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new ViewArticlePerspective instance for the article
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public EditMozilePerspective getEditMozilePerspective(String namespace, String title) throws DBException {
		EditMozilePerspective a = new EditMozilePerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));;
		return a; 
	}
	
	
	/**
	 * Retrieve an article identified by namespace prefix and title from the database and create an EditReferencesPerspective that allows to
	 * edit the references associated with the article.
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new EditReferencesPerspective instance
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public EditMetadataPerspective getEditMetadataPerspective(String namespace, String title) throws DBException {
		return new EditMetadataPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
	}

	/**
	 * Retrieve an article identified by namespace prefix and title from the database and create an EditReferencesPerspective that allows to
	 * edit the types associated with the article.
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new EditReferencesPerspective instance
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public EditTypesPerspective getEditTypesPerspective(String namespace, String title) throws DBException {
		return new EditTypesPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
	}


	/**
	 * Retrieve an article identified by namespace prefix and title from the database and create an EditReferencesPerspective that allows to
	 * edit the references associated with the article.
	 * 
	 * @param namespace namespace prefix of the article
	 * @param title title of the article
	 * @return a new EditReferencesPerspective instance
	 * @throws DBException in case the article cannot be retrieved from the database
	 */
	public EditReferencesPerspective getEditReferencesPerspective(String namespace, String title) throws DBException {
		return new EditReferencesPerspective(this,getStorageBackend().getArticleByNamespaceTitle(namespace,title));
	}

	/**
	 * Create a new RecentChangesPerspective; currently, this does not require database access.
	 * 
	 * @return a new NamespacePerspective instance
	 */
	public RecentChangesPerspective getRecentChangesPerspective() {
		return new RecentChangesPerspective(this);
	}

	/**
	 * Create a new NamespacePerspective; currently, this does not require database access.
	 * 
	 * @return a new NamespacePerspective instance
	 */
	public NamespacePerspective getNamespacePerspective() {
		return new NamespacePerspective(this);
	}

	/**
	 * Create a new RedirectMapPerspective; currently, this does not require database access.
	 * 
	 * @return a new RedirectPerspective instance
	 */
	public RedirectPerspective getRedirectPerspective(String url) {
		return new RedirectPerspective(this,url);
	}

	/**
	 * Create a new ErrorPerspective; currently, this does not require database access.
	 * 
	 * @return a new ErrorPerspective instance
	 */
	public ErrorPerspective getErrorPerspective(String message) {
		return new ErrorPerspective(this,message);
	}

	/**
	 * Create a new RelatedPerspective
	 * 
	 * @return a new RedirectPerspective instance
	 */
	public RelatedPerspective getRelatedPerspective(WikiResource context) throws DBException {
		return new RelatedPerspective(this,context);
	}

	/**
	 * Create a new RelatedPerspective
	 * 
	 * @return a new RedirectPerspective instance
	 */
	public RelatedPerspective getRelatedPerspective(String ns_prefix, String local_title) throws DBException {
		return getRelatedPerspective(this.getStorageBackend().getArticleByNamespaceTitle(ns_prefix,local_title));
	}

	/**
	 * Create a new SiteMapPerspective; currently, this does not require database access.
	 * 
	 * @return a new SiteMapPerspective instance
	 */
	public SiteMapPerspective getSiteMapPerspective() {
		return new SiteMapPerspective(this);
	}

	
	/**
	 * Create a new SparqlPerspective; currently, this does not require database access.
	 * 
	 * @return a new SparqlPerspective instance
	 */
	public SparqlPerspective getSparqlPerspective(Document d) {
		return new SparqlPerspective(this,d);
	}
	
	
	
	/**
	 * Create a new VariablesPerspective; currently, this does not require database access.
	 * 
	 * @return a new VariablesPerspective instance
	 */
	public VariablesPerspective getVariablesPerspective() {
		return new VariablesPerspective(this);
	}

	/**
	 * Create a new NewResourcePerspective; currently, this does not require database access.
	 * 
	 * @return a new NewResourcePerspective instance
	 */
	public NewResourcePerspective getNewResourcePerspective() {
		return new NewResourcePerspective(this);
	}

	/**
	 * Create a new NewResourcePerspective; currently, this does not require database access.
	 * 
	 * @return a new NewResourcePerspective instance
	 */
	public NewClassPerspective getNewClassPerspective() {
		return new NewClassPerspective(this);
	}

	/**
	 * Create a new NewResourcePerspective; currently, this does not require database access.
	 * 
	 * @return a new NewResourcePerspective instance
	 */
	public NewPropertyPerspective getNewPropertyPerspective() {
		return new NewPropertyPerspective(this);
	}

	
	/**
	 * Create a new LoginPerspective; currently, this does not require database access.
	 * 
	 * @return a new NamespacePerspective instance
	 */
	public LoginPerspective getLoginPerspective() {
		return new LoginPerspective(this);
	}

	/**
	 * Create a new SearchPerspective; currently, this does not require database access.
	 * 
	 * @return a new NamespacePerspective instance
	 */
	public SearchPerspective getSearchPerspective(String search) {
		return new SearchPerspective(this,search);
	}

	
	/**
	 * Restore a given revision of an article so that it is the current version. The current implementation simply
	 * retrieves the revision and calls StorageBackend.storeArticle() with this revision.
	 * 
	 * @param namespace the namespace prefix of the article
	 * @param title title of the article
	 * @param revision revision number to restore
	 * @throws DBException in case the article cannot be retrieved or stored
	 */
	public void restoreRevision(String namespace, String title, long revision, User u) throws DBException {
		// we simply retrieve the old revision and store it again; in this way, the timestamp gets
		// updated and everything is fine.:-)
		
		Article a = getStorageBackend().getArticleByRevisionId(namespace,title,revision);
		if(u != null)
			a.setCreator(u.getLogin());
		getStorageBackend().storeArticle(a);
		
		log.info("restored revision "+revision+" of resource with title "+a.getQualifiedTitle());
	}
	
	/**
	 * Parse all request data into a Hashtable.
	 * <p>
	 * This method parses both GET and POST data and is aware of multipart/form-data style POSTs
	 *
	 * @param request
	 * @return
	 * @throws FileUploadException
	 */
	public RequestData parseRequestData(HttpServletRequest request) throws FileUploadException {
		
		RequestData postdata = new RequestData();
		
		
		try {
			if(request.getCharacterEncoding() == null) {
				request.setCharacterEncoding("UTF-8");
			}
		} catch(UnsupportedEncodingException ex) {}
		
		
		
		if(request.getContentLength() > 0) {
			try {
				//	 Create a new file upload handler
				DiskFileUpload upload = new DiskFileUpload();
				upload.setHeaderEncoding("UTF-8");
				
				//	 Parse the request
				List<FileItem> items = upload.parseRequest(request);
				
				for(FileItem fi : items) {
					if(fi.isFormField()) {
						try{
							postdata.put(fi.getFieldName(),fi.getString("UTF-8"));
						} catch(UnsupportedEncodingException ex) {}
					} else {
						try{					
							postdata.put(fi.getFieldName(),fi.getString("UTF-8"));						
						} catch(UnsupportedEncodingException ex) {}
						postdata.putBytes(fi.getFieldName(),fi.get());					
					}
				}
			} catch(Exception ex) {
				// happens when posted content is not multipart/form-data; in this case we
				// create a key "rawdata" and put all the bytes in it
				// read the raw data
				String result = "";
				try {
					ServletInputStream poststrm = request.getInputStream();
					
					// get data from inputstream			
					int count;
					byte[] s = new byte[100];
					try {
						while ((count = poststrm.readLine(s, 0, 100)) > 0) {
							byte[] s2 = new byte[count];
							int k;
							for (k = 0; k < count; k++) {
								s2[k] = s[k];
							}
							result = result + new String(s2);
						}
					} catch (java.io.IOException e) {
						log.error("Exception during processing of raw POST data: ",e);
						
					}
				} catch (java.io.IOException e) {
					log.error("Exception during attempt to read http input stream: ",e);
				}
				
				postdata.putBytes("rawdata",result.getBytes());
				
				// change result from UTF-8 to unicode (java native)
				try {
					result = new String(result.getBytes(), "UTF-8");
					//System.err.println(result);
				} catch(UnsupportedEncodingException e) {
					log.error(e);
				}
				postdata.put("rawdata",result);
			}
		}
		
		Enumeration pnames = request.getParameterNames();
		while(pnames.hasMoreElements()) {
			// servlets unfortunately assume ISO-8859-1 as default for GET parameters, recode everything to UTF-8
			try {
				String name = (String)pnames.nextElement();
				String value = new String(request.getParameter(name).getBytes("iso-8859-1"),"UTF-8");
				postdata.put(name,value);
			} catch(UnsupportedEncodingException ex) {}
		}

		
		if(postdata.get("uri") != null) {
			try {
				String uri = postdata.get("uri");
				WikiResource r = getResourceByUri(uri);
				postdata.put("title",r.getQualifiedTitle());
				log.debug("URI mapped to title: "+r.getQualifiedTitle());
			} catch(DBException ex) {
				log.error(ex);
			}
		}
		
		
		return postdata;
	}
	
	/**
	 * Construct an article from post data. This method might be moved to EditContentPerspective.
	 * 
	 * @param ns_prefix
	 * @param title
	 * @param postdata
	 * @return
	 * @throws URISyntaxException
	 * @throws DBException
	 */
	public Article constructArticle(String ns_prefix, String title, Hashtable<String,String> postdata) throws URISyntaxException, DBException {
		
		
		String summary = postdata.get("wpSummary");
		if(summary == null)
			summary = "";
		
		String content = postdata.get("wpTextbox1");
		if(content == null)
			content = "";
		
		String language = postdata.get("wpLanguage");
		if(language == null)
			language = Locale.getDefault().getLanguage();
		
		
		Namespace ns = database.getNamespaceByPrefix(ns_prefix);

		String uri = postdata.get("wpURI");
		if(uri == null) {
			uri = ns.getUri()+title;
		}
		
//		for(Object key : postdata.keySet()) {
//			System.err.println("Key: "+key + ", value: "+postdata.get(key));
//		}
		
		
		return new Article(ns,title,summary,content,language,uri,new Date());
	}
		
	
	/**
	 * Create a resource based on a qualified title, i.e. composed of a namespace prefix, followed by : and the title.
	 * @param qtitle
	 * @return
	 * @throws DBException
	 */
	public WikiResource getResourceByQTitle(String qtitle) throws DBException {
		SoftReference<WikiResource> ref = resource_qtitle_cache.get(qtitle);
		
		if(ref != null && ref.get() != null) {
			return ref.get();
		} else {
		
			String article_ns, article_title;
			String[] components = qtitle.replace(" ","_").split(":");
			if(components.length == 2) {
				article_ns    = components[0];
				article_title = components[1];				
			} else if(components.length == 1){
				article_ns    = "";
				article_title = components[0];
			} else {
				article_ns    = "";
				article_title = "";
			}
			Namespace ns;
			
			try {
				ns = database.getNamespaceByPrefix(article_ns);
			} catch(NamespaceNotFoundException ex) {
				ns = new Namespace(article_ns,"");
				log.warn("namespace "+article_ns+" not found in database (for resource "+article_title+")");
			}
			
			WikiResource r = new WikiResource(ns,article_title);
			
			resource_qtitle_cache.put(qtitle,new SoftReference<WikiResource>(r));
			
			return r;
		}
	}

	/**
	 * Create a resource based on URI.
	 * 
	 * @param uri
	 * @return
	 * @throws DBException
	 */
	public WikiResource getResourceByUri(String uri) throws DBException {

		SoftReference<WikiResource> ref = resource_uri_cache.get(uri);
		
		if(ref != null && ref.get() != null) {
			return ref.get();
		} else {
					
			try {

				WikiResource r = null;
								
				// first, try to resolve the URI by means of the RDF model;
				// if this does not succeed, try to parse the URI into a namespace and a local part,
				// ensure the existance of the namespace, and use it
				String qname = getRDFStore().getModel().shortForm(uri);
								
				if(!qname.equals(uri)) {				
					r = getResourceByQTitle(qname);
				} else if(!uri.contains("/")) {
					// hack: relative URL
					r = new WikiResource(getStorageBackend().getNamespaceByPrefix(""),uri);									
				} else {
					log.warn("need to create a namespace for external URI "+uri);
					
					// this is basically a hack: we take the last component of the URI that does not contain a "/"
					// or "#" as local part, and the remaining prefix as namespace URI
					// a better way would be to use the stored database information about namespaces to gather
					// this information.
					String article_ns, article_title;
					
					Pattern p = Pattern.compile("^(.*)(/|#)([^/#]+?)/?$");
					//Pattern p = Pattern.compile("^(.*)(/|#)([^/#]+?)$");
					
					Matcher m = p.matcher(uri);
					
					if(m.matches()) {
						article_ns = m.group(1);
						article_title = m.group(3);
					} else {
						article_ns    = getNamespaceBase().getUri();
						article_title = uri;
					}
					
					try {
						Namespace ns = database.getNamespaceByUri(article_ns);
						r = new WikiResource(ns,article_title);
						
					} catch(NamespaceNotFoundException ex) {
						// create a "dummy" namespace with the URI and ensure its existance in the database
						// ensureNamespace will return a new prefix that we may use to construct the resource
						Namespace ns = new Namespace("EXTERN",article_ns);
						/*
						 String nprefix = getStorageBackend().ensureNamespace(ns);
						 return new WikiResource(new Namespace(nprefix,article_ns),article_title);
						 */
						r = new WikiResource(ns,article_title);
					}
					
				}
				
				resource_uri_cache.put(uri,new SoftReference<WikiResource>(r));
				
				return r;
			} catch(DBException ex) {
				log.error("could not create resource for URI "+uri+" (database error)");
				throw ex;
			} catch(Exception ex) {
				log.error("could not create resource for URI "+uri,ex);
				return null;
			}
		}		
	}

	
	/**
	 * Create a WikiLink based on qualified titles for subject, predicate and object.
	 * @param subject_qtitle
	 * @param predicate_qtitle
	 * @param object_qtitle
	 * @return
	 * @throws DBException
	 */
	public WikiLink getWikiLink(String subject_qtitle, String predicate_qtitle, String object_qtitle) throws DBException {
		WikiResource s = getResourceByQTitle(subject_qtitle);
		WikiResource p = getResourceByQTitle(predicate_qtitle);
		WikiResource o = getResourceByQTitle(object_qtitle);
		return new WikiLink(s,p,o,"");
	}
	
	@Deprecated
	public User authenticateUser(String login, String password) {
		return getPermissionManager().authenticateUser(login,password);
	}
		
	@Deprecated
	public User createUser(String login, String firstName, String lastName, String email, String password) throws DBException {
		return getPermissionManager().createUser(login,firstName,lastName,email,password);
	}
	
	@Deprecated
	public void saveUser(User u) throws DBException {
		getPermissionManager().saveUser(u);		
	}
	
	
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.db.StorageBackend#getNamespaceByPrefix(java.lang.String)
	 */
	public Namespace getNamespaceByPrefix(String ns_prefix) throws DBException {
		return database.getNamespaceByPrefix(ns_prefix);
	}



	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.db.StorageBackend#getNamespaceByUri(java.lang.String)
	 */
	public Namespace getNamespaceByUri(String uri) throws DBException {
		return database.getNamespaceByUri(uri);
	}


	public Namespace getNamespaceBase() throws DBException {
		return getStorageBackend().getNamespaceByPrefix("");
	}

	public Namespace getNamespaceSpecial() throws DBException {
		return getStorageBackend().getNamespaceByUri(Namespace.NS_IKEWIKI_SPECIAL);
	}
	
	/**
	 * IkeWiki base namespace; used for internal resources
	 * @return
	 * @throws DBException
	 */
	public Namespace getNamespaceUser() throws DBException {
		return getStorageBackend().getNamespaceByUri(Namespace.NS_IKEWIKI_USER);
	}
	
	
	public String getSkin() {
		return "monobook";
	}
	
	public String getTagLine() {
		return "From IkeWiki, the semantically enriched Wiki";
	}
	
	public String getLogoPath() {
		return "skins/common/images/ikewiki_sr.png";
	}
	
	public Iterable<Portlet> getPortlets() {
		return portlets;
	}

	
	public synchronized StorageBackend getStorageBackend() throws DBException {
		if(database == null) {
			database = new StorageBackend(this);
		}
		return database;		
	}
	
	/**
	 * @return Returns the database.
	 */
	protected DBStore getDBStore() throws DBException {
		return getStorageBackend().getDBStore();
	}
	
	
	protected RDFStore getRDFStore() throws DBException {
		return getStorageBackend().getRDFStore();
	}
	
	
	public Article populateDB() throws Exception {
		Article a = new Article(getNamespaceBase(),"FrontPage","populate DB","== Welcome to IkeWiki ==\n\nThis is a sample article. You may edit its contents by clicking on 'edit content'. You may also refine links to and from this page by clicking on 'edit links'.\n\n","en","http://ikewiki.srfg.at/SampleArticle",new Date());
		getStorageBackend().storeArticle(a);
		return a;
	}
	
	
	/**
	 * Return a list of all available actions
	 * @return
	 * @throws DBException
	 */
	public Iterable<Action> getAllActions() {
		Collection<Action> all_actions = new LinkedList<Action>();
		
		for(String key : actions) {
			all_actions.add(action_registry.getAction(key));
		}
		
		return all_actions;
	}
	
	/**
	 * Return a list of all actions a user has access to
	 * 
	 * @param user
	 * @return
	 * @throws DBException
	 */
	public Iterable<Action> getActions(User user,WikiResource context) throws DBException {
		if(user == null)
			user = getPermissionManager().getAnonymousUser();

		Collection<Action> allowed_actions = new LinkedList<Action>();


		for(String key : actions) {
			Action a = action_registry.getAction(key);
			if(getPermissionManager().hasPermission(user,context,a)) {
				allowed_actions.add(a);
			}
		}
		return allowed_actions;
	}
	

	public Iterable<Action> getContextActions(Perspective p, User user) throws DBException {
		if(p.hasContext()) {
			return getContextActions(p,user,p.getContext());			
		} else {
			return getContextActions(p,user,getDefaultContext());
		}
	}

	/**
	 * Return a list of all actions a user has access to, relevant in the current context
	 * 
	 * @param p
	 * @param user
	 * @return
	 * @throws DBException
	 */
	public Iterable<Action> getContextActions(Perspective p, User user, WikiResource context) throws DBException {
		if(user == null)
			user = getPermissionManager().getAnonymousUser();

		if(p == null) {
			return getActions(user,context);
		} else {
			Collection<Action> allowed_actions = new LinkedList<Action>();
			for(Action a : p.getActions()) {
				if(getPermissionManager().hasPermission(user,context,a)) {
					allowed_actions.add(a);
				}
			}
			return allowed_actions;
		}
	}
	
	public void updateNamespace(String prefix, String uri) throws DBException {
		getStorageBackend().updateNamespace(new Namespace(prefix,uri));
	}
	
	public void deleteNamespace(String uri) throws DBException {
		getStorageBackend().deleteNamespace(uri);
	}
	
	public void addNamespace(String prefix, String uri) throws DBException {
		getStorageBackend().ensureNamespace(new Namespace(prefix,uri));
	}
	
	public boolean isSpecial(String prefix, String title) throws DBException {
		Namespace n = getStorageBackend().getNamespaceByPrefix(prefix);
		return n.getUri().equals("http://ikewiki.srfg.at/Special#");
	}
	
	public String getSpecialAction(String title) {
		return special_to_actions.get(title);
	}
	
	public static String encodeURL(String name) {
		try {
			//return URLEncoder.encode(name,"ISO-8859-1");
			return URLEncoder.encode(name,"UTF-8");
		} catch(UnsupportedEncodingException ex) {
			log.error(ex);
			return name;
		}
	}
	
	@Deprecated
	public static String escapeURL(String name) {
		return encodeURL(name);
	}
	
	public static String decodeURL(String name) {
		try {
			return URLDecoder.decode(name,"UTF-8");
		} catch(UnsupportedEncodingException ex) {
			log.error(ex);
			return name;
		}
	}


	public void invalidateCaches() {
		synchronized(av_cache) {
			av_cache.clear();
		}
	}
	
	public ViewItemFactory getViewItemFactory(){
		if(this.viewItemFactory == null){
			Class ifClass;
			Object ifObject;
			try {
				ifClass = Main.class.forName(this.viewItemFactoryType);
			} catch (ClassNotFoundException e) {
				log.error("Unable to load ItemFactory via Classpath(Class="+this.viewItemFactoryType+")",e);
				return null;
			}
			try {
				ifObject = ifClass.newInstance();
			} catch (InstantiationException e) {
				log.error("Unable to create ItemFactory",e);
				return null;
			} catch (IllegalAccessException e) {
				log.error("Unable to create ItemFactory",e);
				return null;
			}
			if(ifObject instanceof ViewItemFactory){
				this.viewItemFactory = (ViewItemFactory)ifObject;
			} else {
				log.error("Class="+this.viewItemFactoryType+" is not compatible to Interface ItemFactory");
				return null;
			}
		}
		return this.viewItemFactory;
	}

	public void changeEvent(WikiResource r) {
		String av_key = r.getNamespace().getPrefix()+r.getTitle();
		synchronized(av_cache) {
			av_cache.remove(av_key);
		}
		synchronized(tt_cache) {
			tt_cache.remove(av_key);
		}
	}

	public void allChangeEvent() {
		synchronized(av_cache) {
			av_cache.clear();
		}
		// TODO: should be more fine-grained, tooltip cache does not need to be cleared when
		// link is added, ...
		synchronized(tt_cache) {
			tt_cache.clear();
		}
	}

	public RenderingPipeline getRenderingPipeline() {
		return renderer;
	}
	
	/**
	 * Retrieve a list of registered views for the context of a WikiResource.
	 * @param r
	 * @return
	 */
	public Collection<View> getViewsForResource(WikiResource r, View.TYPES type) {
		return view_registry.getViewsForResource(r,type);
	}
	
	/**
	 * Encode a string such that it can be displayed in XML in a suitable manner. Currently, this means 
	 * replacing all occurrences of "<" and "&" by their respective entities.
	 * 
	 * @param input
	 * @return
	 */
	public static String encodeXML(CharSequence input) {
		StringBuilder buf = new StringBuilder();
		for(int i = 0; i < input.length(); i++) {
			switch(input.charAt(i)) {
			case '<': buf.append("&lt;"); break;
			case '&': buf.append("&amp;"); break;
			default:  buf.append(input.charAt(i)); break;
			}
		}
		return buf.toString();
	}

	@Override
	protected void finalize() throws Throwable {
		close();
		super.finalize();
	}
	
	public void close() {
		database.close();		
	}

	public String getVersion() {
		return "@version@";
	}

	public Collection<Template> getTemplates() {
		return templates.values();
	}
	
	public Template getTemplate(String key) {
		return templates.get(key);
	}



	public ActionRegistry getActionRegistry() {
		return action_registry;
	}

	public Action getAction(String id) {
		return action_registry.getAction(id);
	}

	
	public PermissionManager getPermissionManager() {
		return permission_manager;
	}
	
	
	public WikiResource getDefaultContext() throws DBException {
		return getResourceByQTitle("FrontPage");
	}
	
	public String getBaseUri() {
		String base_uri = getConfig().getString("base_uri","http://localhost:8080/IkeWiki/index.jsp");
		return base_uri;
	}
}
