/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;

/**
 * The RDFDownload Servlet treats download of the repository in different serialisation formats. Currently,
 * RDF/XML, N3, N-TRIPLE and TURTLE are supported.
 * 
 * @author sschaffe
 *
 */
public class RDFDownload extends HttpServlet {

	private static Log log = LogFactory.getLog(RDFDownload.class);
	
	private Main main = null;
	
	
	/**
	 * 
	 */
	public RDFDownload() {
		super();
		// TODO Auto-generated constructor stub
	}
	
	
	
	
	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		super.init();
		try {
			main = Main.getInstance();
		} catch(DBException ex) {
			throw new ServletException("error initialising Main class",ex);
		}
	}




	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			RequestData parameters = main.parseRequestData(request);
		
			String format = parameters.get("format");

			if(format == null || format.equals("RDF/XML")) {
				byte[] content = main.getRDFStore().getRDFXML(); 

				response.setContentType("text/xml");
				response.setHeader("Content-Disposition","attachment; filename=ikewiki.rdf");
				response.setContentLength(content.length);
				response.getOutputStream().write(content);
				response.getOutputStream().flush();
			} else if(format.equals("N3")) {
				byte[] content = main.getRDFStore().getN3(); 

				response.setContentType("text/plain");
				response.setHeader("Content-Disposition","attachment; filename=ikewiki.n3");
				response.setContentLength(content.length);
				response.getOutputStream().write(content);
				response.getOutputStream().flush();				
			} else if(format.equals("TURTLE")) {
				byte[] content = main.getRDFStore().getTurtle(); 

				response.setContentType("text/plain");
				response.setHeader("Content-Disposition","attachment; filename=ikewiki.turtle");
				response.setContentLength(content.length);
				response.getOutputStream().write(content);
				response.getOutputStream().flush();				
			} else if(format.equals("N-TRIPLE")) {
				byte[] content = main.getRDFStore().getTriples(); 

				response.setContentType("text/plain");
				response.setHeader("Content-Disposition","attachment; filename=ikewiki.triple");
				response.setContentLength(content.length);
				response.getOutputStream().write(content);
				response.getOutputStream().flush();				
			}
			
		} catch(DBException ex) {
			log.error("error while connecing with database",ex);
			throw new ServletException("Error retrieving resource",ex);		
		} catch(FileUploadException ex) {
			log.error("error while parsing parameters",ex);
			throw new ServletException("Error parsing parameters",ex);
		}
	}


}
