/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;
import at.srfg.ikewiki.permission.PermissionObject;

/**
 * Actions are commands that can be performed by the index.jsp servlet. They are identified by the
 * "action" parameter of index.jsp and return a perspective when they are called. The ActionHandler class
 * manages all Action classes.  
 * 
 * @author sschaffe
 *
 */
public interface Action extends PermissionObject {

	/**
	 * Get the identifier of this action (e.g. "view", "edit_content").
	 *
	 */
	public String getIdentifier();
	
	/**
	 * Return the label to be displayed in the page.
	 * 
	 * @return
	 */
	public String getLabel();
	
	/**
	 * Return the minimum list of parameters required for this action to function.
	 * 
	 * @return
	 */
	public Collection<String> getRequiredParameters();
	
	
	/**
	 * Return a list of all parameters acceptable by this action.
	 * 
	 *  
	 */
	public Collection<String> getAcceptableParameters();
	
	/**
	 * Perform the action and return the associated new perspective.
	 * 
	 * @param parameters
	 * @return
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException;
	
}
