/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Collection;
import java.util.HashSet;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.permission.PermissionObject;

public abstract class BaseAction implements Action, PermissionObject {

	protected Main main;
	
	
	public BaseAction(Main main) {
		super();
		this.main = main;
	}


	
	protected String getArticleNS(RequestData parameters) {
		String req_title = parameters.get("title");
	    
		String article_ns;
		if(req_title == null) {
	 		article_ns    = "";
	 	} else {
	 		String[] components = req_title.replace(" ","_").split(":");
	 		if(components.length == 2) {
		 		article_ns    = components[0];
		 	} else {
		 		article_ns    = "";
		 	}
	 	}		
		return article_ns;
	}
	
	
	protected String getArticleTitle(RequestData parameters) {
		String req_title = parameters.get("title");
	    
		String article_title;
		if(req_title == null) {
	 		article_title = "FrontPage";
	 	} else {
	 		String[] components = req_title.replace(" ","_").split(":");
	 		if(components.length == 2) {
		 		article_title = components[1];
		 	} else if(components.length == 1) {
		 		article_title = components[0];
		 	} else {
		 		article_title = "";
		 	}
	 	}		
		return article_title;		
	}


	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.permission.PermissionObject#getObjectIdentifier()
	 */
	public String getObjectIdentifier() {
		return Namespace.NS_IKEWIKI_ACTION+this.getClass().getSimpleName();
	}


	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.permission.PermissionObject#getRelevantPermissions()
	 */
	public Collection<String> getRelevantPermissions() {
		if(relevantPermissions == null) {
			relevantPermissions = new HashSet<String>();
			relevantPermissions.add("execute");
		}
		return relevantPermissions;
	}
	
	private static Collection<String> relevantPermissions = null;
}
