/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author wastl
 *
 */
public class DeleteAction extends BaseAction {

	private static final String[] required_parameters = { "sure", "title" };

	/**
	 * @param main
	 */
	public DeleteAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "delete";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getLabel()
	 */
	public String getLabel() {
		return "Delete";
	}
	
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData, javax.servlet.http.HttpServletResponse, javax.servlet.http.HttpSession)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		// check whether all parameters are set
		for(String s : required_parameters) {
			if(parameters.get(s) == null) {
				return main.getErrorPerspective("Required parameter \""+s+"\" not specified.");
			}
		}

		if(parameters.get("sure").equals("yes")) {
			String title = parameters.get("title");
			
			WikiResource r = main.getResourceByQTitle(title);
			main.getStorageBackend().deleteArticle(r);
			
		}
		
		
		return main.getRedirectPerspective("index.jsp?title=FrontPage&action=view");
	}

}
