/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.AbstractCollection;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.EditMetadataPerspective;
import at.srfg.ikewiki.layout.perspectives.Perspective;
import at.srfg.ikewiki.layout.perspectives.ViewArticlePerspective;

/**
 * @author sschaffe
 *
 */
public class EditMetadataAction extends BaseAction {

	private static final String[] required_parameters = { "title" };
	private static final String[] acceptable_parameters = { "title", "subaction" };

	/**
	 * @param main
	 */
	public EditMetadataAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "edit_metadata";
	}

	
	
	public String getLabel() {
		return "Edit Metadata";
	}

	
	/* (non-Javadoc)
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		// accepts all parameters!
		return new AbstractCollection<String>() {

			@Override
			public Iterator<String> iterator() {
				return Arrays.asList(acceptable_parameters).iterator();
			}

			@Override
			public int size() {
				return acceptable_parameters.length;
			}

			// contains every parameter
			@Override
			public boolean contains(Object o) {
				return true;
			}
		};
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
				
		String article_ns    = getArticleNS(parameters);
		String article_title = getArticleTitle(parameters); 
		
		EditMetadataPerspective perspective=main.getEditMetadataPerspective(article_ns,article_title);

		// currently available subactions:
		// - store, paramaters: title (all are qualified resource titles of the form ns_prefix:title)
		
		if(parameters.get("subaction") != null && parameters.get("subaction").equals("store")) {
			
			for(String key : parameters.keySet()) {
				if(key.startsWith("metadata-") && parameters.get(key) != null && !parameters.get(key).trim().equals("")) {
					String field = key.substring(9);
					((ViewArticlePerspective)perspective).setMetadataFieldValue(field,parameters.get(key));
				}
			}
						
		}

		return perspective;
	}

}
