/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class EditReferencesAction extends BaseAction {

	private static final String[] required_parameters = { "title" };
	private static final String[] acceptable_parameters = { "title", "subaction", "subject", "predicate", "object", "target" };

	/**
	 * @param main
	 */
	public EditReferencesAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "edit_references";
	}
	
	

	public String getLabel() {
		return "Edit Links";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		// edit the references related to the current article

		String article_ns    = getArticleNS(parameters);
		String article_title = getArticleTitle(parameters); 

		
		// currently available subactions:
		// - delete, parameters: title, subject, predicate, object (all are qualified resource titles of the form ns_prefix:title)
		// - store, paramaters: title, subject, predicate, object (all are qualified resource titles of the form ns_prefix:title)
		// - externalise, parameters: title, object (all are qualified resource titles of the form ns_prefix:title)
		
		if(parameters.get("subaction") != null && parameters.get("subaction").equals("delete")) {
		    // link edit action is "delete external link"; if subject, predicate, and object are given, try to
		    // remove the triple from the RDF store
			String delete_s = parameters.get("subject");
			String delete_p = parameters.get("predicate");
			String delete_o = parameters.get("object");
			
			if(delete_s != null && delete_p != null && delete_o != null) {
			  main.getStorageBackend().removeExternalWikiLink(main.getWikiLink(delete_s,delete_p,delete_o));
			}
		} else if(parameters.get("subaction") != null && parameters.get("subaction").equals("store")) {
		    // link edit action is "store new external link"; if subject, predicate, and object are given, store
		    // the triple in the RDF store
			String subject    = parameters.get("subject");
			String predicate  = parameters.get("predicate");
			String object     = parameters.get("object");
			
			if(subject != null && predicate != null && object != null) {
				main.getStorageBackend().addExternalWikiLink(main.getWikiLink(subject,predicate,object));
			}
		} else if(parameters.get("subaction") != null && parameters.get("subaction").equals("add_outgoing_link") &&
				  parameters.get("target") != null) {
			String subject    = parameters.get("title");
			String predicate  = main.getStorageBackend().getNamespaceByUri(Namespace.NS_IW).getPrefix()+":NavigationalLink";
			String object     = parameters.get("target");
			
			if(subject != null && predicate != null && object != null) {
				main.getStorageBackend().addExternalWikiLink(main.getWikiLink(subject,predicate,object));
			}
		} else if(parameters.get("subaction") != null && parameters.get("subaction").equals("add_incoming_link") &&
				  parameters.get("target") != null) {
			String subject    = parameters.get("target");
			String predicate  = main.getStorageBackend().getNamespaceByUri(Namespace.NS_IW).getPrefix()+":NavigationalLink";
			String object     = parameters.get("title");
			
			if(subject != null && predicate != null && object != null) {
				main.getStorageBackend().addExternalWikiLink(main.getWikiLink(subject,predicate,object));
			}
		}

		// it is important to create the Renderable *after* the subactions have been processed, because the database
		// will have changed:		
		return main.getEditReferencesPerspective(article_ns,article_title);
	}

}
