/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.EmptyPerspective;
import at.srfg.ikewiki.layout.perspectives.Perspective;

public class EditSuperPropertiesAction extends BaseAction implements Action {

	private static final String[] required_parameters = { "title", "subaction", "superproperty" };
	private static final String[] acceptable_parameters = { "title", "subaction", "superproperty" };

	public EditSuperPropertiesAction(Main main) {
		super(main);
	}

	public String getIdentifier() {
		return "edit_superproperties";
	}

	public String getLabel() {
		return "Edit Super-Properties";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	public Perspective handleAction(RequestData parameters,	HttpServletResponse response, HttpSession session) throws DBException {
		String title  = parameters.get("title");
		if(parameters.get("subaction") != null && parameters.get("subaction").equals("delete")) {
		    // link edit action is "delete external link"; if subject, predicate, and object are given, try to
		    // remove the triple from the RDF store
			String superproperty     = parameters.get("superproperty");
			
			if(superproperty != null) {
			    WikiResource o_resource = main.getResourceByQTitle(superproperty);
			    WikiResource context    = main.getResourceByQTitle(title);
				main.getStorageBackend().removeSuperProperty(context,o_resource);
			}
		} else if(parameters.get("subaction") != null && parameters.get("subaction").equals("store")) {
		    // link edit action is "store new external link"; if subject, predicate, and object are given, store
		    // the triple in the RDF store
			String superproperty     = parameters.get("superproperty");
			
			if(superproperty != null) {
			    WikiResource o_resource = main.getResourceByQTitle(superproperty);
			    WikiResource context    = main.getResourceByQTitle(title);
				main.getStorageBackend().addSuperProperty(context,o_resource);
			}
		}
		return new EmptyPerspective(main);
	}

}
