/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class EditTypesAction extends BaseAction {

	private static final String[] required_parameters = { "title" };
	private static final String[] acceptable_parameters = { "title", "subaction", "type" };

	/**
	 * @param main
	 */
	public EditTypesAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "edit_types";
	}

	
	
	public String getLabel() {
		return "Edit Types";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {

		String article_ns    = getArticleNS(parameters);
		String article_title = getArticleTitle(parameters); 
				
		// currently available subactions:
		// - delete, parameters: title, type (all are qualified resource titles of the form ns_prefix:title)
		// - store, paramaters: title, type (all are qualified resource titles of the form ns_prefix:title)
		
		if(parameters.get("subaction") != null && parameters.get("subaction").equals("delete")) {
		    // link edit action is "delete external link"; if subject, predicate, and object are given, try to
		    // remove the triple from the RDF store
			String typename     = parameters.get("type");
			
			if(typename != null) {
			    WikiResource o_resource = main.getResourceByQTitle(typename);
			    WikiResource context    = main.getResourceByQTitle(parameters.get("title"));
				main.getStorageBackend().removeType(context,o_resource);
			}
		} else if(parameters.get("subaction") != null && parameters.get("subaction").equals("store")) {
		    // link edit action is "store new external link"; if subject, predicate, and object are given, store
		    // the triple in the RDF store
			String typename     = parameters.get("type");
			
			if(typename != null) {
			    WikiResource o_resource = main.getResourceByQTitle(typename);
			    WikiResource context    = main.getResourceByQTitle(parameters.get("title"));
				main.getStorageBackend().storeType(context,o_resource);
			}
		}
		
		return main.getEditTypesPerspective(article_ns,article_title);
	}

}
