/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.User;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class HistoryAction extends BaseAction {

	private static final String[] required_parameters = { "title" };
	private static final String[] acceptable_parameters = { "title", "subaction", "restore", "revision" };

	/**
	 * @param main
	 */
	public HistoryAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "history";
	}

	public String getLabel() {
		return "History";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		// show the history for the current article, treat subactions if necessary

		String article_ns    = getArticleNS(parameters);
		String article_title = getArticleTitle(parameters); 

		User user = (User)session.getAttribute("user");
		
	    // currently available subactions:
	    // - restore, parameters: title (qualified resource title), revision (id of revision to restore)
	
		
		if(parameters.get("subaction") != null) {
			String subaction = parameters.get("subaction");
			
			if(subaction.equals("restore")) {
				if(parameters.get("revision") != null) {
					long revision = Long.parseLong(parameters.get("revision"));
					main.restoreRevision(article_ns,article_title,revision,user);
				}
			}
		}
		return main.getViewHistoryPerspective(article_ns,article_title); 
	}

}
