/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.SessionHistory;
import at.srfg.ikewiki.data.User;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.LoginPerspective;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class LoginAction extends BaseAction implements Action {

	private static final String[] required_parameters = { };
	private static final String[] acceptable_parameters = 
		{ "subaction", "wpLoginattempt", "wpName", "wpPassword", "wpRetype", 
		  "wpFirstName", "wpLastName", "wpEmail", "wpCreateaccount", "wpMailmypassword", "wpRemember"  };

	private static Log log = LogFactory.getLog(LoginAction.class);

	/**
	 * @param main
	 */
	public LoginAction(Main main) {
		super(main);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "login";
	}

	public String getLabel() {
		return "Login";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	
	public void setCookies(RequestData parameters, HttpServletResponse response) {
		// if the user wants to remember the password, set a cookie
		if(parameters.get("wpPassword") != null && parameters.get("wpName") != null && 
		   parameters.get("wpRemember") != null && parameters.get("wpRemember").equals("1")) {
			Cookie c_passwd = new Cookie("IkeWikiPassword",parameters.get("wpPassword"));
			Cookie c_user   = new Cookie("IkeWikiUser",parameters.get("wpName"));
			c_passwd.setMaxAge(60*60*24*365);
			c_user.setMaxAge(60*60*24*365);			
			response.addCookie(c_passwd);
			response.addCookie(c_user);
			log.debug("set user cookie: "+c_user);
			log.debug("set password cookie: "+c_passwd);
		}		
	}
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData, javax.servlet.http.HttpServletResponse, javax.servlet.http.HttpSession)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		// load article in any case, we need it for title etc
		LoginPerspective perspective=main.getLoginPerspective();
		

		// handle actual action
		User user;
		
		if(parameters.get("wpLoginattempt") != null && parameters.get("wpName") != null && parameters.get("wpPassword") != null) {
			// attempted login
			user = main.getPermissionManager().authenticateUser(parameters.get("wpName"),parameters.get("wpPassword"));
			if(user == null) {
				// authentication failed, show error message
				perspective.setAuthenticated(false);
				perspective.setErrorMessage("authentication failed: wrong login or password");
			} else {
				perspective.setAuthenticated(true);

				String wpName      = parameters.get("wpName");
				// authentication succeeded, store user in session
				session.setAttribute("user",user);
				
				setCookies(parameters,response);
				
				SessionHistory history = (SessionHistory)session.getAttribute("history");
				if(history != null && history.size() > 0) {
					return main.getRedirectPerspective("index.jsp?action=view&title="+Main.encodeURL(history.first().getQualifiedTitle()));
				} else {
					return main.getRedirectPerspective("index.jsp?action=view&title="+main.getNamespaceByUri(Namespace.NS_IKEWIKI_USER).getPrefix()+":"+Main.encodeURL(wpName));
				}
				//response.sendRedirect("index.jsp?action=edit_content&title="+Main.NS_USER.getPrefix()+":"+Main.escapeURL(wpName));
			}
			
			
		} else if(parameters.get("wpCreateaccount") != null) {
			String wpName      = parameters.get("wpName");
			String wpPassword  = parameters.get("wpPassword");
			String wpRetype    = parameters.get("wpRetype");
			String wpFirstName = parameters.get("wpFirstName");
			String wpLastName  = parameters.get("wpLastName");
			String wpEmail     = parameters.get("wpEmail");
			
			if(wpName == null || wpPassword == null || wpRetype == null || wpFirstName == null || wpLastName == null || wpEmail == null) {
				perspective.setAuthenticated(false);
				perspective.setErrorMessage("please fill in all fields");
			} else if(!wpPassword.equals(wpRetype)) {
				perspective.setAuthenticated(false);
				perspective.setErrorMessage("passwords don't match");
			} else {
				perspective.setAuthenticated(true);

				user = main.getPermissionManager().createUser(wpName,wpFirstName,wpLastName,wpEmail,wpPassword);
				session.setAttribute("user",user);

				setCookies(parameters,response);

				return main.getRedirectPerspective("index.jsp?action=edit_content&title="+main.getNamespaceByUri(Namespace.NS_IKEWIKI_USER).getPrefix()+":"+Main.encodeURL(wpName));
			}
			
		} else if(parameters.get("wpMailmypassword") != null) {
			// TODO
		}

		return perspective;
	}

}
