/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class LogoutAction extends BaseAction implements Action {

	private static final String[] required_parameters = { };
	private static final String[] acceptable_parameters = { "title" };


	/**
	 * @param main
	 */
	public LogoutAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "logout";
	}

	public String getLabel() {
		return "Logout";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/**
	 * Clear username and password cookies stored in the user's browser.
	 * @param response
	 */
	public void clearCookies(HttpServletResponse response) {
		Cookie c_passwd = new Cookie("IkeWikiPassword","");
		Cookie c_user   = new Cookie("IkeWikiUser","");
		c_passwd.setMaxAge(0);
		c_user.setMaxAge(0);			
		response.addCookie(c_passwd);
		response.addCookie(c_user);
	}
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData, javax.servlet.http.HttpServletResponse, javax.servlet.http.HttpSession)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		// load article in any case, we need it for title etc
		
		// remove user from session and set user variable to null
		session.removeAttribute("user");
		
		// unset cookies
		clearCookies(response);

		if(parameters.get("title") != null) {
			return main.getRedirectPerspective("index.jsp?action=view&title="+parameters.get("title"));
		} else {
			return main.getRedirectPerspective("index.jsp?action=view&title=FrontPage");
		}		
	}

}
