/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class NewPropertyAction extends BaseAction implements Action {

	private static final String[] required_parameters = { };
	private static final String[] acceptable_parameters = { "title", "namespace", "superclass" };

	/**
	 * @param main
	 */
	public NewPropertyAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "new_property";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getLabel()
	 */
	public String getLabel() {
		return "New Property";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData, javax.servlet.http.HttpServletResponse, javax.servlet.http.HttpSession)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		if(parameters.get("namespace") != null && parameters.get("title") != null) {
			String title;		
			if(parameters.get("namespace").equals("")) {
				title = parameters.get("title");
			} else {
				title = parameters.get("namespace")+":"+parameters.get("title");
			}
			
			WikiResource subject = main.getResourceByQTitle(title);
			
			if(parameters.get("type") != null) {
				main.getStorageBackend().storeType(subject,main.getResourceByUri(parameters.get("type")));
			} else {
				main.getStorageBackend().storeType(subject,main.getResourceByUri(Namespace.NS_RDF+"Property"));				
			}
			
			if(parameters.get("superclass") != null && !parameters.get("superclass").equals("none")) {
				WikiResource superclass = main.getResourceByQTitle(parameters.get("superclass"));
				main.getStorageBackend().addSuperProperty(subject,superclass);
			}
			return main.getRedirectPerspective("index.jsp?action=edit_content&title="+Main.encodeURL(title));
		} else {		
			return main.getNewPropertyPerspective();
		}
	}

	
}
