/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.w3c.dom.Document;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.EmptyPerspective;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * @author sschaffe
 *
 */
public class SparqlAction extends BaseAction implements Action {

	private static final String[] required_parameters = {  };
	private static final String[] acceptable_parameters = { "query", "submit" };


	/**
	 * @param main
	 */
	public SparqlAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "search_sparql";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getLabel()
	 */
	public String getLabel() {
		return "Advanced Search";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData, javax.servlet.http.HttpServletResponse, javax.servlet.http.HttpSession)
	 */
	public Perspective handleAction(RequestData parameters, HttpServletResponse response, HttpSession session) throws DBException {
		if(parameters.get("submit") != null) {
			
			String query = parameters.get("query");
			
			Document d = main.getStorageBackend().executeSparqlQuery(query);
			
			return main.getSparqlPerspective(d);
			
		}
		
		return new EmptyPerspective(main);
	}

}
