/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.actions;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.RequestData;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;
import at.srfg.ikewiki.layout.perspectives.ViewArticlePerspective;

/**
 * @author sschaffe
 *
 */
public class ViewAction extends BaseAction implements Action {

	private static final String[] required_parameters = { };
	private static final String[] acceptable_parameters = { "title", "revision" };

	/**
	 * @param main
	 */
	public ViewAction(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getIdentifier()
	 */
	public String getIdentifier() {
		return "view";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getLabel()
	 */
	public String getLabel() {
		return "Article";
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getAcceptableParameters()
	 */
	public Collection<String> getAcceptableParameters() {
		return Arrays.asList(acceptable_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#getRequiredParameters()
	 */
	public Collection<String> getRequiredParameters() {
		return Arrays.asList(required_parameters);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.actions.Action#handleAction(at.srfg.ikewiki.data.RequestData, javax.servlet.http.HttpServletResponse, javax.servlet.http.HttpSession)
	 */
	public Perspective handleAction(RequestData parameters,	HttpServletResponse response, HttpSession session) throws DBException {
	    // retrieve an article from the database and show it; if a revision has been selected,
	    // chose the article by revision
		String article_ns    = getArticleNS(parameters);
		String article_title = getArticleTitle(parameters); 

		ViewArticlePerspective perspective;
		
		if(parameters.get("revision") != null) {
			long revision = Long.parseLong(parameters.get("revision"));
			perspective = main.getViewArticlePerspective(article_ns, article_title, revision);
			perspective.setTitlePostfix(" (revision "+revision+")");
		} else {
			try {
				perspective=main.getViewArticlePerspective(article_ns,article_title);
			} catch(DBException ex) {
		
			    // in case FrontPage is not found, populate the database
				if(article_title.equals("FrontPage")) {
					try {
						perspective=new ViewArticlePerspective(main,main.populateDB());
					} catch(Exception ex2) {
						throw new DBException(ex2);
					}
				} else {
					return main.getRedirectPerspective("index.jsp?action=edit_content&title="+Main.encodeURL(article_ns + ":" + article_title));
				}
			}
		}
		return perspective;
	}

}
