/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

import java.net.URISyntaxException;
import java.util.Date;
import java.util.Locale;

import org.w3c.dom.Document;

import at.srfg.ikewiki.misc.CanonicalXMLWriter;

/**
 * @author sschaffe
 *
 */
public class Article extends WikiResource {
	
	private String comment,text;
	
	private Date timestamp;
	private Locale language;
	
	private String renderer;
	
	private Document base_dom, extended_dom;
	
	//private URI uri;
	
	private long revision_id = 0;
	
	public Article(Namespace namespace, String title, String comment, String text, String language, Date date) {
		super(namespace,title);
		this.comment	   = comment;
		this.text      = text;
		this.timestamp = date;
		this.language  = new Locale(language);
		this.base_dom       = null;
		this.extended_dom       = null;
	}

	public Article(Namespace namespace, String title, String comment, Document dom, String language, Date date) {
		super(namespace,title);
		this.comment	   = comment;
		this.text      = null;
		this.timestamp = date;
		this.language  = new Locale(language);
		this.base_dom  = dom;
		this.extended_dom       = null;
		
		if(base_dom != null)
			base_dom.normalizeDocument();
	}

	
	@Deprecated
	public Article(Namespace namespace, String title, 
				   String comment, String text, 
				   String language, String uri, Date date) throws URISyntaxException {
		super(namespace,title);
		this.comment	   = comment;
		this.text      = text;
		this.timestamp = date;
		//this.uri       = new URI(uri);
		this.language  = new Locale(language);
		this.extended_dom       = null;
	}

	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * @param comment The comment to set.
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}

	/**
	 * @return Returns the language.
	 */
	public Locale getLanguage() {
		return language;
	}

	/**
	 * @param language The language to set.
	 */
	public void setLanguage(Locale language) {
		this.language = language;
	}

	/**
	 * @return Returns the text.
	 */
	public String getText() {
		return text;
	}

	/**
	 * @param text The text to set.
	 */
	public void setText(String text) {
		this.text = text;
	}

	/**
	 * @return Returns the timestamp.
	 */
	public Date getTimestamp() {
		return timestamp;
	}

	/**
	 * @param timestamp The timestamp to set.
	 */
	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}


	/**
	 * @param uri The uri to set.
	 */
	/*public void setUri(URI uri) {
		this.uri = uri;
	}*/

	public long getRevisionId() {
		return revision_id;
	}

	public void setRevisionId(long revision_id) {
		this.revision_id = revision_id;
	}

	public String getRenderer() {
		return renderer;
	}

	public void setRenderer(String renderer) {
		this.renderer = renderer;
	}

	
	
	
	public Document getBaseDom() {
		return getBaseDom(false);
	}

	public Document getBaseDom(boolean read_only) {
		if(base_dom != null) {
			if(read_only) {
				return base_dom;
			} else {
				return (Document)base_dom.cloneNode(true);				
			}
		} else {
			return null;
		}
	}
	
	
	public void setBaseDom(Document base_dom) {
		this.base_dom = base_dom;
		
		if(base_dom != null) 
			base_dom.normalizeDocument();
	}

	/**
	 * @return Returns the extended_dom.
	 */
	public Document getExtendedDom() {
		return extended_dom;
	}

	/**
	 * @param extended_dom The extended_dom to set.
	 */
	public void setExtendedDom(Document dom) {
		this.extended_dom = dom;
		
		if(extended_dom != null)
			extended_dom.normalizeDocument();
	}
		
	
	
	
	public String getXML() throws WikiException {
		if(this.base_dom != null) {
			CanonicalXMLWriter writer = new CanonicalXMLWriter(this.base_dom);
			
			return writer.writeToString();
		} else {
			throw new WikiException("article not available as XML");
		}

	}
	

}
