/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collection;
import java.util.HashSet;

import at.srfg.ikewiki.permission.PermissionObject;

/**
 * @author sschaffe
 *
 */
public class Namespace implements Comparable, PermissionObject {

	// originally in RDFStore
	public static final String NS_DC_CORE = "http://purl.org/dc/elements/1.1/";
	public static final String NS_RDF     = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static final String NS_RDFS    = "http://www.w3.org/2000/01/rdf-schema#";
	public static final String NS_OWL     = "http://www.w3.org/2002/07/owl#";
	public static final String NS_EXIF    = "http://www.w3.org/2003/12/exif/ns#";

	// IkeWiki syntax namespaces
	public static final String NS_IKEWIKI_CORE    = "http://ikewiki.srfg.at/syntax/1.0/core";
	public static final String NS_IKEWIKI_EXT     = "http://ikewiki.srfg.at/syntax/1.0/ext";
	public static final String NS_IKEWIKI_WIKLET_CV = "http://ikewiki.srfg.at/syntax/1.0/wiklet/cv"; // classview wiklet
	public static final String NS_IKEWIKI_WIKLET_PV = "http://ikewiki.srfg.at/syntax/1.0/wiklet/pv"; // propertyview wiklet
	
	// IkeWiki knowledge base namespaces
	public static final String NS_IKEWIKI_BASE    = "http://ikewiki.srfg.at/base/";
	public static final String NS_IKEWIKI_HELP    = "http://ikewiki.srfg.at/help/";
	public static final String NS_IKEWIKI_SPECIAL = "http://ikewiki.srfg.at/special/";
	public static final String NS_IKEWIKI_ROLE    = "http://ikewiki.srfg.at/role/";
	public static final String NS_IKEWIKI_USER    = "http://ikewiki.srfg.at/user/";
	public static final String NS_IKEWIKI_WIKLET  = "http://ikewiki.srfg.at/internal/wiklet/";
	public static final String NS_IKEWIKI_ACTION  = "http://ikewiki.srfg.at/internal/action/";
	public static final String NS_IKEWIKI_NS      = "http://ikewiki.srfg.at/internal/namespace/";
	
	// others
	public static final String NS_XFORMS = "http://www.w3.org/2002/xforms";
	public static final String NS_XHTML  = "http://www.w3.org/1999/xhtml";
	public static final String NS_TRIX   = "http://jena.sourceforge.net/TriX/";
	public static final String NS_RXR    = "http://ilrt.org/discovery/2004/03/rxr/";
	public static final String NS_FOAF   = "http://xmlns.com/foaf/0.1/";

	@Deprecated
	public static final String NS_IW      = NS_IKEWIKI_BASE;
	
	public static final String NS_SPARQL_RESULTS    = "http://www.w3.org/2005/sparql-results#";
	
	private String uri,prefix;
	
	public Namespace(String prefix,String uri) {
		this.uri    = uri.trim();
		if(prefix != null)
			this.prefix = prefix.trim();
		else
			this.prefix="";
	}

	public String getPrefix() {
		return prefix;
	}

	public String getUri() {
		return uri;
	}
	
	public URI constructURI(String localPart) throws URISyntaxException {
		return new URI(getUri()+localPart);
	}
	
	public String toString() {
		return "<'"+getPrefix()+"'="+getUri()+">";
	}
	
	
	public boolean equals(Object o) {
		if(o instanceof Namespace) {
			Namespace n = (Namespace)o;
			
			return (n.getUri().equals(this.getUri()) && n.getPrefix().equals(this.getPrefix())); 
		} else {
			return false;
		}
	}
	
	public int hashCode() {
		return toString().hashCode();
	}

	
	public int compareTo(Object o) {
		if(o instanceof Namespace) {
			return getPrefix().compareTo(((Namespace)o).getPrefix());
		} else {
			return 0;
		}
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.permission.PermissionObject#getObjectIdentifier()
	 */
	public String getObjectIdentifier() {
		return Namespace.NS_IKEWIKI_NS+this.getUri().hashCode();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.permission.PermissionObject#getRelevantPermissions()
	 */
	public Collection<String> getRelevantPermissions() {
		if(relevantPermissions == null) {
			relevantPermissions = new HashSet<String>();
			relevantPermissions.add("view");
			relevantPermissions.add("modify");
		}
		return relevantPermissions;
	}
	
	private static Collection<String> relevantPermissions = null;
	
}
