/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

/**
 * @author wastl
 *
 */
public class Permission {

	public enum Type {
		PERMISSION("permission"), RESTRICTION("restriction"), NONE("none");
		
		String label;
		Type(String label) {
			this.label = label;
		}
		
		public String toString() {
			return label;
		}
		
		public String shortLabel() {
			return ""+label.toLowerCase().charAt(0);
		}
	};
	
	private String action_pattern, resource_pattern, description;
	private Type type;
	private int precedence;
	
	boolean direct = true;
	
	
	/**
	 * Construct a permission object representing a permission without pattern.
	 * Used to store different permission types.
	 * 
	 * @param name
	 */
//	public Permission(String name, String description) {
//		this(name,description,null,0,Type.NONE);
//	}
	
	/**
	 * Construct a permission object representing a concrete permission with
	 * name and pattern.
	 * 
	 */
	public Permission(Type type, int precedence, String resource_pattern, String action_pattern, String description) {
		super();
		
		this.resource_pattern 	= resource_pattern;
		this.action_pattern     = action_pattern;
		this.description 		= description;
		this.precedence  		= precedence;
		this.type        		= type;
	}
	

	

	/**
	 * @return Returns the direct.
	 */
	public boolean isDirect() {
		return direct;
	}

	/**
	 * @param direct The direct to set.
	 */
	public void setDirect(boolean direct) {
		this.direct = direct;
	}


	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return Returns the precedence.
	 */
	public int getPrecedence() {
		return precedence;
	}

	/**
	 * @param precedence The precedence to set.
	 */
	public void setPrecedence(int precedence) {
		this.precedence = precedence;
	}

	/**
	 * @return Returns the type.
	 */
	public Type getType() {
		return type;
	}

	/**
	 * @param type The type to set.
	 */
	public void setType(Type type) {
		this.type = type;
	}




	/**
	 * @return Returns the action_pattern.
	 */
	public String getActionPattern() {
		return action_pattern;
	}




	/**
	 * @param action_pattern The action_pattern to set.
	 */
	public void setActionPattern(String action_pattern) {
		this.action_pattern = action_pattern;
	}




	/**
	 * @return Returns the resource_pattern.
	 */
	public String getResourcePattern() {
		return resource_pattern;
	}




	/**
	 * @param resource_pattern The resource_pattern to set.
	 */
	public void setResourcePattern(String resource_pattern) {
		this.resource_pattern = resource_pattern;
	}
	

	
	
}
