/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Locale;

import javax.imageio.ImageIO;

/**
 * @author sschaffe
 *
 */
public class WikiImage extends Article {

	private byte[] bytes;
	
	private String mimeType, comment;
	
	/**
	 * @param ns
	 * @param title
	 */
	public WikiImage(Namespace ns, String title) {
		this(ns,title,"","",null,new Date());
	}
	
	
	public WikiImage(Namespace ns, String title, String comment, String mimeType, byte[] bytes, Date timestamp) {
		super(ns, title,comment,comment,Locale.getDefault().getLanguage(),timestamp);
		this.mimeType = mimeType;
		this.bytes = bytes;
		this.comment = comment;
	}


	public WikiImage(WikiResource title, String comment, String mimeType, byte[] bytes, Date timestamp) {
		this(title.getNamespace(),title.getTitle(),comment,mimeType,bytes,timestamp);
	}
	
	
	
	public byte[] getBytes() {
		return bytes;
	}


	public void setBytes(byte[] bytes) {
		this.bytes = bytes;
	}

	
	public BufferedImage getImage() {
		try {
			ByteArrayInputStream in = new ByteArrayInputStream(bytes);
			return ImageIO.read(in);
		} catch(IOException ex) {
			ex.printStackTrace();
			return null;
		}
		
	}


	public String getComment() {
		return comment;
	}


	public void setComment(String comment) {
		this.comment = comment;
	}


	public String getMimeType() {
		return mimeType;
	}


	public void setMimeType(String mimeType) {
		this.mimeType = mimeType;
	}


	
}
