/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

import java.util.Comparator;

/**
 * @author sschaffe
 *
 */
public class WikiLink implements Comparable<WikiLink> {

	private WikiResource source, property;
	private WikiNode     target;
	private String description;
	
	private boolean implicit = false;
	
	public WikiLink(WikiResource target, String description) {
		this.target = target;
		this.source = null;
		this.property = null;
		this.description = description;
	}
	
	public WikiLink(WikiResource source, WikiResource type, WikiNode target,  String description) {
		this.target = target;
		this.description = description;
		this.source = source;
		this.property = type;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public WikiResource getSource() {
		return source;
	}

	public String getSourceNamespace() {
		if(source != null) {
			return source.getNamespace().getPrefix();
		} else {
			return "";
		}
		
	}
	
	public String getSourceTitle() {
		if(source != null) {
			return source.getTitle();
		} else {
			return "";
		}
	}
	
	public void setSource(WikiResource source) {
		this.source = source;
	}

	public WikiNode getTarget() {
		return target;
	}

	public String getTargetNamespace() {
		if(target != null && target.isResource()) {
			return ((WikiResource)target).getNamespace().getPrefix();
		} else {
			return "";
		}
	}
	
	public String getTargetTitle() {
		if(target != null && target.isResource()) {
			return ((WikiResource)target).getTitle();
		} else {
			return "";
		}
	}

	public void setTarget(WikiNode target) {
		this.target = target;
	}

	public WikiResource getProperty() {
		return property;
	}

	public String getPropertyNamespace() {
		if(property != null) {
			return property.getNamespace().getPrefix();
		} else {
			return "";
		}
	}
	
	public String getPropertyTitle() {
		if(property != null) {
			return property.getTitle();
		} else {
			return "";
		}
	}
	
	
	
	public void setProperty(WikiResource type) {
		this.property = type;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if(obj instanceof WikiLink) {
			return this.hashCode() == obj.hashCode();
		} else {
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		// TODO Auto-generated method stub
		return toString().hashCode();
	}

	public String toString() {
		return "("+source+","+property+","+target+")";
	}

	public boolean isImplicit() {
		return implicit;
	}

	public void setImplicit(boolean implicit) {
		this.implicit = implicit;
	}
	
	
	
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(T)
	 */
	public int compareTo(WikiLink l) {
			
			// order by source, property, target
		
			int sc = this.getSource().compareTo(l.getSource());
			int pc = this.getProperty().compareTo(l.getProperty());
			int tc = this.getTarget().compareTo(l.getTarget());

			if(sc != 0) {
				return sc;
			} else if(pc != 0) {
				return pc;
			} else {
				return tc;
			}
	}

	public static Comparator<WikiLink> getComparator() {
		return new Comparator<WikiLink>() {
				public int compare(WikiLink l1, WikiLink l2) {
					return l1.compareTo(l2);
				}
		};
	}

	
}
