/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

import java.util.Comparator;

import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;


/**
 * @author sschaffe
 *
 */
public class WikiLiteral implements WikiNode {

	private String content, language;
	
	/**
	 * 
	 */
	public WikiLiteral(String content, String language) {
		super();
		this.content = content;
		this.language = language;
	}
	

	public String getContent() {
		return content;
	}




	public void setContent(String content) {
		this.content = content;
	}


	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.data.WikiNode#isResource()
	 */
	public boolean isResource() {
		return false;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.data.WikiNode#isLiteral()
	 */
	public boolean isLiteral() {
		return true;
	}

	public boolean isAnonymous() {
		return false;
	}

	public String getLanguage() {
		return this.language;
	}


	@Override
	public boolean equals(Object obj) {
		if(obj instanceof WikiLiteral) {
			return getContent().equals(((WikiLiteral)obj).getContent());
		} else {
			return false;
		}
	}


	@Override
	public int hashCode() {
		return getContent().hashCode();
	}




	public RDFNode toJenaNode(Model m) {
		return toJenaLiteral(m);
	}

	public Literal toJenaLiteral(Model m) {
		if(getLanguage() != null) {
			return m.createLiteral(getContent(),getLanguage());
		} else {
			return m.createLiteral(getContent());			
		}
	}
	
	public int compareTo(WikiNode o) {
		if(o instanceof WikiLiteral) {
			return getContent().compareTo(((WikiLiteral)o).getContent());
	 	} else if(o instanceof WikiResource) {
			return 1; // resources come before literals and anonymous nodes
		} else {
			return -1; // literals come before anonymous nodes
		}
	}

	
}
