/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.data;

import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;

import at.srfg.ikewiki.permission.PermissionObject;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;

/**
 * @author sschaffe
 *
 */
public class WikiResource implements WikiNode, PermissionObject {

	private Namespace namespace;
	private String title;
	private String creator = null;
	
	private boolean implicit = false;
		
	/**
	 * 
	 */
	public WikiResource(Namespace ns, String title) {
		super();
		this.namespace = ns;
		this.title     = title.trim();
				
	}

	
	
	public String toString() {
		return getQualifiedTitle();
	}



	/**
	 * @return Returns the namespace.
	 */
	public Namespace getNamespace() {
		return namespace;
	}



	/**
	 * @param namespace The namespace to set.
	 */
	public void setNamespace(Namespace namespace) {
		this.namespace = namespace;
	}



	/**
	 * @return Returns the title.
	 */
	public String getTitle() {
		return title;
	}



	/**
	 * @param title The title to set.
	 */
	public void setTitle(String title) {
		this.title = title;
	}


	
	

	public String getCreator() {
		return creator;
	}



	public void setCreator(String creator) {
		this.creator = creator;
	}



	public String getQualifiedTitle() {
		if(namespace.getPrefix().equals("")) {
			return title;
		} else {
			return namespace.getPrefix()+":"+title;
		}
	}
	
	public String getUri() {
		return namespace.getUri()+title;
	}



	public boolean isLiteral() {
		return false;
	}



	public boolean isResource() {
		return true;
	}
	
	public boolean isAnonymous() {
		return false;
	}
	
	
	public boolean equals(Object o) {
		if(o instanceof WikiResource) {
			return getUri().equals(((WikiResource)o).getUri());
		} else {
			return false;
		}
	}



	@Override
	public int hashCode() {
		return getUri().hashCode();
	}



	public int compareTo(WikiNode o) {
		if(o instanceof WikiResource) {
			return getQualifiedTitle().compareTo(((WikiResource)o).getQualifiedTitle());
		} else {
			return -1; // resources come before literals and anonymous nodes
		}
	}



	public boolean isImplicit() {
		return implicit;
	}



	public void setImplicit(boolean implicit) {
		this.implicit = implicit;
	}



	public RDFNode toJenaNode(Model m) {
		return toJenaResource(m);
	}
	
	public Resource toJenaResource(Model m) {
		return m.createResource(getUri());
	}



	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.permission.PermissionObject#getObjectIdentifier()
	 */
	public String getObjectIdentifier() {
		return this.getUri();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.permission.PermissionObject#getRelevantPermissions()
	 */
	public Collection<String> getRelevantPermissions() {
		if(relevantPermissions == null) {
			relevantPermissions = new HashSet<String>();
			relevantPermissions.add("view");
			relevantPermissions.add("modify");
		}
		return relevantPermissions;
	}
	
	private static Collection<String> relevantPermissions = null;
	
	
	
}
