/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.db;

import at.srfg.ikewiki.Main;

/**
 * Contains data common to all classes that access the underlying database system.
 * @author sschaffe
 *
 */
public abstract class DBCommon {

	protected String DB_DRIVER = "org.postgresql.Driver";
	protected String DB_NAME, DB_HOST, DB_PORT, DB_USER, DB_PASS;
		
	protected Main main;
	

	protected DBCommon(Main m) throws DBException {
		this.main = m;
		
		try {
			Class.forName(DB_DRIVER);
		} catch(ClassNotFoundException ex) {
			throw new DBException("PostgreSQL Database Driver not found. Maybe you need to set the CLASSPATH correctly.");
		}

		this.DB_NAME = main.getConfig().getString("database.name","ikewiki");
		this.DB_HOST = main.getConfig().getString("database.host","localhost");
		this.DB_PORT = main.getConfig().getString("database.port","5432");
		this.DB_USER = main.getConfig().getString("database.user","ikewiki");
		this.DB_PASS = main.getConfig().getString("database.pass","ikewiki");
		
		
	}
	
	protected String getConnectionString() {
		return "jdbc:postgresql://"+DB_HOST+":"+DB_PORT+"/"+DB_NAME;
	}
	
}
