/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *
 *  File: RDFNodeEncoder.java
 *
 */

package at.srfg.ikewiki.db;

import at.srfg.ikewiki.data.WikiLiteral;
import at.srfg.ikewiki.data.WikiNode;
import at.srfg.ikewiki.data.WikiResource;

import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;

/**
 * Encode an RDF node or Wiki node suitable for storage in the metadata journal.
 * 
 * @author sschaffe
 *
 *
 */
public class RDFNodeEncoder {

	
	String type, encoded, overflow;
	
	RDFNodeEncoder(RDFNode myNode) {
		
		// determine type
		if(myNode.isAnon()) {
			type = "A"; // anonymous node
		} else if(myNode.isURIResource()) {
			type = "U"; // URI node
		} else {
			type = "L"; // literal node
		}
		
		// set encoded string
		String str,prefix = getType();
		if(myNode.isAnon()) {	
			// format TYPE::ID
			str = ((Resource)myNode).getId().toString();
		} else if(myNode.isURIResource()) {
			// format TYPE::URI
			str = ((Resource)myNode).getURI();
		} else {
			// format TYPE:LANG:OVERFLOW::CONTENT
			// TYPE = "L", OVERFLOW in {"+","-"}
			prefix += ":" + ((Literal)myNode).getLanguage();
			str = ((Literal)myNode).getLexicalForm();
			if(str.length() > 240) {
				str = str.substring(0,240);
				prefix += ":+";
			} else {
				prefix += ":-";
			}
		}
		encoded = prefix + "::" + str;
		
		// overflow
		if(myNode.isLiteral()) {
			overflow = ((Literal)myNode).getValue().toString();				
		} else {
			overflow = null;
		}
	}
	
	public RDFNodeEncoder(WikiNode myNode) {
		// determine type
		if(myNode.isAnonymous()) {
			type = "A"; // anonymous node
		} else if(myNode.isResource()) {
			type = "U"; // URI node
		} else {
			type = "L"; // literal node
		}
		
		// set encoded string
		String str,prefix = getType();
		if(myNode.isAnonymous()) {	
			// format TYPE::ID
			
			throw new UnsupportedOperationException("not implemented!");

		} else if(myNode.isResource()) {
			// format TYPE::URI
			str = ((WikiResource)myNode).getUri();
		} else {
			// format TYPE:LANG:OVERFLOW::CONTENT
			// TYPE = "L", OVERFLOW in {"+","-"}
			prefix += ":" + ((WikiLiteral)myNode).getLanguage();
			str = ((WikiLiteral)myNode).getContent();
			if(str.length() > 240) {
				str = str.substring(0,240);
				prefix += ":+";
			} else {
				prefix += ":-";
			}
		}
		encoded = prefix + "::" + str;
		
		// overflow
		if(myNode.isLiteral()) {
			overflow = ((WikiLiteral)myNode).getContent();				
		} else {
			overflow = null;
		}
		
	}
	
	
	public String getType() {
		return type;
	}
	
	public String getEncoded() {
		return encoded;
	}
	
	
	public String getOverflow() {
		return overflow;
	}
	

}
