/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.diff;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.jrcs.diff.Diff;
import org.apache.commons.jrcs.diff.DifferentiationFailedException;
import org.apache.commons.jrcs.diff.Revision;

/**
 * @author sschaffe
 *
 */
public class DiffEngine {
	
	/**
	 * 
	 */
	public DiffEngine() {
		super();
		// TODO Auto-generated constructor stub
	}

	
	/**
	 * Compute the difference between to text strings. A wrapper function around Apache JRCS's
	 * Diff module.
	 * 
	 * @param textold
	 * @param textnew
	 * @return
	 * @throws DifferentiationFailedException
	 */
	public Revision diff(String textold, String textnew) throws DifferentiationFailedException {
		
		
		String[] words_old = DiffEngine.tokenize(textold);
		String[] words_new = DiffEngine.tokenize(textnew);
		
		Diff d = new Diff(words_old);
		
		return d.diff(words_new);
		
	}
	
	/**
	 * The static function tokenize splits a String into a list of smaller chunks that can be used for
	 * input to the diff algorithm. 
	 * 
	 * The difference to Java's built-in split function is that the separators are still part of the 
	 * chunks so that joining the chunks in order yields the original string. This property is e.g. 
	 * used in the method ViewHistoryPerspective.compareRevisions() to create the highlighted diff output for the Wiki.
	 * 
	 * The current tokenize implementation splits the String into words separated by whitespace.
	 *  
	 * @param s
	 * @return
	 */
	public static String[] tokenize(String s) {
		Pattern p = Pattern.compile("((\\p{Space}+)|([^ \\t\\n\\x0B\\f\\r]+))",Pattern.MULTILINE);
		
		Matcher m = p.matcher(s);
		
		ArrayList<String> results = new ArrayList<String>();
		while(m.find()) {
			results.add(m.group(1));
		}
		return results.toArray(new String[0]);
	}
	
	
	public static void main(String[] args) {
		String s = "The quick brown fox jumps over the lazy dog.";
		
		String[] words = tokenize(s);
		
		for(String word : words) {
			System.err.printf("_%s_",word);
		}
		System.err.println();
	}
	
}
