/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.editing.templates;

import java.util.Map;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.WikiResource;

/**
 * @author sschaffe
 *
 */
public abstract class Template {

	protected Main main;
	
	protected Template(Main main) {
		this.main = main;
	}
	
	
	/**
	 * Get the name of this template.
	 * 
	 * @return
	 */
	public abstract String getName();
	
	
	/**
	 * Get the URI associated with this template. This information is used to select the
	 * correct template in the edit field based on the resource type.
	 * @return
	 */
	public abstract String getUri();
	
	
	/**
	 * 
	 * @return
	 */
	public abstract String getTemplateText(WikiResource r);
	
	
	/**
	 * Apply the rules associated with this template.
	 * 
	 * Examples are to add certain annotations in the RDF store (like setting the type).
	 * 
	 * @param r
	 */
	public abstract void applyTemplateRules(WikiResource r, Map<String,String> parameters);
	
	
	
	public String toString() {
		return getName();
	}
}
