/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.installer;

import java.net.MalformedURLException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;

import javax.servlet.ServletContext;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.data.Namespace;

/**
 * SQL Statements for initialising the database
 * 
 * @author sschaffe
 *
 */
public class InitDB {

	/* a function to update the modification and user fields of pages on change */
	public static final String FUNCTION_f_set_modification =
		"CREATE FUNCTION f_set_modification () RETURNS trigger AS '" +
		"BEGIN "+
		"  NEW.modification    := ''now''; "+
		"  NEW.modification_by := current_user; "+
		"  RETURN NEW; "+
		"END; "+
		"' LANGUAGE plpgsql;";

	/* *** user data *** */
	
	/* sequence of user ids */
	public static final String SEQUENCE_user_ids =
		"CREATE SEQUENCE user_ids;";
	
	/* the table holding user data */
	public static final String TABLE_users =
		"CREATE TABLE users ( "+
		"	id  		INT NOT NULL PRIMARY KEY DEFAULT nextval('user_ids'), "+
		"	login	 	CHAR(80) NOT NULL, "+
		"	password 	CHAR(80) NOT NULL, "+ /* md5 hash */
		"	first_name 	CHAR(80) NOT NULL, "+
		"	last_name   CHAR(80) NOT NULL, "+
		"	email 		CHAR(255) NOT NULL, "+
		"	first_login TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"	last_login 	TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"	login_count INT NOT NULL DEFAULT 0 "+
		");	";
	
	
	public static final String INDEX_idx_users =
		"CREATE UNIQUE INDEX idx_users_login ON users(login);";
	
	public static final String INSERT_users =
		"INSERT INTO roles (login,password,first_name,last_name,email) VALUES ('%s','%s','%s','%s','%s');";

	
	/* *** Roles *** */
	public static final String SEQUENCE_role_ids =
		"CREATE SEQUENCE role_ids;";
	
	public static final String TABLE_roles =
		"CREATE TABLE roles ( "+
		"	id  		INT NOT NULL PRIMARY KEY DEFAULT nextval('role_ids'), "+
		"	name 		CHAR(80) NOT NULL, "+
		"	description TEXT "+	
		");	";

//	public static final String INSERT_roles =
//		"INSERT INTO roles (name,description) VALUES ('%s','%s');";
//	
//	public static Hashtable<String,String> DATA_roles;
//	
//	static {
//		DATA_roles = new Hashtable<String,String>();
//		DATA_roles.put("Administrator", "role for system administrators; full access to all aspects of the system");
//		DATA_roles.put("Manager","role for managagers; create and modify all content in the system");
//		DATA_roles.put("Authenticated","default role for authenticated users; read access to all pages, write access to base namespace");
//		DATA_roles.put("Reader","default role for unauthenticated users; read access to all pages, but no write access");
//	}

	
	public static final String TABLE_user_roles =
		"CREATE TABLE user_roles ( "+
		"	user_id 	INT NOT NULL REFERENCES users(id) ON DELETE CASCADE, "+
		"	role_id 	INT NOT NULL REFERENCES roles(id) ON DELETE CASCADE," +
		"   UNIQUE(user_id,role_id) "+
		");";
	
	public static final String INDEX_idx_user_roles =
		"CREATE INDEX idx_user_roles_user ON user_roles(user_id);";
	
	/* *** URIs *** */
	
	public static final String SEQUENCE_uri_ids =
		"CREATE SEQUENCE uri_ids;";
	
	/**
	 * A table to store URIs and IDs; referenced by the triples_internal table.
	 */
	public static final String TABLE_uris = 
		"CREATE TABLE uris ( "+
		"		id		 	INT NOT NULL PRIMARY KEY DEFAULT nextval('uri_ids'), "+
		"		uri 		CHAR(255) NOT NULL "+
		"); ";
	
	public static final String INDEX_uris_uri =
		"CREATE UNIQUE INDEX idx_uris_uri ON uris(uri);";
	
	
	/* *** Namespaces *** */
	
	public static final String SEQUENCE_namespace_ids =
		"CREATE SEQUENCE namespace_ids;";
	
	public static final String TABLE_namespaces =
		"CREATE TABLE namespaces ( "+
		"		id 				INT NOT NULL DEFAULT nextval('namespace_ids'), "+
		"		uri 			CHAR(255) NOT NULL, "+
		"		prefix 			CHAR(255), "+
		"		PRIMARY KEY(id) "+
		");";
	
	public static final String INDEX_namespaces_uri =
		"CREATE UNIQUE INDEX idx_namespaces_uri ON namespaces(uri);";
	
	public static final String INDEX_namespaces_prefix =
		"CREATE UNIQUE INDEX idx_namespaces_prefix ON namespaces(prefix);";
	
	public static final String INSERT_namespaces =
		"INSERT INTO namespaces (uri,prefix) VALUES ('%s','%s');";
	
	public static Hashtable<String,String> DATA_namespaces;
	
	static {
		DATA_namespaces = new Hashtable<String,String>();
		DATA_namespaces.put("http://www.example.com/","");
		DATA_namespaces.put(Namespace.NS_IKEWIKI_HELP,   "help");
		DATA_namespaces.put(Namespace.NS_IKEWIKI_USER,   "user");		
		DATA_namespaces.put(Namespace.NS_IKEWIKI_ROLE,   "role");		
		DATA_namespaces.put(Namespace.NS_IKEWIKI_BASE,   "ikewiki");		
		DATA_namespaces.put(Namespace.NS_IKEWIKI_SPECIAL,"special");
		DATA_namespaces.put(Namespace.NS_IKEWIKI_ACTION, "action");		
		DATA_namespaces.put(Namespace.NS_IKEWIKI_WIKLET, "wiklet");		
		DATA_namespaces.put(Namespace.NS_IKEWIKI_NS,     "namespace");	
	}
	
	
	/* *** Pages *** */
	
	public static final String FUNCTION_f_set_page_uri =
		"CREATE FUNCTION f_set_page_uri () RETURNS trigger AS ' "+
		"DECLARE "+
		"	namespace_rec RECORD; "+
		"	uri_rec RECORD; "+
		"	full_uri varchar; "+
		"BEGIN "+
		"	/* need to check for URI and insert reference instead! */ "+
		""+
		"	SELECT INTO namespace_rec uri,prefix FROM namespaces WHERE id=NEW.namespace; "+
		""+
		"	full_uri    := namespace_rec.uri || NEW.title ; "+
		""+
		"	SELECT INTO uri_rec * FROM uris WHERE uris.uri = full_uri; "+
		"	IF NOT FOUND THEN "+
		"		INSERT INTO uris (uri) VALUES (full_uri); "+
		"		NEW.uri := currval(''uri_ids''); "+
		"	ELSE "+
		"		NEW.uri := uri_rec.id;"+
		"	END IF;"+
		""+
		"	RETURN NEW; "+
		"END; "+
		"' LANGUAGE plpgsql;";
	
	
	public static final String SEQUENCE_page_ids =
		"CREATE SEQUENCE page_ids;";

	/**
	 * The table pages contains the "static" meta-information about a Wiki page.
	 * 
	 * As the text content of the page is subject to change, it is not stored here, but
	 * instead via an intermediate table "revision".
	 *
	 * The uri column is computed automatically and should never be set explicitly.
	 */
	public static final String TABLE_pages =
		"CREATE TABLE pages ( "+
		"		id				INT NOT NULL DEFAULT nextval('page_ids'), "+
		"		namespace		INT REFERENCES namespaces(id) ON DELETE SET NULL, "+
		"		title			CHAR(255) NOT NULL, "+
		"		uri				INT REFERENCES uris(id) ON DELETE SET NULL, "+
		"		cur_revision	INT, "+
		"		creator         INT REFERENCES users(id) ON DELETE SET NULL, "+
		"		creation		TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"		modification		TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"		modification_by	CHAR(80) DEFAULT current_user, "+
		"		PRIMARY KEY(id) "+
		");";
	
	public static final String INDEX_idx_pages_namespace_title =
		"CREATE UNIQUE INDEX idx_pages_namespace_title ON pages(namespace,title);";
	
	public static final String INDEX_idx_pages_uri =
		"CREATE UNIQUE INDEX idx_pages_uri ON pages(uri);";
	
	public static final String TRIGGER_tr_pages_modification =
		"CREATE TRIGGER tr_pages_modification BEFORE INSERT OR UPDATE ON pages "+
		"		FOR EACH ROW EXECUTE PROCEDURE f_set_modification();";
	
	public static final String TRIGGER_tr_pages_uri =
		"CREATE TRIGGER tr_pages_uri BEFORE INSERT OR UPDATE ON pages "+
		"		FOR EACH ROW EXECUTE PROCEDURE f_set_page_uri();";
	

	/* *** Text content *** */
	public static final String SEQUENCE_text_ids =
		"CREATE SEQUENCE text_ids;";
	
	public static final String TABLE_texts =
		"CREATE TABLE texts ( "+
		"		id				INT NOT NULL DEFAULT nextval('text_ids'), "+
		"		language		CHAR(12), "+
		"		text			TEXT, "+
		"		text_xml			TEXT, "+
		"		PRIMARY KEY(id) "+
		");";
	
	/* *** Revisions *** */
	public static final String SEQUENCE_revision_ids =
		"CREATE SEQUENCE revision_ids;";
	
	/*
	 * Store the revision meta-information about pages.
	 *
	 * references a page description and a text; if updated, the cur_revision link in the page
	 * is also updated.
	 */	
	public static final String TABLE_revisions =
		"CREATE TABLE revisions ( "+
		"		id				INT NOT NULL DEFAULT nextval('revision_ids'), "+
		"		page			INT NOT NULL REFERENCES pages(id) ON DELETE CASCADE, "+
		"		text			INT NOT NULL REFERENCES texts(id) ON DELETE CASCADE, "+
		"		creator         INT REFERENCES users(id) ON DELETE SET NULL, "+
		"		comment			TEXT, "+	
		"		deleted			BOOLEAN DEFAULT FALSE, "+
		"		timestamp		TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"		creation		TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"		modification		TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"		modification_by	CHAR(80) DEFAULT current_user, "+
		"		PRIMARY KEY(id) "+
		");";
	
	public static final String INDEX_idx_revisions_page =
		"CREATE INDEX idx_revisions_page ON revisions(page);";
		
	public static final String INDEX_idx_revisions_timestamp =
		"CREATE INDEX idx_revisions_timestamp ON revisions(timestamp);";
	
	public static final String FUNCTION_f_update_revisions =
		"CREATE FUNCTION f_update_revisions() RETURNS trigger AS ' "+
		"BEGIN "+
		"	UPDATE pages SET cur_revision=NEW.id WHERE id=NEW.page; "+
		"	RETURN NEW; "+
		"END; "+
		"' LANGUAGE plpgsql;";
	
	
	public static final String TRIGGER_tr_update_revisions =
		"CREATE TRIGGER	tr_update_revisions AFTER INSERT OR UPDATE ON revisions "+
		"		FOR EACH ROW EXECUTE PROCEDURE f_update_revisions();";
	
	
	/* *** View Definitions *** */
	
	public static final String VIEW_current_page =
		"CREATE VIEW current_page AS ( "+
		"	SELECT P.id AS page_id, R.id AS revision_id, N.prefix AS ns_prefix, N.uri AS ns_uri, (SELECT login FROM users WHERE users.id = R.creator) AS creator, title, (N.uri || title) AS uri, R.timestamp, R.comment, T.language, T.text, T.text_xml "+
		"	FROM pages P, revisions R, texts T, namespaces N "+
		"	WHERE R.deleted = FALSE "+
		"	  AND R.page = P.id "+
		"	  AND R.text = T.id "+
		"	  AND P.cur_revision = R.id "+
		"	  AND P.namespace = N.id "+
		");";
	
	
	public static final String VIEW_page_revisions =
		"CREATE VIEW page_revisions AS ( "+
		"	SELECT P.id AS page_id, R.id AS revision_id, N.prefix AS ns_prefix, N.uri AS ns_uri, (SELECT login FROM users WHERE users.id = R.creator) AS creator, title, (N.uri || title) AS uri, R.timestamp, R.comment, T.language, T.text, T.text_xml "+
		"	FROM pages P, revisions R, texts T, namespaces N "+
		"	WHERE R.deleted = FALSE "+
		"	  AND R.page = P.id "+
		"	  AND R.text = T.id "+
		"	  AND P.namespace = N.id "+
		");";

	
	
	public static final String FUNCTION_f_rank_page =
		"CREATE FUNCTION f_rank_page(varchar,varchar,varchar) RETURNS float(24) AS ' "+
		"DECLARE "+
		"		pattern 	ALIAS FOR $1; "+
		"		title	 	ALIAS FOR $2; "+
		"		text	 	ALIAS FOR $3; "+
		"		title_pos	integer; "+
		"		text_pos	integer; "+
		"		rank		float(24); "+
		"BEGIN "+
		"	title_pos := position(pattern in title); "+
		"	text_pos  := position(pattern in text); "+
		"	rank := 0.0; "+
		""+
		"	IF title_pos > 0 THEN "+
		"		rank := rank + 1000 * 1/title_pos; "+
		"	END IF; "+
		""+
		"	IF text_pos > 0 THEN "+
		"		rank := rank + 100 * 1/text_pos; "+
		"	END IF; "+
		"	RETURN rank; "+
		"END; "+
		"' LANGUAGE plpgsql;";

	
	
	/* *** Table Space for storing images *** */
	public static final String SEQUENCE_image_ids =
		"CREATE SEQUENCE image_ids;";
	
	
	public static final String TABLE_images = 
		"CREATE TABLE images ( "+
		"	id 				INT NOT NULL DEFAULT nextval('image_ids'), "+
		"	namespace 		INT REFERENCES namespaces(id) ON DELETE SET NULL, "+
		"	title	 		CHAR(255) NOT NULL, "+
		"	uri				INT REFERENCES uris(id) ON DELETE SET NULL, " +
		"	comment 			TEXT, "+
		"	mimetype	 		CHAR(80) NOT NULL, "+
		"	image		 	BYTEA NOT NULL, "+
		"	creator      	INT REFERENCES users(id) ON DELETE SET NULL, "+
		"	creation 		TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"	modification	 	TIMESTAMP NOT NULL DEFAULT current_timestamp, "+
		"	modification_by CHAR(80) DEFAULT current_user, "+
		"	PRIMARY KEY(id) "+
		");";
	
	public static final String INDEX_idx_images_namespace_title =
		"CREATE UNIQUE INDEX idx_images_namespace_title ON images(namespace,title);";
	
	public static final String TRIGGER_tr_images_modification =
		"CREATE TRIGGER tr_images_modification BEFORE INSERT OR UPDATE ON images "+
		"		FOR EACH ROW EXECUTE PROCEDURE f_set_modification();";
	
	public static final String TRIGGER_tr_images_uri =
		"CREATE TRIGGER tr_images_uri BEFORE INSERT OR UPDATE ON images "+
		"		FOR EACH ROW EXECUTE PROCEDURE f_set_page_uri();";
	
	
	
	
	public static final String VIEW_current_image =
		"CREATE VIEW current_image AS ( "+
		"	SELECT 	I.id AS image_id, " +
		"			N.prefix AS ns_prefix, " +
		"			N.uri AS ns_uri, " +
		"			(SELECT login FROM users WHERE users.id = I.creator) AS creator, " +
		"			title, " +
		"			(N.uri || title) AS uri, " +
		"			I.creation AS timestamp, "+		
		"			I.mimetype, " +
		"			I.comment AS comment, "+
		"			I.image "+
		"	FROM images I, namespaces N "+
		"	WHERE I.namespace = N.id " +
		");";
	

	
	/* *** Variables *** */
	public static final String SEQUENCE_variable_ids =
		"CREATE SEQUENCE variable_ids;";
	
	public static final String TABLE_variables =
		"CREATE TABLE variables ( "+
		"		id 				INT NOT NULL DEFAULT nextval('variable_ids'), "+
		"		name 			CHAR(80) NOT NULL, "+
		"		value 			TEXT DEFAULT '' "+
		");";
	

	public static final String INDEX_idx_variables_name =
		"CREATE INDEX idx_variables_name ON variables(name);";
	

	
	public static final String SEQUENCE_metadata_journal_ids =
		"CREATE SEQUENCE metadata_journal_ids;";

	
	public static final String SEQUENCE_metadata_journal_transactions = 
		"CREATE SEQUENCE metadata_journal_transaction_ids;";
	
	public static final String TABLE_metadata_journal =
		"CREATE TABLE metadata_journal (" +
		"		id              INT NOT NULL DEFAULT nextval('metadata_journal_ids')," +
		"		transaction     INT NOT NULL," +
		"		timestamp       TIMESTAMP NOT NULL DEFAULT current_timestamp," +
		"		uid             INT REFERENCES users(id) ON DELETE SET NULL," +
		"		action          CHAR(1) NOT NULL, " +
		"		persistent      BOOLEAN NOT NULL,"+
		"		subject         CHAR(255), " +
		"		predicate       CHAR(255), " +
		"		object          CHAR(255), " +
		"		object_overflow TEXT," +
		"		PRIMARY KEY(id)"+		
		");";
	
	public static final String INDEX_metadata_journal_subject =
		"CREATE INDEX idx_metadata_journal_subject ON metadata_journal(subject)";
	

	public static final String INDEX_metadata_journal_predicate =
		"CREATE INDEX idx_metadata_journal_predicate ON metadata_journal(predicate)";

	public static final String INDEX_metadata_journal_object =
		"CREATE INDEX idx_metadata_journal_object ON metadata_journal(object)";
	
	
	/* *** permission system *** */
//	public static final String SEQUENCE_permission_ids =
//		"CREATE SEQUENCE permission_ids;";
//	
//	public static final String TABLE_permissions =
//		"CREATE TABLE permissions ("+
//		"	id 				INT NOT NULL DEFAULT nextval('permission_ids'),"+
//		"	name 			CHAR(255) NOT NULL,"+
//		"	description 			TEXT,"+
//		"	PRIMARY KEY(id)"+
//		");";
	
	
	public static final String SEQUENCE_role_permission_ids =
		"CREATE SEQUENCE role_permission_ids;";
	
	public static final String TABLE_role_permissions =
		"CREATE TABLE role_permissions ("+
		"	id 				INT NOT NULL DEFAULT nextval('role_permission_ids'),"+
		"	role_id			INT NOT NULL REFERENCES roles(id) ON DELETE CASCADE,"+
//		"	permission_id	INT NOT NULL REFERENCES permissions(id) ON DELETE CASCADE,"+
		"	kind	  		CHAR(1) NOT NULL,"+
		"	resource_pattern  CHAR(255) NOT NULL,"+
		"	action_pattern  CHAR(255) NOT NULL,"+
		"	precedence		INT NOT NULL DEFAULT 0,"+
		"	description 	TEXT,"+
		"	PRIMARY KEY(id)"+
		");";

	public static final String INDEX_role_permissions_role =
		"CREATE INDEX idx_role_permissions_role ON role_permissions(role_id)";
	
	public static final String SEQUENCE_user_permission_ids =
		"CREATE SEQUENCE user_permission_ids;";
	
	public static final String TABLE_user_permissions =
		"CREATE TABLE user_permissions ("+
		"	id 				INT NOT NULL DEFAULT nextval('user_permission_ids'),"+
		"	user_id			INT NOT NULL REFERENCES users(id) ON DELETE CASCADE,"+
//		"	permission_id	INT NOT NULL REFERENCES permissions(id) ON DELETE CASCADE,"+
		"	kind	  		CHAR(1) NOT NULL,"+
		"	resource_pattern  CHAR(255) NOT NULL,"+
		"	action_pattern  CHAR(255) NOT NULL,"+
		"	precedence		INT NOT NULL DEFAULT 0,"+
		"	description 	TEXT,"+
		"	PRIMARY KEY(id)"+
		");	";
	
	public static final String INDEX_user_permissions_user =
		"CREATE INDEX idx_user_permissions_user ON user_permissions(user_id)";
	
	
	public static final String VIEW_all_user_permissions =
		"CREATE VIEW all_user_permissions AS ("+
		"	( SELECT login,resource_pattern,action_pattern,precedence,kind,user_permissions.description, false AS indirect "+ 
		"	  FROM user_permissions, users "+ 
		"	  WHERE user_permissions.user_id = users.id ) "+
		"	UNION "+
		"	( SELECT login,resource_pattern,action_pattern,precedence,kind,role_permissions.description, true AS indirect "+
		"	  FROM role_permissions,roles,user_roles,users "+
		"	  WHERE role_permissions.role_id = roles.id "+
	    "		AND user_roles.role_id = roles.id "+
	    "		AND user_roles.user_id = users.id "+
	    "   )"+
	    ");";

	
	
	
	
	/* *** Database User and Permissions *** */
	public static final String USER_template =
		"CREATE USER %s WITH PASSWORD '%s';";
	
	
	
	public static final String GRANT_db_template =
		"GRANT ALL ON DATABASE %s TO %s;";
	
	public static final String GRANT_tables_template =
		"GRANT ALL ON variables, variable_ids, images, image_ids, current_image, users, user_ids, "+
		"current_page, page_revisions, revisions, pages, texts, namespaces, revision_ids, page_ids, " +
		"text_ids, uris, uri_ids, namespace_ids, metadata_journal, metadata_journal_ids, " +
		"metadata_journal_revisions, role_ids, roles, permission_ids, permissions, role_permission_ids, " +
		"role_permissions, user_permission_ids, user_permissions, all_user_permissions " +
		"TO %s;";
	

	
	public static final String DATABASE_template =
		"CREATE DATABASE %s OWNER %s ENCODING 'UNICODE';";
	

	
	public static final String INSTALL_plpgsql_handler =
		"CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler AS '$libdir/plpgsql' LANGUAGE C;";
	
	public static final String INSTALL_plpgsql =
		"CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler;";

	
	protected final static String DB_DRIVER = "org.postgresql.Driver";

	private String ow_user, ow_pass;
	private String db_host, db_port, db_name, root_user, root_pass;
	
	private static Log log = LogFactory.getLog(InitDB.class);

	private Connection database = null;
	private Connection root_db = null;

	
	
	public InitDB(String db_host, String db_port, String root_user, String root_pass) throws InstallationException {
		this(db_host,db_port,"ikewiki",root_user,root_pass,"ikewiki","ikewiki");
	}
	
	/**
	 * 
	 */
	public InitDB(String db_host, String db_port, String db_name, String root_user, String root_pass, String ow_user, String ow_pass) throws InstallationException {
		super();
		this.db_host = db_host;
		this.db_port = db_port;
		this.db_name = db_name;
		this.root_user = root_user;
		this.root_pass = root_pass;
		this.ow_user = ow_user;
		this.ow_pass = ow_pass;		

		try {
			Class.forName(DB_DRIVER);
		} catch(ClassNotFoundException ex) {
			log.fatal("PostgreSQL driver not found, not proceeding");
			throw new InstallationException("PostgreSQL driver not found, not proceeding");
		}
	
	}
	
	

	protected String getConnectionString() {
		return "jdbc:postgresql://"+db_host+":"+db_port+"/"+db_name;
	}

	
	protected String getRootConnectionString() {
		return "jdbc:postgresql://"+db_host+":"+db_port+"/template1";		
	}
	
	

	
	protected Connection getRootConnection() throws SQLException {
		if(root_db == null || root_db.isClosed()) {
			root_db = DriverManager.getConnection(getRootConnectionString(),root_user,root_pass);
		}
		return root_db;
	}
	

	protected Connection getConnection() throws SQLException {
		if(database == null || database.isClosed()) {
			database = DriverManager.getConnection(getConnectionString(),ow_user,ow_pass);
			database.setAutoCommit(false);
		}
		return database;
	}

	public void initConfig(ServletContext ctx) {
		try {
			PropertiesConfiguration config = new PropertiesConfiguration("config.properties");
			config.setAutoSave(true);
			log.info("loaded configuration file config.properties");
			
			config.setProperty("base_uri",ctx.getResource("index.jsp").toString());
			
			config.save();
			
			log.info("saved database configuration to configuation file path "+config.getPath());
		} catch(ConfigurationException ex) {
			log.error("error while saving configuration file, you might need to adjust the settings in config.properties manually",ex);
		} catch(Exception ex) {
			log.error("could not determine base url of servlet context; please set the URL manually in config.properties",ex);
		}
		
	}
	
		
	public void initDatabase() throws SQLException {
		
		// do things that need to be done as super user
		createPLpgSQL();
		createUser();
		createDatabase();
		
		// disconnect root access
		getRootConnection().close();
		
		
		// do things that can be done as the ontowiki user	
		createModificationFunction();
		createUserTables();
		createPermissionTables();
		createURITables();
		createNamespaceTables();
		createPageURIFunction();
		createPageTables();
		createTextTables();
		createUpdateRevisionsFunction();
		createRevisionTables();
		createImageTables();
		createVariableTables();
		
		createCurrentPageView();
		createRevisionsView();
		createCurrentImageView();
		
		createPageRankFunction();
		
		populateNamespaces();
//		populateRoles();
//		populatePermissions();
		
		createMetadataJournalTables();
		
		getConnection().commit();

		getConnection().close();
		
		
		saveDBSettings();
	}

	protected void createPLpgSQL() throws SQLException {
		Connection root_connection = getRootConnection();
		
		try {
			Statement stmt = root_connection.createStatement();
			stmt.addBatch(INSTALL_plpgsql_handler);
			stmt.addBatch(INSTALL_plpgsql);
			stmt.executeBatch();
			stmt.close();
		
			log.info("registered language PL/pgSQL");
		} catch(SQLException ex) {
			log.warn("exception while registering language PL/pqSQL, maybe it exists already?",ex);
		}
	}

	
	protected void createUser() throws SQLException {
		Connection root_connection = getRootConnection();
		
		Statement stmt = root_connection.createStatement();
		stmt.execute(String.format(USER_template,ow_user,ow_pass));
		stmt.close();		
		
		log.info("created user "+ow_user);
	}
	
	
	/**
	 * Create the database
	 * @throws SQLException
	 */
	protected void createDatabase() throws SQLException {
		Connection root_connection = getRootConnection();
		
		Statement stmt = root_connection.createStatement();
		stmt.execute(String.format(DATABASE_template,db_name,ow_user));
		stmt.close();
		
		log.info("created database "+db_name);
	}
	
	
	protected void createModificationFunction() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(FUNCTION_f_set_modification);
		stmt.close();
		
		log.info("created function f_set_modification");
	}

	
	protected void createUserTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_user_ids);
		stmt.addBatch(TABLE_users);
		stmt.addBatch(INDEX_idx_users);
		
		stmt.addBatch(SEQUENCE_role_ids);
		stmt.addBatch(TABLE_roles);
		
		stmt.addBatch(TABLE_user_roles);
		stmt.addBatch(INDEX_idx_user_roles);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created user tables");
	}
	
	protected void createURITables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_uri_ids);
		stmt.addBatch(TABLE_uris);
		stmt.addBatch(INDEX_uris_uri);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created URI tables");
	}

	protected void createNamespaceTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_namespace_ids);
		stmt.addBatch(TABLE_namespaces);
		stmt.addBatch(INDEX_namespaces_prefix);
		stmt.addBatch(INDEX_namespaces_uri);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created namespace tables");
	}

	
	protected void createPageURIFunction() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(FUNCTION_f_set_page_uri);
		stmt.close();
		
		log.info("created function f_set_page_uri");
	}

	
	protected void createPageTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_page_ids);
		stmt.addBatch(TABLE_pages);
		stmt.addBatch(INDEX_idx_pages_namespace_title);
		stmt.addBatch(INDEX_idx_pages_uri);
		stmt.addBatch(TRIGGER_tr_pages_modification);
		stmt.addBatch(TRIGGER_tr_pages_uri);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created page tables");
	}
	
	protected void createTextTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_text_ids);
		stmt.addBatch(TABLE_texts);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created text tables");
	}
	
	protected void createUpdateRevisionsFunction() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(FUNCTION_f_update_revisions);
		stmt.close();
		
		log.info("created function f_update_revisions");
	}

	protected void createRevisionTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_revision_ids);
		stmt.addBatch(TABLE_revisions);
		stmt.addBatch(INDEX_idx_revisions_page);
		stmt.addBatch(INDEX_idx_revisions_timestamp);
		stmt.addBatch(TRIGGER_tr_update_revisions);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created revision tables");
	}

	protected void createCurrentPageView() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(VIEW_current_page);
		stmt.close();
		
		log.info("created view current_page");
	}

	protected void createRevisionsView() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(VIEW_page_revisions);
		stmt.close();
		
		log.info("created view page_revisions");
	}

	protected void createPageRankFunction() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(FUNCTION_f_rank_page);
		stmt.close();
		
		log.info("created function f_set_page_uri");
	}

	protected void createImageTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_image_ids);
		stmt.addBatch(TABLE_images);
		stmt.addBatch(INDEX_idx_images_namespace_title);
		stmt.addBatch(TRIGGER_tr_images_modification);
		stmt.addBatch(TRIGGER_tr_images_uri);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created image tables");
	}

	protected void createVariableTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_variable_ids);
		stmt.addBatch(TABLE_variables);
		stmt.addBatch(INDEX_idx_variables_name);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created variable tables");
	}

	protected void createMetadataJournalTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
		stmt.addBatch(SEQUENCE_metadata_journal_ids);
		stmt.addBatch(SEQUENCE_metadata_journal_transactions);
		stmt.addBatch(TABLE_metadata_journal);
		stmt.addBatch(INDEX_metadata_journal_subject);
		stmt.addBatch(INDEX_metadata_journal_predicate);
		stmt.addBatch(INDEX_metadata_journal_object);
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created triple history tables");
	}

	protected void createPermissionTables() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		
//		stmt.addBatch(SEQUENCE_permission_ids);
//		stmt.addBatch(TABLE_permissions);

		stmt.addBatch(SEQUENCE_role_permission_ids);
		stmt.addBatch(TABLE_role_permissions);
		stmt.addBatch(INDEX_role_permissions_role);

		stmt.addBatch(SEQUENCE_user_permission_ids);
		stmt.addBatch(TABLE_user_permissions);
		stmt.addBatch(INDEX_user_permissions_user);
		
		stmt.addBatch(VIEW_all_user_permissions);

		
		stmt.executeBatch();
		stmt.close();
		
		log.info("created variable tables");
	}
	
	protected void createCurrentImageView() throws SQLException {
		Connection conn = getConnection();
		
		Statement stmt = conn.createStatement();
		stmt.execute(VIEW_current_image);
		stmt.close();
		
		log.info("created view current_image");
	}
	
	
	protected void populateNamespaces() throws SQLException {
		Connection conn = getConnection();
		Statement stmt = conn.createStatement();
		
		for(String key : DATA_namespaces.keySet()) {
			stmt.addBatch(String.format(INSERT_namespaces,key,DATA_namespaces.get(key)));
		}
		
		
		stmt.executeBatch();
		stmt.close();
		
		log.info("inserted initial namespace data");
	}
	

	
	/**
	 * Store the user specified database settings in the config.properties file.
	 *
	 */
	protected void saveDBSettings() {
		try {
			PropertiesConfiguration config = new PropertiesConfiguration("config.properties");
			config.setAutoSave(true);
			log.info("loaded configuration file config.properties");
			
			config.setProperty("database.host",this.db_host);
			config.setProperty("database.port",this.db_port);
			config.setProperty("database.name",this.db_name);
			config.setProperty("database.user",this.ow_user);
			config.setProperty("database.pass",this.ow_pass);
			
			config.save();
			
			log.info("saved database configuration to configuation file path "+config.getPath());
		} catch(ConfigurationException ex) {
			log.error("error while saving configuration file, you might need to adjust the settings in config.properties manually",ex);
		}
	}
	
	
	public void close() {
		try {
			getRootConnection().close();
		} catch(SQLException ex) {}
		try {
			getConnection().close();
		} catch(SQLException ex) {}
	}
}
