/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.installer;

import java.io.InputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.db.RDFStore;

import com.hp.hpl.jena.rdf.model.Model;

/**
 * @author sschaffe
 *
 */
public class InitOntologies {

	private static Log log = LogFactory.getLog(InitOntologies.class);

	private Main main;
	
	private String[] names;
		
	/**
	 * 
	 */
	public InitOntologies(String[] names) throws DBException {
		super();
		main = Main.getInstance();
		
		// effectively disable maintanance thread
		main.getStorageBackend().getRDFStore().getModelMaintanance().setInterval(10000);
		
		this.names = names;
	}

	
	public void initOntologies() throws Exception {
		for(String name : names) {
			if(name.equals("rdf-schema")) {
				loadOntology("rdf-schema.rdf");
			} else if(name.equals("owl")) {
				loadOntology("owl.rdfs");
			} else if(name.equals("dc")) {
				loadOntology("dublin-core-terms.rdfs");
				loadOntology("dublin-core.rdfs");
			} else if(name.equals("sumo")) {
				loadOntology("sumo.rdfs");
			} else if(name.equals("ikewiki")) {
				loadOntology("base/ikewiki.owl");
			} else if(name.equals("exif")) {
				loadOntology("exif/exif.owl");
			} else if(name.equals("skos")) {
				loadOntology("skos/skos-core.rdf");
				loadOntology("skos/skos-extensions.rdf");
			} else if(name.equals("lom")) {
				loadOntology("lom/lom-base.rdf");
				loadOntology("lom/lom-annotation.rdf");
				loadOntology("lom/lom-classification.rdf");
				loadOntology("lom/lom-educational.rdf");
				loadOntology("lom/lom-general.rdf");
				loadOntology("lom/lom-lifecycle.rdf");
				loadOntology("lom/lom-metametadata.rdf");
				loadOntology("lom/lom-relation.rdf");
				loadOntology("lom/lom-rights.rdf");
				loadOntology("lom/lom-technical.rdf");
			} else if(name.equals("dolce-lite")) {
				loadOntology("dolce-extract.owl");
			} else if(name.equals("biology")) {
				loadOntology("bio/biology.owl");
			} else if(name.equals("bibtex")) {
				loadOntology("bibtex/bibtex.owl");
			} else if(name.equals("foaf")) {
				loadOntology("foaf/foaf.owl");
			} else if(name.equals("cc")) {
				loadOntology("cc/creativecommons.rdfs");
			} else if(name.equals("help")) {
				loadOntology("help/help.rdf");
			}			
		}
		
		
		// reinitialise the Main instance to ensure that everything that depends on the loaded
		// ontologies gets aware of them
		main.reInit();
	}
	
	
		
	public RDFStore getRDFStore() throws DBException {
		return main.getStorageBackend().getRDFStore();
	}
	
	
	private void loadOntology(String filename) throws Exception {
		try {
			RDFStore r = main.getStorageBackend().getRDFStore();
			
			InputStream is = this.getClass().getResourceAsStream(filename);
						
			Model m = r.addTripleData(is);
			
			// this is a severe bug; instead of storing all triples, we should only
			// store those that have actually been added!
			main.getStorageBackend().storeTriples(m,null);	

			log.info("loaded ontology "+filename);
			
		} catch(Exception ex) {
			log.error("error while preloading ontologies: ",ex);
			throw ex;
		}
	}
	
}
