/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout;

import java.util.Iterator;

import at.srfg.ikewiki.data.WikiResource;

public interface ViewItem {
	/**
	 * Returns the default(pref.) label for an item
	 * @return The label or null if no available
	 */
	String getLabel();
	/**
	 * Returns the default(pref.) label for an item and an locale 
	 * @param loc The locale
	 * @return The Label or null if non label exists for the given locale
	 */
	//String getLabel(Locale loc);
	/**
	 * Returns the Description for a Item
	 * @return The description or null if no available
	 */
	String getDescription();
	/**
	 * Returns the Description for an item an locale or null if no defined
	 * @param loc the locale to use
	 * @return The description or null if no description is defined for the given locale
	 */
	//String getDescription(Locale loc);
	/**
	 * Getter gor the data element of the Item. It's asumed that there are
	 * no item wrappers for literals.
	 * @return WikiResource The data element wrapped by that item.
	 */
	WikiResource getResource();
	/**
	 * Getter for the Type of the item. Note that the type is represented
	 * by an other wiki resource and therefor it makes sence to use Item as
	 * an return type.<p>
	 * Implementation should instanciate the ttem on demand to avoid the
	 * creation of all items. 
	 * @return The Item used as Type
	 */
	ViewItem getType();
	/**
	 * Returns alternate labels based on the parsed key. 
	 * @param literalKey The name of the property
	 * @return Iterator over all defined labels for the parsed key
	 */
	Iterator<String> getLiteral(String literalKey);
	/**
	 * Returns alternate labels based on the parsed key anbd locale
	 * @param literalKey The label key
	 * @param loc The locale to use
	 * @return Iterator over all defined labels for the parsed key and locale
	 */
	//Iterator<String> getLiteral(String literalKey, Locale loc);
	/**
	 * Returns all types for an Item.<p>
	 * Implementation should instanciate the ttem on demand to avoid the
	 * creation of all items. 	 * @return Iterator over the types
	 */
	Iterator<ViewItem> getAllTypes();
	
	/**
	 * Returns the unique ID of the ViewItem.
	 * @return The ID of the actual Item
	 */
	String getID();
	/**
	 * Url encoded version of the ID. This String is typically used in URLs
	 * to link to the actual view item
	 * @return The url encoded Version of the ID of the view item
	 */
	String getUrlID();
	/**
	 * Returns all "active" keys for the actual ViewItem.
	 * Active means that for these keys there are literals present.
	 * @return All keys with defined literals
	 */
	Iterator<String> getLiteralKeys();
}
