/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout;

import java.util.Iterator;
import java.util.List;

import at.srfg.ikewiki.data.WikiResource;

public interface ViewItemFactory {
	/**
	 * Creates an Item for an WikiResource
	 * @param resource The resource to create the item for
	 * @return The Item wrapping the resource
	 */
	ViewItem getViewItem(WikiResource resource);
	/**
	 * Getter for the List of defined item keys
	 * @return the Itemkeys as Iterator
	 */
	Iterator<String> getLiteralKeys();
	/**
	 * Checks if an item key is defined
	 * @param key the key
	 * @return true if the key is active, otherwise false
	 */
	boolean isLiteralKey(String key);
	/**
	 * Returns the WikiResource for an literal key
	 * @param key The key
	 * @return the resource or null for invalid keys
	 */
	WikiResource getLiteralProperty(String key);
	/**
	 * Return true if a mapping is present for the given resource q-name 
	 * @param qName the qualified name of the resouorce
	 * @return true if an mapping is present.
	 */
	boolean isViewMapping(String qName);
	/**
	 * Return true if a mapping is present for the given property 
	 * @param property the property
	 * @return true if an mapping is present.
	 */
	boolean isViewMapping(WikiResource property);
	/**
	 * Returns the target resource of the mapping for qName or null if no
	 * mapping is present.
	 * @param qName the qualified name of the resouorce
	 * @return the target of the mapping or null if no mapping is present
	 */
	WikiResource getViewMapping(String qName);
	/**
	 * Returns the target resource of the mapping for the parsed property
	 * or null if no mapping is present.
	 * @param property the property
	 * @return the target of the mapping or null if no mapping is present
	 */
	WikiResource getViewMapping(WikiResource property);
	/**
	 * Returns all properties mapped to the via its qName parsed resource.
	 * @param qName The qName
	 * @return A list of all properties mapped to the parsed qName or null
	 * if no mapping is active.
	 */
	List<WikiResource> getMappedProps(String qName);
	/**
	 * Returns all properties mapped to the parsed property.
	 * @param property The property
	 * @return A list of all properties mapped to the parsed property or null
	 * if no mapping is active.
	 */
	List<WikiResource> getMappedProps(WikiResource property);
}
