/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.perspectives;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * @author sschaffe
 *
 */
public abstract class BasePerspective implements Perspective {

	protected static Log log = LogFactory.getLog(BasePerspective.class);

	protected Main main;
	
	private String error_msg = null;
	
	/**
	 * 
	 */
	public BasePerspective(Main m) {
		super();
		main = m;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getNamespace()
	 */
	public Namespace getNamespace() {
		try {
			return main.getStorageBackend().getNamespaceByUri("http://ikewiki.srfg.at/special/");
		} catch(DBException ex) {
			log.error("could not retrieve 'Special' namespace, probably a database error",ex);
			return null;
		}
	}

	/**
	 * The readable title of the page;
	 * 
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getTitle()
	 */
	public abstract String getTitle();

	
	
	public String getTitlePrefix() {
		return "";
	}

	public String getTitlePostfix() {
		return "";
	}
	
	
	public String getContentType() {
		return "application/xhtml+xml";
	}

	
	
	public String getErrorMessage() {
		return error_msg;
	}
	
	public void setErrorMessage(String s) {
		error_msg = s;
	}

	/**
	 * The local part of the page; namespace is "Special" in all cases.
	 * @return
	 */
	protected abstract String getLocalTitle();
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getQualifiedTitle()
	 */
	public String getQualifiedTitle() {
		return getNamespace().getPrefix()+":"+getLocalTitle();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getContent()
	 */
	public String getContent() {
		return "";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getLastModified()
	 */
	public Date getLastModified() {
		return new Date();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getLanguage()
	 */
	public Locale getLanguage() {
		return Locale.getDefault();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getURI()
	 */
	public URI getURI() {
		try {
			return getNamespace().constructURI(getLocalTitle());
		} catch(URISyntaxException ex) {
			log.error("error constructing URI",ex);
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Perspective#getViewCount()
	 */
	public int getViewCount() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getContext()
	 */
	public WikiResource getContext() {
		return null;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#hasContext()
	 */
	public boolean hasContext() {
		return false;
	}
	

}
