/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.perspectives;

import java.net.URI;
import java.util.Date;
import java.util.Locale;

import at.srfg.ikewiki.actions.Action;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;

public interface Perspective {

	
	public Namespace getNamespace();
	
	
	/**
	 * Return a prefix for the header of the resulting page.
	 * @return
	 */
	public String getTitlePrefix();
	
	
	/**
	 * Return a postfix for the header of the resulting page.
	 * @return
	 */
	public String getTitlePostfix();
	
	
	/**
	 * Return the title of this object (to be used in the header of the resulting page).
	 * @return
	 */
	public String getTitle();
	
	
	/**
	 * Return the qualified title of this object (used for referring to the object).
	 * 
	 * Usually, this is of the form namespace_prefix + ':' + title.
	 * If namespace_prefix is empty, it is just the title
	 */
	public String getQualifiedTitle();
	
	/**
	 * Create an HTML representation of the object. The HTML output is inserted in a &lt;div>
	 * element of class "main" and thus should not contain HTML header tags.
	 * @return
	 */
	public String getContent();
	
	/**
	 * Return a date describing when this page has been last modified. The result is used in 
	 * page footers for illustration purposes.
	 * @return
	 */
	public Date getLastModified();
	
	/**
	 * Get the locale of the language of the page.
	 * @return
	 */
	public Locale getLanguage();
	
	
	public URI getURI();
	
	public int getViewCount();
	
	
	public String getContentType();
	
	/**
	 * Return the last error message that happend when performing the action associated with this perspective.
	 * @return
	 */
	public String getErrorMessage();
	
	
	/**
	 * Set the error message for this perspective. Called by Action implementations upon errors.
	 * @param message
	 */
	public void setErrorMessage(String message);
	
	
	/**
	 * Return the list of actions that are applicable for this perspective.
	 * 
	 * @return
	 */
	public Iterable<Action> getActions();
	

	/**
	 * Return true if the perspective has a context resource it displays/modifies/...
	 */
	public boolean hasContext();
	
	/**
	 * Return the context resource, if existant; null otherwise.
	 */
	public WikiResource getContext();
}
