/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.perspectives;

import java.util.Iterator;
import java.util.List;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.actions.Action;
import at.srfg.ikewiki.data.Article;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.render.RenderingException;

/**
 * @author sschaffe
 *
 */
public class SearchPerspective extends BasePerspective {

	private String search;
	
	/**
	 * @param m
	 */
	public SearchPerspective(Main m, String search) {
		super(m);
		this.search = search;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.BasePerspective#getTitle()
	 */
	@Override
	public String getTitle() {
		return "Search Results";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.BasePerspective#getLocalTitle()
	 */
	@Override
	protected String getLocalTitle() {
		return "Search";
	}

	
	public List<Article> getSearchResults(int limit, int offset) throws DBException {
		return main.getStorageBackend().searchArticles(this.search,limit,offset);
	}
	
	
	public String getPreview(Article a)  {
		try {
			return main.getRenderingPipeline().transformSearchResult(a);
		} catch(RenderingException ex) {
			return "<div class=\"search-preview preview-error\">error rendering preview of this article</div>\n";
		}
	}
	
	
	public Iterable<Action> getActions() {
		return new Iterable<Action> () {

			public Iterator<Action> iterator() {
				return new Iterator<Action>() {
					
					boolean retrieved = false;
					
					public boolean hasNext() {
						return !retrieved;
					}

					public Action next() {
						retrieved = true;
						return main.getAction("search");
					}

					public void remove() {
						throw new UnsupportedOperationException();						
					}
					
				};
			}
		};
	}

}
