/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.perspectives;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.actions.Action;
import at.srfg.ikewiki.data.Article;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.render.RenderingException;
import at.srfg.ikewiki.render.RenderingPipeline;

/**
 * @author sschaffe
 *
 */
public class XMLExchangePerspective implements Perspective {

	protected Article article;
	protected Main main;
	private static RenderingPipeline engine;
	private String renderedContent;

	private static Log log = LogFactory.getLog(XMLExchangePerspective.class);

	
	/**
	 * 
	 */
	public XMLExchangePerspective(Main main,Article a) {
		article = a;			
		this.main = main;
		
		// load rendering engine that belongs to this article
		engine = (RenderingPipeline)main.getRenderingPipeline();
		
		try {
			renderedContent = engine.transformExchange(article);
		} catch(RenderingException ex) {
			renderedContent = "<div class=\"error\">"+Main.encodeXML(ex.getMessage())+"</div>";
			log.error("rendering error",ex);
		}
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getNamespace()
	 */
	public Namespace getNamespace() {
		return article.getNamespace();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getTitle()
	 */
	public String getTitle() {
		return article.getTitle();
	}


	
	public String getTitlePrefix() {
		return "";
	}

	public String getTitlePostfix() {
		return "";
	}
	
	
	public String getContentType() {
		return "application/xhtml+xml";
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getQualifiedTitle()
	 */
	public String getQualifiedTitle() {
		return article.getQualifiedTitle();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getContent()
	 */
	public String getContent() {
		return renderedContent;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getLastModified()
	 */
	public Date getLastModified() {
		return article.getTimestamp();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getLanguage()
	 */
	public Locale getLanguage() {
		return article.getLanguage();
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getURI()
	 */
	public URI getURI() {
		try {
			return new URI(article.getUri());
		} catch(URISyntaxException ex) {
			ex.printStackTrace();
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getViewCount()
	 */
	public int getViewCount() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getActions()
	 */
	public Iterable<Action> getActions() {
		return Collections.emptyList();
	}
	
	String error_msg;
	
	public String getErrorMessage() {
		return error_msg;
	}

	public void setErrorMessage(String message) {
		error_msg = message;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#getContext()
	 */
	public WikiResource getContext() {
		return article;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.perspectives.Perspective#hasContext()
	 */
	public boolean hasContext() {
		return true;
	}

	
}
