/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.portlet;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Vector;

import at.srfg.ikewiki.data.User;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.permission.PermissionManager;

public class Portlet {

	private PermissionManager pm;
	private String title,key;
	private Vector<PortletItem> myItems;
	
	public Portlet(PermissionManager pm, String key, String title) {
		this.pm = pm;
		this.title = title;
		this.key   = key;
		myItems      = new Vector<PortletItem>();
	}
	
	/**
	 * Return a title to display in the portal box
	 * @return
	 */
	public String getTitle() {
		return title;
	}
	
	/**
	 * Return the key (unique identifier) of this portlet box
	 * @return
	 */
	public String getKey() {
		return key;
	}
	
	
	public void addPortletItem(PortletItem pi) {
		myItems.add(pi);	
	}	
	
	
	/**
	 * Return an iterable container for the items in this portlet.
	 * @return
	 */
	public Iterable<PortletItem> portletItems(User user, WikiResource context) throws DBException {
		Collection<PortletItem> results = new LinkedList<PortletItem>();
		
		if(user == null)
			user = pm.getAnonymousUser();

		
		for(PortletItem p : myItems) {
			if(p.getAction() == null || pm.hasPermission(user,context,p.getAction())) {
				results.add(p);
			}
		}
		return results;
	}
}
