/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.portlet;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import at.srfg.ikewiki.actions.Action;

public class PortletItem {
	
	private static Pattern p_variables = Pattern.compile("\\{([^\\}]+)\\}");
	
	private String key, title, action_str;
	private Action action;
	
	@Deprecated
	public PortletItem(String key, String title, String action) {
		this.key = key;
		this.title = title;
		this.action_str = action;
	}
	
	public PortletItem(Action action) {
		this(action.getLabel(),action);
	}

	public PortletItem(String title, Action action) {
		this(title,action,null);
	}
	
	/**
	 * Create a portlet item for a given action displaying the given title and using the given parameters in addition to the template parameters
	 * @param title
	 * @param action
	 * @param parameters
	 */
	public PortletItem(String title, Action action,Map<String,String> parameters) {
		this.key = action.getIdentifier();
		this.title = title;
		this.action = action;
		
		StringBuilder buf = new StringBuilder();
		buf.append("index.jsp?action=");
		buf.append(action.getIdentifier());
		
		for(String param : action.getRequiredParameters()) {
			if(parameters == null || !parameters.containsKey(param)) {
				buf.append("&amp;");
				buf.append(param);
				buf.append("={");
				buf.append(param);
				buf.append("}");
			}
		}
		
		if(parameters != null) {
			for(String param : parameters.keySet()) {
				buf.append("&amp;");
				buf.append(param);
				buf.append("=");
				buf.append(parameters.get(param));
			}
		}
		
		this.action_str = buf.toString();
	}

	/**
	 * Return a unique identifier for this portlet item
	 * @return
	 */
	public String getKey() {
		return key;
	}
	
	/**
	 * Return a title to be displayed for this item in the portlet box
	 * @return
	 */
	public String getTitle() {
		return title;
	}
	
	
	
	/**
	 * @return Returns the action.
	 */
	public Action getAction() {
		return action;
	}

	/**
	 * Return an action string specifying what to do when a user clicks the portlet item
	 * 
	 * @param a list of variable -> value mappings; variables occurring in { } will be replaced by their value
	 * @return
	 */
	public String getActionString(Map<String,String> variables) {
		
		StringBuffer buf = new StringBuffer();
		
		Matcher m = p_variables.matcher(action_str);
		while(m.find()) {
			String replacement = variables!=null?variables.get(m.group(1)):null; 
			if(replacement != null) {
				m.appendReplacement(buf,replacement);
			} else {
				m.appendReplacement(buf,m.group());
			}
		}
		m.appendTail(buf);
		
		return buf.toString();
	}
	
}
