/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.viewitem;

import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.WikiLiteral;
import at.srfg.ikewiki.data.WikiNode;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.ViewItem;
import at.srfg.ikewiki.layout.ViewItemFactory;
import at.srfg.ikewiki.layout.perspectives.ViewArticlePerspective;

public class BasicViewItem implements ViewItem {

	public static final String RDF_Type = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
	protected WikiResource resource;
	protected Main main;
	protected ViewItemFactory itemFactory;
	private static Log log = LogFactory.getLog(BasicViewItem.class);
	private static Iterator<String> emtyIterator = new Iterator<String>() {
		public void remove() {}
		public String next() {return null;}
		public boolean hasNext() {return false;}
	};
	private static Iterator<ViewItem> emtyItemIterator = new Iterator<ViewItem>() {
		public void remove() {}
		public ViewItem next() {return null;}
		public boolean hasNext() {return false;}
	};
	
	public BasicViewItem(WikiResource resource) throws DBException {
		main = Main.getInstance();
		this.resource = resource;
		this.itemFactory = main.getViewItemFactory();
		
	}
	/**
	 * Uses the qualified name and ignores any Locale.
	 * Does some layouting ... as a Showcase ...
	 */
	public String getLabel() {
		
		//log.info("getLabel ->"+resource.getQualifiedTitle());
		StringBuffer buff = new StringBuffer();
		if(resource.getNamespace().getPrefix() != null && resource.getNamespace().getPrefix().length() > 0){
			buff.append("<small>");
			buff.append(resource.getNamespace().getPrefix());
			buff.append(":</small>");
		}
		buff.append(resource.getTitle());
		return buff.toString();
		//return "<small>"+resource.getNamespace().getPrefix()+":</small>"+resource.getTitle();
	}

	/**
	 * Uses the qualified name and ignores any Locale
	 */
/*	public String getLabel(Locale loc) {
		return getLabel();
	}*/

	public String getDescription() {
		try {
			return main.getViewArticlePerspective(resource.getNamespace().getPrefix(),resource.getTitle()).getContent();
		} catch(DBException ex) {
			try {
				return new ViewArticlePerspective(main,main.getNamespaceSpecial(),"Warning","no description available").getContent();
			} catch(Exception ex2) {
				log.error(ex2);
				return null;
			}
		}		
	}

/*	public String getDescription(Locale loc) {
		return getDescription();
	}*/

	public WikiResource getResource() {
		return resource;
	}

	public ViewItem getType() {
		Iterator<ViewItem> typeIterator = getAllTypes();
		return typeIterator.hasNext()?typeIterator.next():null;
	}
	
	public Iterator<String> getLiteralKeys(){
		List<WikiResource> propList = null;
		try {
			propList = Main.getInstance().getStorageBackend().getPropertiesBySubject(resource);
		} catch (DBException e) {
			log.warn("Unable to get active properties for "+resource.getQualifiedTitle(), e);
			return emtyIterator;
		}
		final Iterator<WikiResource> props = propList.iterator();
		return new Iterator<String>() {
			boolean hasNext;
			String next;
			public void remove() {
				throw new UnsupportedOperationException("Remove is not supported for this Iterator");
			}
		
			public String next() {
				if(hasNext){
					hasNext = false;
					return next;
				}else{
					throw new NoSuchElementException("No more Elements available");
				}
			}
		
			public boolean hasNext() {
				while(props.hasNext()){
					//search for the first Object of type Literal
					//and if one found set the QName of the Prop as next item
					WikiResource actProp = props.next();
					try {
						List<WikiNode> actValues = Main.getInstance().getStorageBackend().getObjectsBySubjectProperty(resource, actProp);
						for(WikiNode act : actValues){
							if(act.isLiteral()){
								WikiResource mappedResource = itemFactory.getViewMapping(actProp.getQualifiedTitle());
								if(mappedResource != null){
									//log.info("return LiteralKey [mapped]"+mappedResource.getQualifiedTitle()+" (from "+actProp.getQualifiedTitle()+")");
									next = mappedResource.getQualifiedTitle();
								} else {
									//log.info("return LiteralKey "+actProp.getQualifiedTitle());
									next = actProp.getQualifiedTitle();
								}
								hasNext =  true;
								return hasNext;
							}
						}
					} catch (DBException e) {
						log.warn("Unable to check if property "+actProp.getQualifiedTitle()+" has any Literals as Object for resource"+resource.getQualifiedTitle()+" (ignored)",e);
					}
				}
				return false;
			}
		};
	}
	
	public Iterator<String> getLiteral(String labelKey) {
			List<WikiResource> mappedPropList = itemFactory.getMappedProps(labelKey);
			//log.info("Get Literal for "+labelKey);
			final Iterator<WikiResource> mappedProps;
			if(mappedPropList != null){
				mappedProps = mappedPropList.iterator();
			} else {
				mappedProps =  null;
			}
			List<WikiNode> literalList = null;
			final Iterator<WikiNode> literals;
			try {
				//TODO: Filtering of dc:description is a workaround ...
				if(!labelKey.equals("dc:description")){
					literalList = Main.getInstance().getStorageBackend().getObjectsBySubjectProperty(resource, Main.getInstance().getResourceByQTitle(labelKey));
					//log.info(" ... get LiteralIterator for ["+resource.getQualifiedTitle()+","+labelKey+",?] -> "+literalList.size()+" Result(s)");
				}
			} catch (DBException e) {
				log.warn("Unable to get literal for "+resource.getQualifiedTitle()+" and key "+labelKey+"(return emty iterator)",e);
			}
			if(literalList != null){
				literals = literalList.iterator();
			} else {
				literals = null;
			}
			return new Iterator<String>() {
				Iterator<WikiNode> actLiterals = literals;
				boolean hasNext = false;
				String next = null;
				public void remove() {
					throw new UnsupportedOperationException("Remove is not supported for this Iterator");
				}
			
				public String next() {
					if(hasNext){
						hasNext = false;
						return next;
					}else{
						throw new NoSuchElementException("No more Elements available");
					}
				}
			
				public boolean hasNext() {
					while((actLiterals == null || !actLiterals.hasNext())
							&& mappedProps != null
							&& mappedProps.hasNext()){ // if no more Elements in the LiteralIterator but still more mapped Props
						log.info("     > search mapped Keys");
						while(mappedProps.hasNext()
								&& (actLiterals == null || !actLiterals.hasNext())){ //go to the next mapped Prop and reset the Iterator
							WikiResource wr = mappedProps.next();
							try {
								List<WikiNode> actMappedLiteralList = Main.getInstance().getStorageBackend().getObjectsBySubjectProperty(resource,wr);
								//log.info(" ... get LiteralIterator for ["+resource.getQualifiedTitle()+","+wr.getQualifiedTitle()+",?] -> "+actMappedLiteralList.size()+" Result(s)");
								actLiterals = actMappedLiteralList.iterator();
							} catch (DBException e) {
								log.warn("Unable to get Literals for Resource "+resource.getQualifiedTitle()+" and Property "+wr.getQualifiedTitle()+"(possible Literals get ignored)", e);
							}
						}
					} // now we have an valid iterator, or there are no more Elements
					// iterate over all Literals defined in the current iterator
					if(actLiterals != null){
						while(actLiterals.hasNext()){
							WikiNode act = actLiterals.next();
							//log.info("     > read WikiNode from Iterator");
							if(act.isLiteral()){
								next = ((WikiLiteral)act).getContent();
								//log.info("     > is Literal ["+next+"] -> return true");
								hasNext =  true;
							}
						}
					}
					return hasNext;
				}
			};
	}

/*	public Iterator<String> getLiteral(String labelKey, Locale loc) {
		return emtyIterator;
	}*/

	public Iterator<ViewItem> getAllTypes() {
		try {
			
			List<WikiNode> typeList = Main.getInstance().getStorageBackend().getObjectsBySubjectProperty(resource, Main.getInstance().getResourceByUri(RDF_Type));
			final Iterator<WikiNode>  types = typeList.iterator();
			return new Iterator<ViewItem>() {
				boolean hasNext = false;
				ViewItem next = null;
				public void remove() {
					throw new UnsupportedOperationException("Remove is not supported for this Iterator");
				}
				public ViewItem next() {
					if(hasNext){
						hasNext = false;
						return next;
					}else{
						throw new NoSuchElementException("No more Elements available");
					}
				}
				public boolean hasNext() {
					while(types.hasNext()){
						WikiNode act = types.next();
						if(act.isResource()){
							next = main.getViewItemFactory().getViewItem((WikiResource)act);
							hasNext =  true;
						}
					}
					return hasNext;
				}
			};
		} catch (DBException e) {
			log.warn("Unable to get Types for "+resource.getQualifiedTitle()+"(return emty iterator)",e);
			return emtyItemIterator;
		}
	}
	/**
	 * Returns the QualifiedTitle of the wrapped resource
	 */
	public String getID() {
		return resource.getQualifiedTitle();
	}
	/**
	 * Uses the encodeURL methode of Main to encode the ID provided by getID()
	 */
	public String getUrlID() {
		return Main.encodeURL(getID());
	}

}
