/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.views;

import java.util.Collection;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiLink;
import at.srfg.ikewiki.data.WikiLiteral;
import at.srfg.ikewiki.data.WikiNode;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.perspectives.Perspective;

import com.drew.metadata.exif.ExifDirectory;

/**
 * @author sschaffe
 *
 */
public class ImageMetadataView extends View {

	private static String NS_EXIF  = "http://www.w3.org/2003/12/exif/ns#";
	private static String NS_XHTML = "http://www.w3.org/1999/xhtml";
	
	private ExifDirectory directory;
	
	/**
	 * @param main
	 */
	public ImageMetadataView(Main main) throws DBException {
		super(main);

		directory  = new ExifDirectory();
		
	}


	
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.views.View#getType()
	 */
	@Override
	public TYPES getType() {
		return TYPES.CONTENT_START;
	}




	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.views.View#responsibleFor(at.srfg.ikewiki.data.WikiResource)
	 */
	@Override
	public boolean responsibleFor(WikiResource r) {
		try {
			Namespace ns = main.getStorageBackend().getNamespaceByUri(NS_EXIF);
			return r.getNamespace().equals(ns);
		} catch(DBException ex) {
			return false;
		}
	}

	
	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.views.View#getHTMLContent(at.srfg.ikewiki.data.WikiResource)
	 */
	@Override
	public String getHTMLContent(WikiResource context, Perspective perspective) {
		return "<div class=\"error\">BioTaxonomyView handled via Wiklets</div>";
	}

	
	
	private Node formatWikiNode(WikiNode n, String label, Document root) {
		if(n.isLiteral()) {
			WikiLiteral l = (WikiLiteral)n;
			return root.createTextNode(l.getContent()+" ");
		} else { // isResource()
			WikiResource r = (WikiResource)n;

			Element a = root.createElementNS(NS_XHTML,"a");
			a.setAttribute("name",        r.getQualifiedTitle());
			a.setAttribute("href",        "index.jsp?action=view&amp;title="+r.getQualifiedTitle());
			a.setAttribute("onmouseover", "showWMTT('tt-"+r.getQualifiedTitle()+"')");
			a.setAttribute("onmouseout",  "hideWMTT()");
			
			a.appendChild(root.createTextNode(label));
			
			return a;
		}
	}
	
	
	
	/**
	 * Create the view as HTML DOM for the given WikiResource
	 * 
	 * @param context     the WikiResource to be shown in this view
	 */
	public DocumentFragment createView(WikiResource context, Document root) {
		DocumentFragment result = root.createDocumentFragment();

		Element row, heading, col, e, f;
		
		try {
			result.appendChild(root.createComment("content generated from context by ImageMetadataView"));
			
			// table header
			Element table = root.createElementNS(NS_XHTML,"table");
			result.appendChild(table);
			table.setAttribute("class","exif");
			
			row = root.createElementNS(NS_XHTML,"tr");
			table.appendChild(row);

			heading = root.createElementNS(NS_XHTML,"th");
			heading.setAttribute("colspan","2");
			row.appendChild(heading);
			heading.appendChild(root.createTextNode("Image Metadata"));
			
			
			// the real taxonomy			

			e = table;
			
			Collection<WikiLink> edges = main.getStorageBackend().getTriplesBySubject(context);
			
			for(WikiLink l : edges) {

				if(!l.isImplicit() && l.getProperty().getNamespace().getUri().equals(NS_EXIF)) {

					int tag_id = main.getStorageBackend().getRDFStore().getEXIFTagId(l.getProperty());
					String label = "Unknown";
					
					if(tag_id > -1) {
						label = directory.getTagName(tag_id);						
					}
					if(label.startsWith("Unknown")) {
						label = l.getPropertyTitle();
					}
					
					row = root.createElementNS(NS_XHTML,"tr");
					e.appendChild(row);
					
					col = root.createElementNS(NS_XHTML,"td");
					row.appendChild(col);

					f = root.createElementNS(NS_XHTML,"i");
					col.appendChild(f);
					
					f.appendChild(formatWikiNode(l.getProperty(),label,root));
					
					
					col = root.createElementNS(NS_XHTML,"td");
					row.appendChild(col);
					if(l.getTarget().isLiteral()) {
						col.appendChild(root.createTextNode(((WikiLiteral)l.getTarget()).getContent()));
					} else {
						col.appendChild(root.createTextNode("unknown"));
					}
					
				}
			}
			
			
			
		} catch(DBException ex) {
		}
		
		return result;
		
	}

}
