/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.views;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiNode;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.layout.ViewItem;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * A simple view that displays the RDFS/OWL class hierarchy with a context of 1
 * 
 * @author sschaffe
 *
 */
public class SimpleHierarchyView extends View {

	private String ns_uri = "http://www.w3.org/2000/01/rdf-schema#";
	private String skos_core_ns_uri = "http://www.w3.org/2004/02/skos/core#";
	private String skos_ext_ns_uri = "http://www.w3.org/2004/02/skos/extensions#";
	
	List<WikiResource> supportedResources;
	//WikiResource p_subclassof;
	
	/**
	 * @param main
	 */
	public SimpleHierarchyView(Main main) throws DBException {
		super(main);

		Namespace rdfs = main.getStorageBackend().getNamespaceByUri(ns_uri);
		Namespace skos = main.getStorageBackend().getNamespaceByUri(skos_core_ns_uri);
		Namespace skos_ext = main.getStorageBackend().getNamespaceByUri(skos_ext_ns_uri);
		supportedResources = new ArrayList<WikiResource>();
		///p_subclassof = new WikiResource(myns,"subClassOf");
		supportedResources.add(new WikiResource(rdfs,"subClassOf"));
		supportedResources.add(new WikiResource(skos,"broader"));
		supportedResources.add(new WikiResource(skos_ext,"broaderInstantive"));
		supportedResources.add(new WikiResource(skos_ext,"broaderGeneric"));
		supportedResources.add(new WikiResource(skos_ext,"broaderPartitive"));
		supportedResources.add(new WikiResource(skos_ext,"relatedPartOf"));
	}


	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.views.View#getType()
	 */
	@Override
	public TYPES getType() {
		return TYPES.CONTENT_START;
	}


	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.views.View#responsibleFor(at.srfg.ikewiki.data.WikiResource)
	 */
	@Override
	public boolean responsibleFor(WikiResource r) {
		return supportedResources.contains(r);
/*		try {
			Namespace ns = main.getStorageBackend().getNamespaceByUri(ns_uri);
			return r.getNamespace().equals(ns);
		} catch(DBException ex) {
			return false;
		}*/
	}

	
	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.layout.views.View#getHTMLContent(at.srfg.ikewiki.data.WikiResource)
	 */
	@Override
	public String getHTMLContent(WikiResource context, Perspective perspective) {
		try {
			StringBuilder result = new StringBuilder();
			result.append("<!-- generated by RDFSClassHierarchyView view -->\n");
			result.append("<div id=\"class-hierarchy\">\n");
			Set<WikiNode> superclasses = new HashSet<WikiNode>();
			Set<WikiResource> subclasses = new HashSet<WikiResource>();
			for(WikiResource actProp : supportedResources){
				superclasses.addAll(main.getStorageBackend().getObjectsBySubjectProperty(context,actProp));
				subclasses.addAll(main.getStorageBackend().getSubjectsByPropertyObject(actProp,context));
			}
			ViewItem contextItem = main.getViewItemFactory().getViewItem(context);
				result.append("<table>");
				//int rows = Math.max(superclasses.size(), subclasses.size());
				//Iterator<WikiNode> superClassesIter = superclasses.iterator();
				//Iterator<WikiResource> subClassesIter = subclasses.iterator();
				//for(int i=0;i<rows;i++){
				result.append("<tr><td>");
				if(!(subclasses.isEmpty() && superclasses.isEmpty())){
					result.append("<b>Hierarchy:</b>");
				}
				result.append("</td><td>");
				if(superclasses.size()>1){
					result.append("{");
				}
				boolean first = true;
				for(WikiNode node : superclasses){
					if(node.isResource()){
						ViewItem item = main.getViewItemFactory().getViewItem((WikiResource)node);
						//TODO: Action is hardcoded to "view"
						if(!first){
							result.append(", ");
						} else {
							first = false;
						}
						result.append("<a href=\"index.jsp?title="+item.getUrlID()+"&amp;action="+"view"+"\" onmouseover=\"showWMTT('tt-"+item.getID()+"')\" onmouseout=\"hideWMTT()\">"+item.getLabel()+"</a>");
					}
				}
				if(superclasses.size()>1){
					result.append("}");
				}
				if(!superclasses.isEmpty()){
					result.append("</td><td> -&gt; </td><td>");
				}
				result.append(contextItem.getLabel());
				if(!subclasses.isEmpty()){
					result.append("</td><td> -&gt; </td><td>");
				}
				if(subclasses.size()>1){
					result.append("{");
				}
				first = true;
				for(WikiResource node : subclasses){
					ViewItem item = main.getViewItemFactory().getViewItem(node);
					if(!first){
						result.append(", ");
					} else {
						first = false;
					}
					result.append("<a href=\"index.jsp?title="+item.getUrlID()+"&amp;action="+"view"+"\" onmouseover=\"showWMTT('tt-"+item.getID()+"')\" onmouseout=\"hideWMTT()\">"+item.getLabel()+"</a>");
				}
				if(subclasses.size()>1){
					result.append("}");
				}
				result.append("</td></tr></table><hr/>");

/*			result.append("<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"300\" height=\"200\" version=\"1.1\">");
            result.append("<circle cx=\"150\" cy=\"100\" r=\"50\" />");
            result.append("</svg>\n");
			*/
			
			result.append("</div>\n");
			
			return result.toString();
			
		} catch(DBException ex) {
			return "";
		}
	}

}
