/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.views;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.layout.perspectives.Perspective;

/**
 * Views are fragments of a page that display context information about a resource in a
 * suitable way. For example, a view that is aware of a biology ontology could display a taxonomy
 * box for each resource that describes an animal or plant. Also, a "hasImage" view could be
 * used to display images if there is a "hasImage" link to a photography of the resource.
 * 
 * @author sschaffe
 *
 */
public abstract class View {

	public enum TYPES {
		PORTLET_RIGHT,  // display the view as a portlet on the right side of the content
		PORTLET_LEFT,   // display the view as a portlet on the left side of the content
		CONTENT_START,  // display the view as part of the content (at the beginning)
		CONTENT_END     // display the view as part of the content (at the end)
	};
	
	
	
	protected Main main;
	
	public View(Main main) {
		this.main = main;
	}
	
	
	
	
	/**
	 * Return true if the view considers itself responsible for a certain property, false otherwise.
	 * If the view is responsible for one of the properties of a context, the getHTMLContent() method
	 * is called by the servlet/JSP.
	 * 
	 * 
	 * @param r
	 * @return
	 */
	public abstract boolean responsibleFor(WikiResource r);
	
	/**
	 * Returns the type of this view. This information is used for determining where to position a view.
	 * 
	 * @return
	 */
	public abstract TYPES getType();
	
	
	/**
	 * Create the view in HTML format for the given WikiResource.
	 * 
	 * @param context     the WikiResource to be shown in this view
	 * @param perspective the currently active perspective; implementations may decide to make use of perspective information
	 * @return
	 */
	@Deprecated
	public abstract String getHTMLContent(WikiResource context, Perspective perspective);
	
	
	/**
	 * Create the view as HTML DOM for the given WikiResource
	 * 
	 * @param context     the WikiResource to be shown in this view
	 */
	public DocumentFragment createView(WikiResource context, Document root) {
		DocumentFragment result = root.createDocumentFragment();
		
		return result;
	}
}
