/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.layout.views;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * @author sschaffe
 *
 */
public class ViewRegistry {

	private static Log log = LogFactory.getLog(ViewRegistry.class);

	
	private static final String[] VIEWS = {
		"at.srfg.ikewiki.layout.views.BioTaxonomyView",
		"at.srfg.ikewiki.layout.views.SimpleHierarchyView",
		"at.srfg.ikewiki.layout.views.RDFSClassHierarchyView",
		"at.srfg.ikewiki.layout.views.ReferencesView",
		"at.srfg.ikewiki.layout.views.ImageMetadataView"
	};
	

	private Main main;
	
	private Hashtable<String,View> registry;


	/**
	 * 
	 */
	public ViewRegistry(Main main) {
		super();
		
		this.main = main;
		
		registry = new Hashtable<String,View>();
		
		for(String name : VIEWS) {
			registerView(name);
		}
	}
		
	
	private void registerView(String classname) {
		try {
			Class cls = Class.forName(classname);
			Constructor cons = cls.getConstructor(Main.class);
			
			registry.put(classname,(View)cons.newInstance(this.main));
			
		} catch(ClassNotFoundException ex) {
			log.error("could not register view "+classname+" (no class definition found)");
		} catch(NoSuchMethodException ex) {
			log.error("could not register view "+classname+" (class definition contains errors)",ex);
		} catch(InstantiationException ex) {
			log.error("could not register view "+classname+" (class is abstract)",ex);
		} catch(IllegalAccessException ex) {
			log.error("could not register view "+classname+ " (access not allowed)",ex);
		} catch(InvocationTargetException ex) {
			log.error("could not register view "+classname+" ("+ex.getCause().getLocalizedMessage()+")");
		}
	}
	

	
	public Collection<View> getViewsForResource(WikiResource r, View.TYPES type) {
		// TODO: currently, only considers outgoing properties; do we need incoming as well?
		
		// initalise a list as result. a set would be more efficient for avoiding duplicates,
		// but we want the views in the correct order
		List<View> result = new ArrayList<View>();

		try {
			Collection<WikiResource> relevant = new HashSet<WikiResource>(); 
			relevant.addAll(main.getStorageBackend().getPropertiesBySubject(r));
			relevant.addAll(main.getStorageBackend().getPropertiesByObject(r));
			
			for(View v : registry.values()) {
				for(WikiResource prop : relevant) {				
					if(v.responsibleFor(prop) && v.getType() == type && !result.contains(v)) {
						result.add(v);
					}
				}				
			}


		} catch(DBException ex) {
			log.error("could not retrieve context of resource "+r.toString());
		}
			
		return result;
	}
	
}
