
/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.misc;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import nu.xom.Document;
import nu.xom.canonical.Canonicalizer;
import nu.xom.converters.DOMConverter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import at.srfg.ikewiki.data.WikiException;

/**
 * @author sschaffe
 *
 */
public class CanonicalXMLWriter extends XMLWriter {

	private static Log log = LogFactory.getLog(CanonicalXMLWriter.class);

	private Document xom_document;
		
	
	
	/**
	 * @param document
	 * @param canonical
	 * @throws WikiException
	 */
	public CanonicalXMLWriter(org.w3c.dom.Document document) throws WikiException {
		super(document);

		document.normalizeDocument();
		
		xom_document = DOMConverter.convert(getDocument());

	}

	@Override
	public void setParameter(String key, Object value) {
		
	}

	@Override
	public String writeToString() throws WikiException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		
		writeToStream(out);
		
		return new String(out.toByteArray());
	}

	@Override
	public void writeToStream(OutputStream out) throws WikiException {
		Canonicalizer c14n = new Canonicalizer(out,Canonicalizer.EXCLUSIVE_XML_CANONICALIZATION);
		
		try {
			c14n.write(xom_document);
		} catch(IOException ex) {
			throw new WikiException(ex);
		}
		
	}

	
	
}
