/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.misc;

import java.io.IOException;
import java.io.Writer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;

/**
 * @author sschaffe
 *
 */
public class LogWriter extends Writer {

	private Log log;
	private Level level;
	
	private StringBuffer buffer;
	
	public LogWriter() {
		this(LogFactory.getLog(LogWriter.class),Level.DEBUG);
	}
	
	/**
	 * @param out
	 */
	public LogWriter(Log log, Level level) {
		this.log = log;
		this.level  = level;
		buffer = new StringBuffer();
	}

	@Override
	public void close() throws IOException {
		
	}

	@Override
	public void flush() throws IOException {
		synchronized(buffer) {
			if(level == Level.DEBUG) {
				log.debug(buffer.toString());
			} else if(level == Level.INFO) {
				log.info(buffer.toString());
			} else if(level == Level.WARN) {
				log.warn(buffer.toString());
			} else if(level == Level.ERROR) {
				log.error(buffer.toString());
			} else if(level == Level.FATAL) {
				log.fatal(buffer.toString());
			}
			
		}
	}

	@Override
	public void write(char[] cbuf, int off, int len) throws IOException {
		buffer.append(cbuf,off,len);		
	}

	
	

}
