/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.misc;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import org.w3c.dom.Document;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSSerializer;

import at.srfg.ikewiki.data.WikiException;

/**
 * @author sschaffe
 *
 */
public class StandardXMLWriter extends XMLWriter {

	private DOMImplementationLS impl;
	private LSSerializer writer;

	/**
	 * @param document
	 * @throws WikiException
	 */
	public StandardXMLWriter(Document document) throws WikiException {
		super(document);
		try {
			System.setProperty(DOMImplementationRegistry.PROPERTY, "org.apache.xerces.dom.DOMImplementationSourceImpl");
			DOMImplementationRegistry registry = DOMImplementationRegistry.newInstance();

			impl = (DOMImplementationLS)registry.getDOMImplementation("LS");
			
			writer = impl.createLSSerializer();
			
		} catch(IllegalAccessException ex) {
			throw new WikiException(ex);
		} catch(InstantiationException ex) {
			throw new WikiException(ex);
		} catch(ClassNotFoundException ex) {
			throw new WikiException(ex);
		}
	}
	
	public void setParameter(String key, Object value) {
		writer.getDomConfig().setParameter(key,value);		
	}
	
	public String writeToString() {
		return writer.writeToString(getDocument().getDocumentElement());
	}

	@Override
	public void writeToStream(OutputStream out) {
		PrintWriter w = new PrintWriter(new OutputStreamWriter(out));
		w.println(writeToString());
	}


}
