/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.misc;

import java.io.IOException;
import java.io.OutputStream;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import at.srfg.ikewiki.data.WikiException;

public abstract class XMLWriter {
	
	private Document document;
	
	
	/**
	 * 
	 */
	public XMLWriter(Document document) throws WikiException {
		super();
		
		this.document = document;
	}
	
	public abstract void setParameter(String key, Object value);
	
	public abstract String writeToString() throws WikiException;
	
	public abstract void writeToStream(OutputStream out) throws WikiException;
	

	public Document getDocument() {
		return document;
	}

	public void setDocument(Document document) {
		this.document = document;
	}
	
	
	public void debugXML(Document document) {
		
		try {
			document.normalizeDocument();
		} catch(Exception ex) {
			OutputFormat format = new OutputFormat(document);
			format.setIndenting(true);
			XMLSerializer output = new XMLSerializer(System.err, format);
			try {
				output.serialize(document);
			} catch(IOException e) {}
			
			
			Node sibling, currentChild = document.getDocumentElement();
			do {
				if(currentChild.getNodeType() == Node.TEXT_NODE) {
					if(((Text)currentChild).getData() == null) {
						System.err.println("text node contained 'null', parent: "+currentChild.getParentNode().getNodeName());
					}
				}
				if(currentChild.hasChildNodes()) {
					currentChild = currentChild.getFirstChild();
				} else {
					sibling = currentChild.getNextSibling();
					
					while(sibling == null && currentChild != document.getDocumentElement()) {
						currentChild = currentChild.getParentNode();
						sibling = currentChild.getNextSibling();
					}
					currentChild = sibling;
				}
				
			} while(currentChild != document.getDocumentElement());
		}
	}
	
}
