/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.permission;

import java.util.Collection;

/**
 * Objects associated with a permission implement this interface
 * 
 * @author wastl
 *
 */
public interface PermissionObject {

	/**
	 * Return the object identifier of this object as used in the permission database.
	 * - for Wiklets, this is "http://ikewiki.srfg.at/internal/wiklet/"+ classname
	 * - for Actions, this is "http://ikewiki.srfg.at/internal/action/"+ classname
	 * - for resources, this is the resource URI.
	 * 
	 * @return
	 */
	public String getObjectIdentifier();
	
	
	/**
	 * Return the list of permissions relevant for this PermissionObject.
	 * - for Wiklets and Actions, this is usually "execute"
	 * - for Resources and Namespaces, this is usually "view", "modify"
	 */
	public Collection<String> getRelevantPermissions();
}
