/**
 * 
 */
package at.srfg.ikewiki.render.engine.wikipedia;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import at.srfg.ikewiki.data.Article;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.render.Serializer;

/**
 * @author sschaffe
 *
 */
public class WikipediaSerializer implements Serializer {

	private static Log log = LogFactory.getLog(WikipediaSerializer.class);

	Article article;
	Document wif;
	
	/**
	 * 
	 */
	public WikipediaSerializer(Document wif) {
		super();
		
		this.wif = wif;
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.Serializer#serialize(org.w3c.dom.Document)
	 */
	public String serialize() {
		StringBuilder buf = new StringBuilder();
		
		// perform a depth-first left-to-right traversal of the document tree
		serializeNode(buf,this.wif.getDocumentElement(),"",false);
		
		
		return buf.toString();
	}

	
	private void serializeNode(StringBuilder buf, Node n, String prefix,boolean trim) {
		if(n.getNodeType() == Node.ELEMENT_NODE) {
			Element e = (Element)n;
			String tag = e.getLocalName();
			
			if(e.getNamespaceURI().equals(Namespace.NS_IKEWIKI_CORE) 
					|| e.getNamespaceURI().equals(Namespace.NS_IKEWIKI_EXT)) {
				if(tag.equals("page")) {
					// page element, do nothing; no need for encapsulation in Wikipedia syntax
					serializeChildNodes(buf,e,prefix,trim);
					
				} else if(tag.equals("title")) {

				} else if(tag.equals("qtitle")) {

				} else if(tag.equals("author")) {

				} else if(tag.equals("language")) {

				} else if(tag.equals("date")) {
					
				} else if(tag.equals("heading")) {
					// heading, get the level attribute to determine the number of "="
					int level = Integer.parseInt(e.getAttribute("level"));
					for(int i=0; i<level; i++) buf.append("=");
					buf.append(" ");
					
					serializeChildNodes(buf,e,prefix,trim);
					
					buf.append(" ");
					for(int i=0; i<level; i++) buf.append("=");
					buf.append("\n\n");
				} else if(tag.equals("paragraph")) {
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("\n\n");
				} else if(tag.equals("olist")) {
					serializeChildNodes(buf,e,prefix+'#',trim);
				} else if(tag.equals("ulist")) {
					serializeChildNodes(buf,e,prefix+'*',trim);
				} else if(tag.equals("item")) {
					if(buf.length() > 0 && buf.charAt(buf.length()-1) != '\n') {
						buf.append("\n");
					}
					buf.append(prefix);
					buf.append(" ");
					serializeChildNodes(buf,e,prefix,true);
					if(buf.length() > 0 && buf.charAt(buf.length()-1) != '\n') {
						buf.append("\n");
					}
				} else if(tag.equals("table")) {
					buf.append("{|");
					serializeAttributes(buf,e);
					buf.append("\n");
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("|}\n");
				} else if(tag.equals("row")) {
					buf.append("|-");
					serializeAttributes(buf,e);
					buf.append("\n");
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("\n");
				} else if(tag.equals("col")) {
					if(e.getPreviousSibling() == null) {
						buf.append("| ");
					} else {
						buf.append(" || ");
					}
					serializeChildNodes(buf,e,prefix,trim);
				} else if(tag.equals("quote")) {
					buf.append("\n\n<blockquote>\n");
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("\n</blockquote>\n\n");
				} else if(tag.equals("code")) {
					buf.append("\n\n<code>\n");
					serializeChildNodes(buf,e,prefix,false);
					buf.append("\n</code>\n\n");
				} else if(tag.equals("separator")) {
					buf.append("\n\n----\n\n");
				} else if(tag.equals("newline")) {
					buf.append("<br/>");
				} else if(tag.equals("emph")) {
					buf.append("''");
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("''");
				} else if(tag.equals("strong")) {
					buf.append("'''");
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("'''");
				} else if(tag.equals("verystrong")) {
					buf.append("''''");
					serializeChildNodes(buf,e,prefix,trim);
					buf.append("''''");
				} else if(tag.equals("intlink")) {
					buf.append("[[");
					buf.append(e.getAttribute("target"));
					if(e.getChildNodes().getLength() > 0) {
						buf.append(" | ");
						serializeChildNodes(buf,e,prefix,trim);
					}
					buf.append("]]");
				} else if(tag.equals("extlink")) {
					buf.append("[");
					buf.append(e.getAttribute("href"));
					if(e.getChildNodes().getLength() > 0) {
						buf.append(" ");
						serializeChildNodes(buf,e,prefix,trim);
					}
					buf.append("]");
				} else {
					log.warn("unrecognized element: "+tag);
					serializeChildNodes(buf,e,prefix,trim);
				}
			} else {
				buf.append("<");
				buf.append(e.getTagName());
				serializeAttributes(buf,e);
				buf.append(">");
				serializeChildNodes(buf,e,prefix,trim);
				buf.append("</");
				buf.append(e.getTagName());
				buf.append(">");				
			}
		} else if(n.getNodeType() == Node.TEXT_NODE) {
			Text txt = (Text)n;
			
			if(trim) {
				buf.append(txt.getData().trim().replace("\n"," "));
			} else {
				buf.append(txt.getData());
			}
		}
	}
	
	
	private void serializeChildNodes(StringBuilder buf, Node n, String prefix, boolean trim) {
		NodeList nl = n.getChildNodes();

		for(int i=0; i<nl.getLength(); i++) {
			serializeNode(buf,nl.item(i),prefix,trim);
		}
	}
	
	private void serializeAttributes(StringBuilder buf, Element e) {
		NamedNodeMap map = e.getAttributes();
		for(int i=0; i<map.getLength(); i++) {
			Attr attr = (Attr)map.item(i);
			buf.append(" ");
			buf.append(attr.getName());
			buf.append("=\"");
			buf.append(attr.getValue());
			buf.append("\"");
		}
	}
}
