<?xml version="1.0" encoding="UTF-8"?>

<!--
    *  Copyright (C) 2005
    *  Sebastian Schaffert and Rupert Westenthaler
    *  Salzburg Research Forschungsgesellschaft m.b.H.
    *
    *  IkeWiki is free software; you can redistribute it and/or
    * 	modify it under the terms of the GNU Lesser General Public
    * 	License as published by the Free Software Foundation; either
    * 	version 2 of the License, or (at your option) any later version.
    *
    * 	This program is distributed in the hope that it will be useful,
    * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
    * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    * 	Lesser General Public License for more details.
    *
    * 	You should have received a copy of the GNU Lesser General Public
    * 	License along with this library; if not, write to the
    * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    * 	Boston, MA 02111-1307, USA.
    * 
-->
 
<!-- 
    This stylesheet is a modified version from the normal HTML stylesheet; if the XML data contains
    a iw:form node, the page will be displayed with changeable form controls that allow a partial 
    editing of the data.
    
    TODO: it would be better to avoid redundancies by appropriate inclusion mechanisms
 -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:fn="http://www.w3.org/2005/xpath-functions"
                xmlns:iw="http://ikewiki.srfg.at/syntax/1.0/ext"
                xmlns:wiklet-cv="http://ikewiki.srfg.at/syntax/1.0/wiklet/cv"
                xmlns:wiklet-pv="http://ikewiki.srfg.at/syntax/1.0/wiklet/pv"
                xmlns:xfm="http://www.w3.org/2002/xforms"
                xmlns:sparql="http://www.w3.org/2005/sparql-results#" 
                xpath-default-namespace="http://ikewiki.srfg.at/syntax/1.0/core"
                exclude-result-prefixes="fn iw" 
                version="2.0">
    <xsl:import href="transform-wif-common.xsl"/>
    
    <!--    <xsl:output method="xhtml" encoding="UTF-8" indent="no" omit-xml-declaration="yes"/> -->
    <xsl:output method="xhtml" encoding="UTF-8" indent="no" omit-xml-declaration="yes"/>
    
    


    <!-- normal pages -->
    <xsl:template match="/page">
        <div id="content">
            <a name="top" id="contentTop"></a>
            
            <!-- page title -->
            <h1 class="firstHeading">
                <xsl:value-of select="title" />
            </h1>

                                       
            <xsl:call-template name="type-wiklet"/>
          
            <!-- if the page contains a form, we encapsulate the content in an HTML form element and add reset and submit buttons -->
            <xsl:choose>
                <xsl:when test="//iw:form">
                    <form   action="index.jsp?action=edit_form&amp;title={normalize-space(/page/iw:qtitle/text())}" 
                            method="post" enctype="multipart/form-data">
                        <xsl:call-template name="content"/>
                        <hr />
                        <div id="form-buttons">
                            <input type="reset" name="reset" value="Reset" /> <input type="submit" name="submit" value="Submit" />
                        </div>
                    </form> 
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="content"/>                    
                </xsl:otherwise>
            </xsl:choose>  
        </div>        
    </xsl:template>



    <xsl:template name="content">
        <!-- page content -->
        <div id="bodyContent">
            <!-- subtitle -->
            <h3 id="siteSub"><xsl:value-of select="subtitle" /></h3>
            
            <xsl:apply-templates select="iw:view[@position = 'before']"/>
            
            <xsl:apply-templates select="heading|paragraph|olist|ulist|dlist|table|image|float|quote|code|separator|iw:form"/>
            
            <xsl:apply-templates select="iw:view[@position = 'after']"/>
        </div>        
    </xsl:template>


    <xsl:template name="type-wiklet">
        <!-- add type widget -->
        <div class="typeContent">
            <table>
                <tr>
                    <td>Types:</td>
                    <td>
                        <xsl:apply-templates select="iw:type[@direct = 'true']"/>
                        
                        <xsl:apply-templates select="iw:type[@direct = 'false']"/>                                
                        
                        <a name="page-type-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageTypeForm('{normalize-space(/page/iw:qtitle/text())}');">
                            <img src="skins/common/images/type_add.png" alt="add type" width="12" height="12"/>                
                        </a>
                    </td>
                </tr>                    
                
                <!-- list superclasses if page describes a class -->
                <xsl:if test="iw:type = 'owl:Class'">
                    <tr>
                        <td>Superclasses:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:superclass) > 0">
                                    <xsl:apply-templates select="iw:superclass"/>
                                </xsl:when>
                                <xsl:otherwise><i>none</i></xsl:otherwise>
                            </xsl:choose>
                            <a name="page-domain-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageSuperClassForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                </xsl:if>
                
                <!-- show domain and range for properties -->                                
                <xsl:if test="iw:type = 'rdf:Property'">
                    <tr>
                        <td>Superproperties:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:superproperty) > 0">
                                    <xsl:apply-templates select="iw:superproperty"/>
                                </xsl:when>
                                <xsl:otherwise><i>none</i></xsl:otherwise>
                            </xsl:choose>
                            <a name="page-domain-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageSuperPropertyForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <td>Domain:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:domain) > 0">
                                    <xsl:apply-templates select="iw:domain"/>                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <i>any</i>
                                </xsl:otherwise>
                            </xsl:choose>                                
                            
                            <a name="page-domain-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageDomainForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <td>Range:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:range) > 0">
                                    <xsl:apply-templates select="iw:range"/>                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <i>any</i>
                                </xsl:otherwise>
                            </xsl:choose>
                            
                            <a name="page-range-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageRangeForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <td>Inverse:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:inverseOf) > 0">
                                    <xsl:apply-templates select="iw:inverseOf"/>                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <i>none</i>
                                </xsl:otherwise>
                            </xsl:choose>
                            <a name="page-inverse-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageInverseForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>                </xsl:if>
            </table>
        </div>        
    </xsl:template>


    <!-- XForms stuff -->

    <xsl:template match="iw:form">
        <table>
            <xsl:apply-templates select="xfm:input|xfm:select|xfm:select1|xfm:textarea"/>
        </table>        
    </xsl:template>
    
    
    <xsl:template match="xfm:input">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <input type="text" name="fv-{$name}" size="60">
                    <xsl:attribute name="value">
                        <xsl:value-of select="//xfm:instance//*[name(.) = $name]/text()"/>
                    </xsl:attribute>
                </input>
            </td>
        </tr>
    </xsl:template>
    
    <xsl:template match="xfm:textarea">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <textarea name="fv-{$name}" cols="60" rows="4">
                        <xsl:value-of select="//xfm:instance//*[name(.) = $name]/text()"/>
                </textarea>
            </td>
        </tr>        
    </xsl:template>
    
    
    <xsl:template match="xfm:select[@appearance='minimal']">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <select name="fv-{$name}" multiple="multiple">
                    <xsl:for-each select="xfm:item">
                        <option>
                            <xsl:attribute name="value">
                                <xsl:value-of select="xfm:value/text()"/>
                            </xsl:attribute>
                            <xsl:if test="contains(//xfm:instance//*[name(.) = $name]/text(),xfm:value/text())">
                                <xsl:attribute name="selected">selected</xsl:attribute>
                            </xsl:if>
                            <xsl:value-of select="xfm:label"/>
                        </option>
                    </xsl:for-each>
                </select>
            </td>
        </tr>                
    </xsl:template>

    <xsl:template match="xfm:select">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <xsl:for-each select="xfm:item">
                    <input type="checkbox" name="fv-{$name}">
                        <xsl:attribute name="value">
                            <xsl:value-of select="xfm:value/text()"/>
                        </xsl:attribute>
                        <xsl:if test="contains(//xfm:instance//*[name(.) = $name]/text(),xfm:value/text())">
                            <xsl:attribute name="checked">checked</xsl:attribute>
                        </xsl:if>
                    </input>
                    <xsl:value-of select="xfm:label"/>
                </xsl:for-each>
            </td>
        </tr>                
    </xsl:template>
    

    <xsl:template match="xfm:select1[@appearance='minimal']">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <select name="fv-{$name}">
                    <xsl:for-each select="xfm:item">
                        <option>
                            <xsl:attribute name="value">
                                <xsl:value-of select="xfm:value/text()"/>
                            </xsl:attribute>
                            <xsl:if test="contains(//xfm:instance//*[name(.) = $name]/text(),xfm:value/text())">
                                <xsl:attribute name="selected">selected</xsl:attribute>
                            </xsl:if>
                            <xsl:value-of select="xfm:label"/>
                        </option>
                    </xsl:for-each>
                </select>
            </td>
        </tr>                
    </xsl:template>

    <xsl:template match="xfm:select1">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <xsl:for-each select="xfm:item">
                    <input type="radio" name="fv-{$name}">
                        <xsl:attribute name="value">
                            <xsl:value-of select="xfm:value/text()"/>
                        </xsl:attribute>
                        <xsl:if test="contains(//xfm:instance//*[name(.) = $name]/text(),xfm:value/text())">
                            <xsl:attribute name="checked">checked</xsl:attribute>
                        </xsl:if>
                    </input>
                    <xsl:value-of select="xfm:label"/>
                </xsl:for-each>
            </td>
        </tr>                
    </xsl:template>
    
    
    <xsl:template match="xfm:label">
        <strong><xsl:apply-templates/></strong>
    </xsl:template>
    
    
</xsl:stylesheet>