<?xml version="1.0" encoding="UTF-8"?>

<!--
    *  Copyright (C) 2006
    *  Sebastian Schaffert and Rupert Westenthaler
    *  Salzburg Research Forschungsgesellschaft m.b.H.
    *
    *  IkeWiki is free software; you can redistribute it and/or
    * 	modify it under the terms of the GNU Lesser General Public
    * 	License as published by the Free Software Foundation; either
    * 	version 2 of the License, or (at your option) any later version.
    *
    * 	This program is distributed in the hope that it will be useful,
    * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
    * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    * 	Lesser General Public License for more details.
    *
    * 	You should have received a copy of the GNU Lesser General Public
    * 	License along with this library; if not, write to the
    * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    * 	Boston, MA 02111-1307, USA.
    * 
-->

<!--
    This stylesheet transforms an XHTML fragment back to wiki interchange format (WIF). 
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:iw="http://ikewiki.srfg.at/syntax/1.0/ext"
                xmlns:wif="http://ikewiki.srfg.at/syntax/1.0/core"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                version="2.0">
    <xsl:output method="xml" encoding="UTF-8" indent="no" omit-xml-declaration="no"/>
    
    <!-- root element -->
    <xsl:template match="//xhtml:html">
        <xsl:apply-templates select="//xhtml:div[@id='editor']"/>    
    </xsl:template>
    
    
    <xsl:template match="//xhtml:div[@id='editor']">
        <wif:page>
            <wif:title><xsl:value-of select="substring-after(@MozileID,':')"/></wif:title>
            <iw:qtitle><xsl:value-of select="@MozileID"/></iw:qtitle>
            
            <!-- language, author, date to be set by action handler -->
            <xsl:apply-templates/>
            
        </wif:page>
    </xsl:template>
    
    <xsl:template match="xhtml:b/xhtml:i | xhtml:i/xhtml:b">
        <wif:verystrong><xsl:apply-templates/></wif:verystrong>
    </xsl:template>

    <xsl:template match="xhtml:i | xhtml:em">
        <wif:emph><xsl:apply-templates/></wif:emph>
    </xsl:template>
    
    <xsl:template match="xhtml:b | xhtml:strong">
        <wif:strong><xsl:apply-templates/></wif:strong>
    </xsl:template>
    
    <xsl:template match="xhtml:a[@class='add-type']"/>

    <xsl:template match="xhtml:a[@class='external']">
        <wif:extlink href="{@href}"><xsl:apply-templates/></wif:extlink>
    </xsl:template>
    
    <!-- internal link get an additional lTarget attribute, action handler sets create links -->
    <xsl:template match="xhtml:a[@lTarget]">
        <wif:intlink target="{@lTarget}"><xsl:apply-templates/></wif:intlink>
    </xsl:template>

    <xsl:template match="xhtml:a[not(@href)]"/>    
    
    <xsl:template match="xhtml:br">
        <wif:newline/>
    </xsl:template>
    
    <xsl:template match="xhtml:hr">
        <wif:separator/>
    </xsl:template>
    
    <xsl:template match="xhtml:h2">
        <wif:heading level="1"><xsl:apply-templates/></wif:heading>
    </xsl:template>

    <xsl:template match="xhtml:h3">
        <wif:heading level="2"><xsl:apply-templates/></wif:heading>
    </xsl:template>

    <xsl:template match="xhtml:h4">
        <wif:heading level="3"><xsl:apply-templates/></wif:heading>
    </xsl:template>

    <xsl:template match="xhtml:h5">
        <wif:heading level="4"><xsl:apply-templates/></wif:heading>
    </xsl:template>

    <xsl:template match="xhtml:h6">
        <wif:heading level="5"><xsl:apply-templates/></wif:heading>
    </xsl:template>
    
    <!-- take non-empty paragraphs -->
    <xsl:template match="xhtml:p[child::node()]">
        <wif:paragraph><xsl:apply-templates/></wif:paragraph>
    </xsl:template>
    
    <!-- skip empty paragraphs -->
    <xsl:template match="xhtml:p"/>
    
    <xsl:template match="xhtml:blockquote">
        <wif:quote><xsl:apply-templates/></wif:quote>
    </xsl:template>
    
    <xsl:template match="xhtml:ol">
        <wif:olist><xsl:apply-templates/></wif:olist>
    </xsl:template>
    
    <xsl:template match="xhtml:ul">
        <wif:ulist><xsl:apply-templates/></wif:ulist>
    </xsl:template>

    <xsl:template match="xhtml:dl">
        <wif:dlist><xsl:apply-templates/></wif:dlist>
    </xsl:template>
    
    <xsl:template match="xhtml:li">
        <wif:item><xsl:apply-templates/></wif:item>
    </xsl:template>
    
    <xsl:template match="xhtml:dt">
        <wif:phrase><xsl:apply-templates/></wif:phrase>
    </xsl:template>

    <xsl:template match="xhtml:dd">
        <wif:definition><xsl:apply-templates/></wif:definition>
    </xsl:template>
    
    
    <xsl:template match="xhtml:pre">
        <wif:code><xsl:value-of select="."/></wif:code>
    </xsl:template>

    <!-- spans are generated by the modified mozile editor solely for DOM purposes, we thus simply skip them here -->    
    <xsl:template match="xhtml:span">
        <xsl:apply-templates/>
    </xsl:template>
    

    <xsl:template match="xhtml:a[@class='add-type']/xhtml:img"/>
    
    
    <!-- ##################################################################################### -->    
    <!-- all remaining nodes are assumed to be HTML and kept -->
    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>
    </xsl:template>
    
</xsl:stylesheet>