<?xml version="1.0" encoding="UTF-8"?>

<!--
    *  Copyright (C) 2006
    *  Sebastian Schaffert and Rupert Westenthaler
    *  Salzburg Research Forschungsgesellschaft m.b.H.
    *
    *  IkeWiki is free software; you can redistribute it and/or
    * 	modify it under the terms of the GNU Lesser General Public
    * 	License as published by the Free Software Foundation; either
    * 	version 2 of the License, or (at your option) any later version.
    *
    * 	This program is distributed in the hope that it will be useful,
    * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
    * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    * 	Lesser General Public License for more details.
    *
    * 	You should have received a copy of the GNU Lesser General Public
    * 	License along with this library; if not, write to the
    * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    * 	Boston, MA 02111-1307, USA.
    * 
-->
 

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:fn="http://www.w3.org/2005/xpath-functions"
                xmlns:iw="http://ikewiki.srfg.at/syntax/1.0/ext"
                xmlns:xfm="http://www.w3.org/2002/xforms"
                xmlns:wiklet-cv="http://ikewiki.srfg.at/syntax/1.0/wiklet/cv"
                xmlns:wiklet-pv="http://ikewiki.srfg.at/syntax/1.0/wiklet/pv"
                xpath-default-namespace="http://ikewiki.srfg.at/syntax/1.0/core"
                exclude-result-prefixes="fn iw xfm wiklet-cv wiklet-pv sparql" 
                xmlns:sparql="http://www.w3.org/2005/sparql-results#" 
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                version="2.0">
    <xsl:import href="transform-wif-common.xsl"/>

    <xsl:output method="xhtml" encoding="UTF-8" indent="no" omit-xml-declaration="yes"/>



    <!-- normal pages -->
    <xsl:template match="/page">
        <div id="content">
            <a name="top" id="contentTop"></a>
            
            <!-- page title -->
            <h1 class="firstHeading">
                <xsl:value-of select="title" />
            </h1>

                                       
            <xsl:call-template name="type-wiklet"/>
            
            <!-- page content -->
            <div id="bodyContent">
                <!-- subtitle -->
                <h3 id="siteSub"><xsl:value-of select="subtitle" /></h3>
                
                <xsl:apply-templates select="iw:view[@position = 'before']"/>
                
                <!-- style="-moz-user-modify: read-write; -moz-user-input: enabled; -moz-user-select: normal;" -->
                <div id="editor" qtitle="{normalize-space(/page/iw:qtitle/text())}" class="editor" >
                    <xsl:apply-templates select="heading|paragraph|olist|ulist|dlist|table|image|float|quote|code|separator|iw:form"/>
                </div>
                
                <xsl:apply-templates select="iw:view[@position = 'after']"/>
            </div>
        </div>        
    </xsl:template>


    <xsl:template name="type-wiklet">
        <!-- add type widget -->
        <div class="typeContent">
            <table>
                <tr>
                    <td>Types:</td>
                    <td>
                        <xsl:apply-templates select="iw:type[@direct = 'true']"/>
                        
                        <xsl:apply-templates select="iw:type[@direct = 'false']"/>                                
                        
                        <a class="add-type" name="page-type-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageTypeForm('{normalize-space(/page/iw:qtitle/text())}');">
                            <img src="skins/common/images/type_add.png" alt="add type" width="12" height="12"/>                
                        </a>
                    </td>
                </tr>                    
                
                <!-- list superclasses if page describes a class -->
                <xsl:if test="iw:type = 'owl:Class'">
                    <tr>
                        <td>Superclasses:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:superclass) > 0">
                                    <xsl:apply-templates select="iw:superclass"/>
                                </xsl:when>
                                <xsl:otherwise><i>none</i></xsl:otherwise>
                            </xsl:choose>
                            <a class="add-type" name="page-domain-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageSuperClassForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                </xsl:if>
                
                <!-- show domain and range for properties -->                                
                <xsl:if test="iw:type = 'rdf:Property'">
                    <tr>
                        <td>Superproperties:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:superproperty) > 0">
                                    <xsl:apply-templates select="iw:superproperty"/>
                                </xsl:when>
                                <xsl:otherwise><i>none</i></xsl:otherwise>
                            </xsl:choose>
                            <a class="add-type" name="page-domain-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageSuperPropertyForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <td>Domain:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:domain) > 0">
                                    <xsl:apply-templates select="iw:domain"/>                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <i>any</i>
                                </xsl:otherwise>
                            </xsl:choose>                                
                            
                            <a class="add-type" name="page-domain-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageDomainForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <td>Range:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:range) > 0">
                                    <xsl:apply-templates select="iw:range"/>                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <i>any</i>
                                </xsl:otherwise>
                            </xsl:choose>
                            
                            <a class="add-type" name="page-range-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageRangeForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <td>Inverse:</td>
                        <td>
                            <xsl:choose>
                                <xsl:when test="count(iw:inverseOf) > 0">
                                    <xsl:apply-templates select="iw:inverseOf"/>                            
                                </xsl:when>
                                <xsl:otherwise>
                                    <i>none</i>
                                </xsl:otherwise>
                            </xsl:choose>
                            <a class="add-type" name="page-inverse-{normalize-space(/page/iw:qtitle/text())}" href="javascript:showPageInverseForm('{normalize-space(/page/iw:qtitle/text())}');">
                                <img src="skins/common/images/type_add.png"/>                
                            </a>
                        </td>
                    </tr>                </xsl:if>
            </table>
        </div>        
    </xsl:template>




    <!-- ##################################################################################### -->
    <!-- paragraph level elements -->


    <!-- headings are rendered using the HTML h2-h6 elemnents; 
         in the mozile interface, we skip the table of contents -->
    <xsl:template match="heading">
                
        <a name="_{translate(fn:string-join(./text(),' '),' ','_')}"/>
        
        <xsl:choose>
            <xsl:when test="@level='1'"><h2><xsl:apply-templates/></h2></xsl:when>
            <xsl:when test="@level='2'"><h3><xsl:apply-templates/></h3></xsl:when>
            <xsl:when test="@level='3'"><h4><xsl:apply-templates/></h4></xsl:when>
            <xsl:when test="@level='4'"><h5><xsl:apply-templates/></h5></xsl:when>
            <xsl:when test="@level='5'"><h6><xsl:apply-templates/></h6></xsl:when>
            <xsl:otherwise><h6><xsl:apply-templates/></h6></xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>


    <xsl:template match="iw:view"/>
        
</xsl:stylesheet>