<?xml version="1.0" encoding="UTF-8"?>

<!--
    *  Copyright (C) 2005
    *  Sebastian Schaffert and Rupert Westenthaler
    *  Salzburg Research Forschungsgesellschaft m.b.H.
    *
    *  IkeWiki is free software; you can redistribute it and/or
    * 	modify it under the terms of the GNU Lesser General Public
    * 	License as published by the Free Software Foundation; either
    * 	version 2 of the License, or (at your option) any later version.
    *
    * 	This program is distributed in the hope that it will be useful,
    * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
    * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    * 	Lesser General Public License for more details.
    *
    * 	You should have received a copy of the GNU Lesser General Public
    * 	License along with this library; if not, write to the
    * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    * 	Boston, MA 02111-1307, USA.
-->
 

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:fn="http://www.w3.org/2005/xpath-functions"
                xmlns:iw="http://ikewiki.srfg.at/syntax/1.0/ext"
                xpath-default-namespace="http://ikewiki.srfg.at/syntax/1.0/core"
                exclude-result-prefixes="fn iw" 
                version="2.0">
    <xsl:output method="xhtml" encoding="UTF-8" indent="no" omit-xml-declaration="yes"/>

    <xsl:template match="/page">
        <div id="tooltip">
            <!-- subtitle -->
            <h3 id="siteSub"><xsl:value-of select="subtitle" /></h3>
            
            <xsl:apply-templates select="heading|paragraph|olist|ulist|dlist|table|image|float|quote|code|separator"/>
                
        </div>
    </xsl:template>

    <!-- ##################################################################################### -->
    <!-- line level elements -->
    
    <!-- emphasis -->
    <xsl:template match="emph">
        <i><xsl:apply-templates/></i>
    </xsl:template>

    <xsl:template match="strong">
        <b><xsl:apply-templates/></b>
    </xsl:template>

    <xsl:template match="verystrong">
        <b><i><xsl:apply-templates/></i></b>
    </xsl:template>

    <!-- links -->
    
    <!-- internal link -->
    <xsl:template match="intlink">

        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <span class="tt-intlink">
                    <xsl:apply-templates/>
                </span>                
            </xsl:when>
            <xsl:otherwise>
                <span class="tt-intlink">
                    <xsl:value-of select="@target"/>
                </span>                                
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

    <!-- external link -->
    <xsl:template match="extlink">
        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <span class="tt-extlink"><xsl:apply-templates/></span>                
            </xsl:when>
            <xsl:otherwise>
                <span class="tt-extlink"><xsl:value-of select="@href"/></span>                                
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- newline -->
    <xsl:template match="newline">
        <br />
    </xsl:template>

    <!-- separator -->
    <xsl:template match="separator">
        <hr />
    </xsl:template>
    

    <!-- ##################################################################################### -->
    <!-- paragraph level elements -->


    <!-- headings are rendered using the HTML h2-h6 elemnents -->
    <xsl:template match="heading">
                
        <xsl:choose>
            <xsl:when test="@level='1'"><h2><xsl:apply-templates/></h2></xsl:when>
            <xsl:when test="@level='2'"><h3><xsl:apply-templates/></h3></xsl:when>
            <xsl:when test="@level='3'"><h4><xsl:apply-templates/></h4></xsl:when>
            <xsl:when test="@level='4'"><h5><xsl:apply-templates/></h5></xsl:when>
            <xsl:when test="@level='5'"><h6><xsl:apply-templates/></h6></xsl:when>
            <xsl:otherwise><h6><xsl:apply-templates/></h6></xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

    <!-- paragraphs are rendered in HTML as <p> elements -->
    <xsl:template match="paragraph[@iw:style]">
        <p style="{@iw:style}">
            <xsl:apply-templates/>
        </p>        
    </xsl:template>
    
    <xsl:template match="paragraph">
        <p>
            <xsl:apply-templates/>
        </p>        
    </xsl:template>
    
    <!-- blockquote -->
    <xsl:template match="quote">
        <blockquote>
            <xsl:apply-templates/>            
        </blockquote>
    </xsl:template>
    
    <!-- ordered lists are rendered in HTML using ol elements -->
    <xsl:template match="olist">
        <ol>
            <xsl:apply-templates/>
        </ol>        
    </xsl:template>

    <!-- unordered lists are rendered in HTML using ul elements -->
    <xsl:template match="ulist">
        <ul>
            <xsl:apply-templates/>
        </ul>
    </xsl:template>

    <!-- definition lists are rendered in HTML using dd elements -->
    <xsl:template match="dlist">
        <dl>
            <xsl:apply-templates/>
        </dl>
    </xsl:template>
    

    <!-- list items are rendered in HTML using li elements -->
    <xsl:template match="item[name(..) = 'olist' or name(..) = 'ulist']">
        <li><xsl:apply-templates/></li>
    </xsl:template>
    
    <!-- definition lists, item (has no representation in XHTML) -->
    <xsl:template match="item[name(..) = 'dlist']">
        <xsl:apply-templates/>
    </xsl:template>
    
    
    <!-- definition lists, phrase -->
    <xsl:template match="phrase">
        <dt><xsl:apply-templates/></dt>
    </xsl:template>
    
    <!-- definition lists, definition -->
    <xsl:template match="definition">
        <dd><xsl:apply-templates/></dd>
    </xsl:template>
    
    <!-- Code blocks -->
    <xsl:template match="code">
        <pre><xsl:value-of select="."/></pre>
    </xsl:template>

    <!-- ##################################################################################### -->    
    <!-- tables, images, floats -->
    
    <xsl:template match="table[caption]">
        <div class="table">
            <table>
                <xsl:apply-templates select="row"/>
            </table>
            <div class="caption"><xsl:apply-templates select="caption"/></div>
        </div>
    </xsl:template>
    
    <xsl:template match="table">
        <table>
            <xsl:if test="@iw:style">
                <xsl:attribute name="style"><xsl:value-of select="@iw:style"/></xsl:attribute>
            </xsl:if>
            <xsl:apply-templates/>
        </table>
    </xsl:template>
    
    <xsl:template match="row">
        <tr>
            <xsl:if test="@iw:style">
                <xsl:attribute name="style"><xsl:value-of select="@iw:style"/></xsl:attribute>
            </xsl:if>
            <xsl:apply-templates/>
        </tr>
    </xsl:template>
    
    <xsl:template match="col">
        <td>
            <xsl:if test="@iw:style">
                <xsl:attribute name="style"><xsl:value-of select="@iw:style"/></xsl:attribute>
            </xsl:if>
            <xsl:apply-templates/>
        </td>
    </xsl:template>

    <xsl:template match="head">
        <th>
            <xsl:if test="@iw:style">
                <xsl:attribute name="style"><xsl:value-of select="@iw:style"/></xsl:attribute>
            </xsl:if>
            <xsl:apply-templates/>
        </th>
    </xsl:template>
    
    <xsl:template match="iw:view">
        <xsl:copy-of select="./*"/>
    </xsl:template>
    
    <xsl:template match="iw:direct-type">
        <a class="direct-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>                                
        <xsl:if test="count(following-sibling::iw:indirect-type | following-sibling::iw:direct-type) > 0">
            -
        </xsl:if>                                
    </xsl:template>
    
    <xsl:template match="iw:indirect-type">
        <a class="indirect-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>
        <xsl:if test="count(following-sibling::iw:indirect-type | following-sibling::iw:direct-type) > 0">
            -
        </xsl:if>                                
    </xsl:template>
    
    <xsl:template match="iw:entity">
        <xsl:value-of select="@value" disable-output-escaping="yes"/>
    </xsl:template>

    <!-- all remaining nodes are assumed to be HTML and kept -->
    <xsl:template match="@*|node()">
      <xsl:copy>
        <xsl:apply-templates select="@*|node()"/>
      </xsl:copy>
    </xsl:template>
</xsl:stylesheet>