<?xml version="1.0" encoding="UTF-8"?>

<!--
    *  Copyright (C) 2006
    *  Sebastian Schaffert and Rupert Westenthaler
    *  Salzburg Research Forschungsgesellschaft m.b.H.
    *
    *  IkeWiki is free software; you can redistribute it and/or
    * 	modify it under the terms of the GNU Lesser General Public
    * 	License as published by the Free Software Foundation; either
    * 	version 2 of the License, or (at your option) any later version.
    *
    * 	This program is distributed in the hope that it will be useful,
    * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
    * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    * 	Lesser General Public License for more details.
    *
    * 	You should have received a copy of the GNU Lesser General Public
    * 	License along with this library; if not, write to the
    * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    * 	Boston, MA 02111-1307, USA.
    * 
-->

<!-- this is a stylesheet module intended for inclusion by other stylesheets; it contains
     template rules common to all transformations -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:fn="http://www.w3.org/2005/xpath-functions"
    xmlns:iw="http://ikewiki.srfg.at/syntax/1.0/ext"
    xmlns:xfm="http://www.w3.org/2002/xforms"
    xmlns:wiklet-cv="http://ikewiki.srfg.at/syntax/1.0/wiklet/cv"
    xmlns:wiklet-pv="http://ikewiki.srfg.at/syntax/1.0/wiklet/pv"
    xmlns:sparql="http://www.w3.org/2005/sparql-results#" 
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    xmlns:rxr="http://ilrt.org/discovery/2004/03/rxr/"
    xpath-default-namespace="http://ikewiki.srfg.at/syntax/1.0/core"
    exclude-result-prefixes="fn iw xfm wiklet-cv wiklet-pv sparql rxr" 
    version="2.0">


    <!-- ##################################################################################### -->
    <!-- line level elements -->
    
    <!-- emphasis -->
    <xsl:template match="emph">
        <i><xsl:apply-templates/></i>
    </xsl:template>

    <xsl:template match="strong">
        <b><xsl:apply-templates/></b>
    </xsl:template>

    <xsl:template match="verystrong">
        <b><i><xsl:apply-templates/></i></b>
    </xsl:template>

    <!-- links -->
    
    <!-- create link -->
    <xsl:template match="intlink[@create='true' or @create='yes']">
        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <a class="new"  lTarget="{@target}" name="link-{./text()}" href="index.jsp?action=edit_content&amp;title={@target}">
                    <xsl:apply-templates/>
                </a>
            </xsl:when>
            <xsl:otherwise>
                <a class="new"  lTarget="{@target}" name="link-{@target}" href="index.jsp?action=edit_content&amp;title={@target}">
                    <xsl:value-of select="@target"/>
                </a>                
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Wikipedia link (starts with Wikipedia:) -->
    <xsl:template match="intlink[starts-with(@target,'Wikipedia:')]">
        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <a class="external" href="http://en.wikipedia.org/wiki/{substring-after(@target,'Wikipedia:')}" lTarget="{@target}">
                    <xsl:apply-templates/>
                </a>                
            </xsl:when>
            <xsl:otherwise>
                <a class="external" href="http://en.wikipedia.org/wiki/{substring-after(@target,'Wikipedia:')}" lTarget="{@target}">
                    <xsl:value-of select="substring-after(@target,'Wikipedia:')"/>
                </a>                                
            </xsl:otherwise>
        </xsl:choose>        
    </xsl:template>
    
    <!-- image -->
    <xsl:template match="intlink[contains(@iw:typeURI,'http://ikewiki.srfg.at/base/ImageLink') or contains(@iw:typeURI,'http://ikewiki.srfg.at/IkeWiki#asImage')]">
        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <a  class="image" lTarget="{@target}" name="{./text()}" onmouseover="showWMTT('tt-{@target}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={@target}">
                    <img src="ImageDownload?title={@target}" alt="{./text()}" />
                </a>                
            </xsl:when>
            <xsl:otherwise>
                <a  class="image" lTarget="{@target}" name="{@target}" onmouseover="showWMTT('tt-{@target}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={@target}">
                    <img src="ImageDownload?title={@target}" alt="{@target}" />
                </a>                                
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    


    <!-- internal link -->
    <xsl:template match="intlink">
        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <a name="link-{./text()}" lTarget="{@target}" onmouseover="showWMTT('tt-{@target}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={@target}">
                    <xsl:apply-templates/>
                </a>                
            </xsl:when>
            <xsl:otherwise>
                <a name="link-{@target}" lTarget="{@target}" onmouseover="showWMTT('tt-{@target}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={@target}">
                    <xsl:value-of select="@target"/>
                </a>                                
            </xsl:otherwise>
        </xsl:choose>
        
        <!-- externalise widget -->
        <a class="add-type"  name="link-ext-{normalize-space(/page/iw:qtitle/text())}::{@target}" href="javascript:showExternaliseForm('{normalize-space(/page/iw:qtitle/text())}','{@target}');" onmouseover="showWMTTWithText('tt-ext-{@target}','Types: {@iw:type}')" onmouseout="hideWMTT()">
            <xsl:choose >
                <xsl:when test="not(@iw:type)">
                    <img src="skins/common/images/type_new.png"/>                                            
                </xsl:when>
                <xsl:otherwise>
                    <img src="skins/common/images/type_add.png"/>                                            
                </xsl:otherwise>
            </xsl:choose>
        </a>
    </xsl:template>

    <!-- external link -->
    <xsl:template match="extlink">
        <xsl:choose>
            <xsl:when test="count(./node()) > 0">
                <a class="external" href="{@href}"><xsl:apply-templates/></a>                
            </xsl:when>
            <xsl:otherwise>
                <a class="external" href="{@href}"><xsl:value-of select="@href"/></a>                                
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- newline -->
    <xsl:template match="newline">
        <br />
    </xsl:template>

    <!-- separator -->
    <xsl:template match="separator">
        <hr />
    </xsl:template>
    


    <!-- ##################################################################################### -->
    <!-- paragraph level elements -->


    <!-- headings are rendered using the HTML h2-h6 elemnents -->
    <xsl:template match="heading">
        
        <!-- insert a table of contents before the first heading; needs some refinement to support nesting-->
        <xsl:if test=". = //heading[1]">
            <table id='toc' class='toc'>
                <tr>
                    <td>
                        <div id='toctitle'><h2>Contents</h2></div>
                        <ul>
                            <xsl:for-each select="//heading[@level = 2]">
                                <li class="toclevel-{@level-1}">
                                    <xsl:variable name="level" select="@level"/>
                                    <a href="#_{translate(fn:string-join(./text(),' '),' ','_')}">
                                        <!-- TODO: needs refinement: -->
                                        <span class="tocnumber"><xsl:value-of select="count(./preceding-sibling::heading[@level = $level]) + 1"/></span>
                                        <span class="toctext"><xsl:apply-templates/></span>
                                    </a>
                                </li>
                            </xsl:for-each>
                        </ul>
                    </td>
                </tr>
            </table>
            <script type="text/javascript"> if (window.showTocToggle) { var tocShowText = "show"; var tocHideText = "hide"; showTocToggle(); } </script>            
        </xsl:if>
        
        <a name="_{translate(fn:string-join(./text(),' '),' ','_')}"/>
        
        <xsl:choose>
            <xsl:when test="@level='1'"><h2><xsl:apply-templates/></h2></xsl:when>
            <xsl:when test="@level='2'"><h3><xsl:apply-templates/></h3></xsl:when>
            <xsl:when test="@level='3'"><h4><xsl:apply-templates/></h4></xsl:when>
            <xsl:when test="@level='4'"><h5><xsl:apply-templates/></h5></xsl:when>
            <xsl:when test="@level='5'"><h6><xsl:apply-templates/></h6></xsl:when>
            <xsl:otherwise><h6><xsl:apply-templates/></h6></xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

    <!-- paragraphs are rendered in HTML as <p> elements -->
    <xsl:template match="paragraph[@iw:style]">
        <p style="{@iw:style}">
            <xsl:apply-templates/>
        </p>        
    </xsl:template>

    <xsl:template match="paragraph">
        <p>
            <xsl:apply-templates/>
        </p>        
    </xsl:template>
    
    <!-- blockquote -->
    <xsl:template match="quote">
        <blockquote>
            <xsl:apply-templates/>            
        </blockquote>
    </xsl:template>


    <!-- ordered lists are rendered in HTML using ol elements -->
    <xsl:template match="olist">
        <ol>
            <xsl:apply-templates/>
        </ol>        
    </xsl:template>

    <!-- unordered lists are rendered in HTML using ul elements -->
    <xsl:template match="ulist">
        <ul>
            <xsl:apply-templates/>
        </ul>
    </xsl:template>

    <!-- definition lists are rendered in HTML using dl elements -->
    <xsl:template match="dlist">
        <dl>
            <xsl:apply-templates/>
        </dl>
    </xsl:template>
    

    <!-- list items are rendered in HTML using li elements -->
    <xsl:template match="item[local-name(..) = 'olist' or local-name(..) = 'ulist']">
        <li><xsl:apply-templates/></li>
    </xsl:template>
    
    <!-- definition lists, item (has no representation in XHTML) -->
    <xsl:template match="item[local-name(..) = 'dlist']">
        <xsl:apply-templates/>
    </xsl:template>


    <!-- definition lists, phrase -->
    <xsl:template match="phrase">
        <dt><xsl:apply-templates/></dt>
    </xsl:template>

    <!-- definition lists, definition -->
    <xsl:template match="definition">
        <dd><xsl:apply-templates/></dd>
    </xsl:template>

    <!-- Code blocks -->
    <xsl:template match="code">
        <pre><xsl:value-of select="."/></pre>
    </xsl:template>
    
    

    <xsl:template match="iw:form">
        <table>
            <xsl:apply-templates select="xfm:input|xfm:select|xfm:select1|xfm:textarea"/>
        </table>        
    </xsl:template>
    
    
    <xsl:template match="xfm:input|xfm:select|xfm:select1|xfm:textarea">
        <xsl:variable name="name" select="@ref"/>
        <tr>
            <td valign="top"><xsl:apply-templates select="xfm:label"/>:</td>
            <td>
                <xsl:for-each select="fn:tokenize(//xfm:instance//*[name(.) = $name]/text(),'\n')">
                    <xsl:value-of select="."/>
                    <xsl:if test="position() != last()"><br /></xsl:if>
                </xsl:for-each>
                <!-- <xsl:value-of select="//xfm:instance//*[name(.) = $name]/text()"/> -->
            </td>
        </tr>
    </xsl:template>
    


    <xsl:template match="xfm:label">
        <strong><xsl:apply-templates/></strong>
    </xsl:template>
    
    
    <!-- ##################################################################################### -->    
    <!-- tables, images, floats -->
    
    <xsl:template match="table[caption]">
        <div class="table">
            <table>
                <xsl:copy-of select="@*"/>
                <xsl:apply-templates select="row"/>
            </table>
            <div class="caption"><xsl:apply-templates select="caption"/></div>
        </div>
    </xsl:template>
    
    <xsl:template match="table">
        <table>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </table>
    </xsl:template>
    
    <xsl:template match="row">
        <tr>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </tr>
    </xsl:template>
    
    <xsl:template match="col">
        <td>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </td>
    </xsl:template>

    <xsl:template match="head">
        <th>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </th>
    </xsl:template>
    
    
    <!-- ##################################################################################### -->    
    <!-- OWLClassWiklet -->
    <xsl:template match="iw:view[@wiklet='CreativeCommonsWiklet']">
        
        <div class="wiklet">
            <hr />
            This content is distributed under a Creative Commons license.
            
            <xsl:apply-templates select="iw:license"/>
        </div>
    </xsl:template>
    
    <xsl:template match="iw:license">
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="iw:noncommercial">
        <img src="skins/common/images/cc/noncomm.gif" alt="non-commercial"/>
    </xsl:template>

    <xsl:template match="iw:attribution">
        <img src="skins/common/images/cc/attrib.gif" alt="attribution"/>
    </xsl:template>

    <xsl:template match="iw:sharealike">
        <img src="skins/common/images/cc/standard.gif" alt="sharealike"/>
    </xsl:template>

    <xsl:template match="iw:no-derivative">
        <img src="skins/common/images/cc/nomod.gif" alt="no derivative"/>
    </xsl:template>
    
    
    <!-- ##################################################################################### -->    
    <!-- Wiklet data (page types, domain, range, ...) -->
    
    <xsl:template match="iw:view">
        <xsl:apply-templates/>
<!--        <xsl:copy-of select="./*"/> -->
    </xsl:template>
    
    <xsl:template match="iw:type[@direct = 'true'] | iw:superclass[@direct = 'true'] | iw:superproperty[@direct = 'true']">
        <xsl:variable name="myname" select="name(.)"/>
        <a class="direct-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>                                
        <!-- add separators if there is a following element with the same name -->
        <xsl:if test="count(following-sibling::*[name(.) = $myname]) > 0"> - </xsl:if>                                
    </xsl:template>
    
    <xsl:template match="iw:type | iw:superclass | iw:superproperty">
        <xsl:variable name="myname" select="name(.)"/>
        <a class="indirect-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>
        <!-- add separators if there is a following element with the same name -->
        <xsl:if test="count(following-sibling::*[name(.) = $myname]) > 0"> - </xsl:if>                                
    </xsl:template>
    
    
    
    <xsl:template match="iw:entity">
        <xsl:choose>
            <xsl:when test="@value = 'nbsp'"><xsl:text> </xsl:text></xsl:when>
            <xsl:otherwise>
                <xsl:text disable-output-escaping="yes">&amp;</xsl:text><xsl:value-of select="@value" disable-output-escaping="yes"/>;                
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="iw:range">
        <a class="direct-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>                                
        <xsl:if test="count(following-sibling::iw:range) > 0">
            +
        </xsl:if>                                
    </xsl:template>
    
    <xsl:template match="iw:domain">
        <a class="direct-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>                                
        <xsl:if test="count(following-sibling::iw:domain) > 0">
            +
        </xsl:if>                                
    </xsl:template>

    <xsl:template match="iw:inverseOf">
        <a class="direct-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
            <xsl:apply-templates/>
        </a>                                
        <xsl:if test="count(following-sibling::iw:inverseOf) > 0">
            +
        </xsl:if>                                
    </xsl:template>
    
    <!-- ##################################################################################### -->    
    <!-- OWLClassWiklet -->
    <xsl:template match="iw:view[@wiklet='OWLClassWiklet']">
        
        <div class="wiklet">
        <hr />
        
        <div>
            This resource describes a class, i.e. a collection of individuals. The following properties are defined for individuals in this class:
        </div>
        
        <table width="100%">
            <tr>
                <td width="50%" valign="top">
                    <strong>instances of <xsl:value-of select="/page/title"/> are targets of</strong> (<a name="rdfs:range" onmouseover="showWMTT('tt-rdfs:range')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title=rdfs:range">rdfs:range</a>):
                    <ul>
                        <xsl:if test="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'owl:ObjectProperty']">
                            <li>
                                <i>ObjectProperties</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'owl:ObjectProperty']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'owl:DatatypeProperty']">
                            <li>
                                <i>DatatypeProperties</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'owl:DatatypeProperty']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'owl:AnnotationProperty']">
                            <li>
                                <i>AnnotationProperties</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'owl:AnnotationProperty']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'rdf:Property']">
                            <li>
                                <i>Others</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='incoming' and iw:type[@direct='true']/text() = 'rdf:Property']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <!--                       <xsl:apply-templates select="wiklet-cv:property[@type='incoming']"/> -->
                    </ul>
                </td>
                <td width="50%"  valign="top">
                    <strong>instances of <xsl:value-of select="/page/title"/> are sources of</strong> (<a name="rdfs:domain" onmouseover="showWMTT('tt-rdfs:domain')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title=rdfs:domain">rdfs:domain</a>):
                    <ul>
                        <xsl:if test="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'owl:ObjectProperty']">
                            <li>
                                <i>ObjectProperties</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'owl:ObjectProperty']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'owl:DatatypeProperty']">
                            <li>
                                <i>DatatypeProperties</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'owl:DatatypeProperty']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'owl:AnnotationProperty']">
                            <li>
                                <i>AnnotationProperties</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'owl:AnnotationProperty']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'rdf:Property']">
                            <li>
                                <i>Others</i>:
                                <ul>
                                    <xsl:apply-templates select="wiklet-cv:property[@type='outgoing' and iw:type[@direct='true']/text() = 'rdf:Property']"/>                                
                                </ul>
                            </li>
                        </xsl:if>
                    </ul>
                </td>
            </tr>
        </table>
        </div>
        
    </xsl:template>
    
    <xsl:template match="wiklet-cv:property">
        <li>
            <a name="{normalize-space(@name)}" onmouseover="showWMTT('tt-{normalize-space(@name)}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(@name)}">
                <xsl:value-of select="@name"/>
            </a>
            <xsl:text> </xsl:text><span class="typeContent">(Types: <xsl:apply-templates select="iw:type[@direct='true'] | iw:type[@direct='false']"/>)</span>
        </li>        
    </xsl:template>

    <!-- ##################################################################################### -->    
    <!-- OWLPropertyWiklet -->
    <xsl:template match="iw:view[@wiklet='OWLPropertyWiklet']">
        <div class="wiklet">
            <hr />
            
            <div>
                This resource describes a class of properties between resources. Currently, instances of the following classes are applicable to this property:
            </div>
            
            <table width="100%">
                <tr>
                    <th width="50%">as source (subject)</th>
                    <th width="50%">as target (object)</th>
                </tr>
                <tr>
                    <td>
                        <ul>
                            <xsl:apply-templates select="wiklet-pv:subject-type"/>
                        </ul>
                    </td>
                    <td>
                        <ul>
                            <xsl:apply-templates select="wiklet-pv:object-type"/>
                        </ul>
                    </td>
                </tr>
            </table>

        </div>            
    </xsl:template>        


    <xsl:template match="wiklet-pv:subject-type | wiklet-pv:object-type">
        <li>
            <xsl:choose>
                <xsl:when test="@direct = 'yes' or @direct = 'true'">
                    <a  name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
                        <xsl:apply-templates/>                        
                    </a>            
                </xsl:when>
                <xsl:otherwise>
                    <a class="indirect-type" name="{normalize-space(./text())}" onmouseover="showWMTT('tt-{normalize-space(./text())}')" onmouseout="hideWMTT()" href="index.jsp?action=view&amp;title={normalize-space(./text())}">
                        <xsl:apply-templates/>                        
                    </a>                                
                </xsl:otherwise>
            </xsl:choose>
        </li>
    </xsl:template>
    
    <!-- ##################################################################################### -->    
    <!-- SPARQL Query result -->
    <xsl:template match="iw:sparql">
        <xsl:apply-templates select="sparql:sparql"/>
    </xsl:template>
    
    <xsl:template match="sparql:sparql">
        <xsl:apply-templates select="sparql:results"/>
    </xsl:template>
    
    <xsl:template match="sparql:results">
        <xsl:choose>
            <xsl:when test="count(*) = 1">
                <xsl:for-each select="sparql:result">
                    <xsl:apply-templates select="sparql:binding[position() = 1]"/>
                </xsl:for-each>                
            </xsl:when>
            
            <xsl:otherwise>
                <ul>
                    <xsl:for-each select="sparql:result">
                        <li><xsl:apply-templates select="sparql:binding[position() = 1]"/></li>
                    </xsl:for-each>                
                </ul>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
        
    <xsl:template match="sparql:binding">
        <xsl:apply-templates/>
    </xsl:template>        
        
    <xsl:template match="sparql:unbound">
        <i>-</i>
    </xsl:template>
    
    <xsl:template match="sparql:literal">
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="sparql:uri">
        <a href="index.jsp?action=view&amp;uri={fn:replace(./text(),'#','%23')}"><xsl:value-of select="@lName"/></a>
    </xsl:template>
    
    <xsl:template match="sparql:bnode">
        <i><xsl:apply-templates/></i>
    </xsl:template>
            
    
    
    <!-- ##################################################################################### -->    
    <!-- all remaining nodes are assumed to be HTML and kept -->
    <xsl:template match="@*|node()">
      <xsl:copy>
        <xsl:apply-templates select="@*|node()"/>
      </xsl:copy>
    </xsl:template>

    
</xsl:stylesheet>