/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.widget;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Set;

import org.apache.log4j.Logger;

import at.srfg.ikewiki.render.elements.WikiElement;

public abstract class AbstractWidget implements Widget {
	
	Set<String> types;
	static Logger log = Logger.getLogger(AbstractWidget.class); 
	
	public AbstractWidget(Set<String> types){
		setTypes(types);
	}
	public final void doPre(WikiElement data) {
		if(!callSpecific(WidgetPos.pre,data)){
			log.debug("do nothing for [pre|"+data.getClass().getName()+"|"+data.getPageResource().getQualifiedTitle()+"]");
		}
	}
	public final void doStart(WikiElement data) {
		if(!callSpecific(WidgetPos.start,data)){
			log.debug("do nothing for [start|"+data.getClass().getName()+"|"+data.getPageResource().getQualifiedTitle()+"]");
		}
	}
	public final void doEnd(WikiElement data) {
		if(!callSpecific(WidgetPos.end,data)){
			log.debug("do nothing for [end|"+data.getClass().getName()+"|"+data.getPageResource().getQualifiedTitle()+"]");
		}
	}
	public final void doPost(WikiElement data) {
		if(!callSpecific(WidgetPos.post,data)){
			log.debug("do nothing for [post|"+data.getClass().getName()+"|"+data.getPageResource().getQualifiedTitle()+"]");
		}
	}

	public Set<String> getTypes() {
		return types;
	}
	protected final void setTypes(Set<String> types) {
		if(types == null){
			throw new IllegalArgumentException("Null value is not allowed for the Set<String> containing the Type-registration of the Widget");
		}
		this.types = types;
	}
	
	private boolean callSpecific(WidgetPos pos, WikiElement data){
		try {
			log.debug("Search for Methode "+pos.getMethodName()+" for "+data.getClass());
			Method m = this.getClass().getMethod(pos.getMethodName(), new Class[]{data.getClass()});
			log.debug("   ... got Methode: "+m);
			m.invoke(this, new Object[]{data});
			log.debug("    >> success <<");
			return true;
		} catch (SecurityException e) {
			log.warn("unable to search for specific Implementation for doEnd("+data.getClass()+")",e);
		} catch (NoSuchMethodException e) {
			// ignore
			// --> no specific Implementation Available
		} catch (IllegalArgumentException e) {
			log.warn("unable to call more specific Implementation for doEnd("+data.getClass()+")",e);
		} catch (IllegalAccessException e) {
			log.warn("unable to call more specific Implementation for doEnd("+data.getClass()+")",e);
		} catch (InvocationTargetException e) {
			log.warn("unable to call more specific Implementation for doEnd("+data.getClass()+")",e);
		}
		return false;
	}

}
