/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.widget;

import java.util.Set;

import at.srfg.ikewiki.render.elements.WikiElement;
/**
 * Widgets can be used to add Elements to an XHTML Page. They are called by the
 * Renderer of a Page if Elements of an specific type get rendert.<br>
 * Widgets can add XHTML Fragments in front of and after such elements.
 * The envisioned usage of widgets is to add interactive elements for
 * specific Elements.
 * @author rwesten
 */
public interface Widget {

	/**
	 * Use this Methode to add XHTML Syntax before the actual Element
	 * @param data The data for the Widget
	 */
	void doPre(WikiElement data);
	/**
	 * Use this Methode to add XHTML Syntax after the actual Element
	 * @param data The Data for the Widget
	 */
	void doPost(WikiElement data);
	/**
	 * Use this Methode to add XHTML Syntax at the start position inside the
	 * Element
	 * @param data The Data for the Widget
	 */
	void doStart(WikiElement data);
	/**
	 * Use this Methode to add XHTML Syntax after the content of the Element
	 * @param data The Data for the Widget
	 */
	void doEnd(WikiElement data);
	/**
	 * Returns the supported types of that widget
	 * @return A list with oll supported Types
	 */
	Set<String> getTypes();
}
