/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import java.util.Collection;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.layout.views.View;

/**
 * A Wiklet that adds views to the content of the page
 * 
 * @author wastl
 *
 */
public class AddContentViewsWiklet extends AbstractWiklet implements Wiklet {

	/**
	 * @param main
	 */
	public AddContentViewsWiklet(Main main) {
		super(main);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document, at.srfg.ikewiki.data.WikiResource)
	 */
	public Document apply(Document d, WikiResource context) {
		Collection<View> v_before_content = main.getViewsForResource(context,View.TYPES.CONTENT_START);
		Collection<View> v_after_content  = main.getViewsForResource(context,View.TYPES.CONTENT_END);
		
		for(View v : v_before_content) {
			Element view = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"view");
			view.setAttribute("position","before");
			d.getDocumentElement().appendChild(view);
			
			view.appendChild(v.createView(context,d));
		}
		

		for(View v : v_after_content) {
			Element view = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"view");
			view.setAttribute("position","after");
			d.getDocumentElement().appendChild(view);
			
			view.appendChild(v.createView(context,d));
		}

		return d;
	}

}
