/**
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiAnonResource;
import at.srfg.ikewiki.data.WikiLink;
import at.srfg.ikewiki.data.WikiLiteral;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * This Wiklet adds the RDF triple context of a resource to the Wiki interchange format document.
 * RDF data is wrapped inside a iw:context element using RXR triple syntax.
 * 
 * Currently, this data is used by the "references" transformation (transform-references.xsl) and
 * in the wiki exchange download. 
 * 
 * @see Namespace.NS_RXR
 * 
 * @author sschaffe
 *
 */
public class AddContextWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(AddContextWiklet.class);

	/**
	 * @param main
	 */
	public AddContextWiklet(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document, at.srfg.ikewiki.data.WikiResource)
	 */
	public Document apply(Document d, WikiResource context) {
		if(d.getDocumentElement().getLocalName().equals("page")) {
			try {
				
				Collection<WikiLink> incoming = main.getStorageBackend().getTriplesByObject(context);
				Collection<WikiLink> outgoing = main.getStorageBackend().getTriplesBySubject(context);
				
				
				Element ctx = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"context");
				d.getDocumentElement().appendChild(ctx);
				
				
				Element graph = d.createElementNS(Namespace.NS_RXR,"graph");
				ctx.appendChild(graph);
				
				for(WikiLink l : incoming) {
					Element triple = d.createElementNS(Namespace.NS_RXR,"triple");
					triple.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"type","incoming");
					graph.appendChild(triple);
					
					Element subject = d.createElementNS(Namespace.NS_RXR,"subject");
					subject.setAttribute("uri",l.getSource().getUri());
					subject.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"qname",l.getSource().getQualifiedTitle());
					triple.appendChild(subject);
					

					Element predicate = d.createElementNS(Namespace.NS_RXR,"predicate");
					predicate.setAttribute("uri",l.getProperty().getUri());
					predicate.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"qname",l.getProperty().getQualifiedTitle());
					triple.appendChild(predicate);

					Element object = d.createElementNS(Namespace.NS_RXR,"object");
					object.setAttribute("uri",context.getUri());
					object.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"qname",context.getQualifiedTitle());
					triple.appendChild(object);
					
//					el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"type",l.getProperty().getQualifiedTitle());
//					el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"typeURI",l.getProperty().getUri());
//					el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"source",l.getSource().getQualifiedTitle());
//					el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"sourceURI",l.getSource().getUri());
				}
				
				for(WikiLink l : outgoing) {
					if(!l.getProperty().getUri().equals("http://ikewiki.srfg.at/IkeWiki#article-xml")
					   && !l.getProperty().getUri().equals("http://ikewiki.srfg.at/IkeWiki#article-structured")	) {
					
						Element triple = d.createElementNS(Namespace.NS_RXR,"triple");
						triple.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"type","outgoing");
						graph.appendChild(triple);
						
						Element subject = d.createElementNS(Namespace.NS_RXR,"subject");
						subject.setAttribute("uri",context.getUri());
						subject.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"qname",context.getQualifiedTitle());
						triple.appendChild(subject);
						
						
						Element predicate = d.createElementNS(Namespace.NS_RXR,"predicate");
						predicate.setAttribute("uri",l.getProperty().getUri());
						predicate.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"qname",l.getProperty().getQualifiedTitle());
						triple.appendChild(predicate);
						
						Element object = d.createElementNS(Namespace.NS_RXR,"object");
						if(l.getTarget().isResource()) {
							WikiResource r = (WikiResource)l.getTarget();
							object.setAttribute("uri",r.getUri());
							object.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"qname",r.getQualifiedTitle());
						} else if(l.getTarget().isAnonymous()) {
							WikiAnonResource r = (WikiAnonResource)l.getTarget();
							object.setAttribute("blank",r.getId());
						} else if(l.getTarget().isLiteral()) {
							WikiLiteral r = (WikiLiteral)l.getTarget();
							object.setTextContent(r.getContent());
						}
						triple.appendChild(object);
					}
					
					
//					Element el = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"reference");
//					el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"type",l.getProperty().getQualifiedTitle());
//					el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"typeURI",l.getProperty().getUri());
//					if(l.getTarget().isResource()) {
//						el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"targetType","resource");
//						el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"target",((WikiResource)l.getTarget()).getQualifiedTitle());
//						el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"targetURI",((WikiResource)l.getTarget()).getUri());
//					} else if(l.getTarget().isAnonymous()) {
//						el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"targetType","anonymous");
//						el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"targetAnonID",((WikiAnonResource)l.getTarget()).getId());					
//					} else if(l.getTarget().isLiteral()) {
//						el.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"targetType","literal");
//						el.appendChild(d.createTextNode(((WikiLiteral)l.getTarget()).getContent()));
//					}
//					ctx.appendChild(el);
				}
				
				
			} catch(DBException ex) {
				log.error("error while retrieving context of resource "+context.getQualifiedTitle(),ex);
			}
		}
		return d;
	}

}
