/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;
import at.srfg.ikewiki.render.RenderingPipeline;

/**
 * Add the domain and range for resources that have them.
 * 
 * @author sschaffe
 *
 */
public class AddDomainRangeWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(AddDomainRangeWiklet.class);
	/**
	 * @param main
	 */
	public AddDomainRangeWiklet(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document, at.srfg.ikewiki.data.WikiResource)
	 */
	public Document apply(Document d, WikiResource context) {
		Element page = d.getDocumentElement();
		
		try {
			Collection<WikiResource> range  = main.getStorageBackend().getRange(context);
			Collection<WikiResource> domain = main.getStorageBackend().getDomain(context);
	
			for(WikiResource t : range) {
				Element elem = d.createElementNS(RenderingPipeline.NS_IKEWIKI_EXT,"range");
				elem.appendChild(d.createTextNode(t.getQualifiedTitle()));
				page.appendChild(elem);
			}					
					
			for(WikiResource t : domain) {
				Element elem = d.createElementNS(RenderingPipeline.NS_IKEWIKI_EXT,"domain");
				elem.appendChild(d.createTextNode(t.getQualifiedTitle()));
				page.appendChild(elem);					
			}
			
		} catch(DBException ex) {
			log.error("error determining range/domain information for page "+context.getQualifiedTitle(),ex);
		} catch(NullPointerException ex) {
			// happens when page does not exist, we should check for that probably!
			log.error("error determining type information for page "+context.getQualifiedTitle()+"; maybe page does not exist?",ex);
		}
		return d;
	}

}
