/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * Verify for each link in the document whether it already exists in the Wiki. If no, add a "create"
 * attribute to it.
 * 
 * TODO: Rename to ExistsLinkWiklet
 * 
 * @author sschaffe
 *
 */
public class AddLinkTypesWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(AddLinkTypesWiklet.class);
	
	
	/**
	 * 
	 */
	public AddLinkTypesWiklet(Main main) {
		super(main);
		
	}

	/**
	 * 
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document)
	 */
	public Document apply(Document d, WikiResource context) {
		NodeList nl = d.getElementsByTagName("intlink");
		
		for(int i=0; i<nl.getLength(); i++) {
			Element link = (Element)nl.item(i);
			
			// check whether the target of the link exists
			if(link.getAttribute("target") != null && !link.getAttribute("target").equals("")) {
				
				Collection<WikiResource> types = getTypes(link.getAttribute("target"),context);
				
				if(!types.isEmpty()) {
					// one or more types for the link exist, add an "iw:type" attribute
					StringBuilder sb = new StringBuilder();
					StringBuilder usb = new StringBuilder();
					for( Iterator<WikiResource> it = types.iterator() ; it.hasNext() ; ) {
						WikiResource ntype = it.next();
						
						// do not count IkeWiki navigational links
						if(!ntype.getUri().equals(Namespace.NS_IW+"NavigationalLink")) {
							sb.append(ntype.getQualifiedTitle());
							usb.append(ntype.getUri());
							if(it.hasNext()) {
								sb.append(' ');
								usb.append(' ');
							}
						}
					}
					
					if(sb.length() > 0) {
						link.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"type",sb.toString());
						link.setAttributeNS(Namespace.NS_IKEWIKI_EXT,"typeURI",usb.toString());
					}
						
				} else {
					// the link is untyped, skip
				}
			}
		}
		
		return d;
	}


	
	private Collection<WikiResource> getTypes(String name, WikiResource context) {
		try {
			return main.getStorageBackend().getPropertiesBySubjectObject(context,main.getResourceByQTitle(name));
		} catch(DBException ex) {
			log.error(ex);
			return Collections.emptyList();
		}
		
	}
}
