/**
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiNode;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * Display license information in case CC metadata is given.
 * 
 * @author wastl
 *
 */
public class CreativeCommonsWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(CreativeCommonsWiklet.class);

	private static final String NS_CC = "http://web.resource.org/cc/";
	private WikiResource r_cc_work, r_cc_permits, r_cc_requires, r_cc_prohibits, r_cc_license, r_cc_attribution, r_cc_derivative, r_cc_sharealike, r_cc_commercial;
	
	
	/**
	 * @param main
	 */
	public CreativeCommonsWiklet(Main main) {
		super(main);
		
		try {
			r_cc_work        = main.getResourceByUri(NS_CC+"Work");
			r_cc_attribution = main.getResourceByUri(NS_CC+"Attribution");
			r_cc_derivative  = main.getResourceByUri(NS_CC+"DerivativeWorks");
			r_cc_sharealike  = main.getResourceByUri(NS_CC+"ShareAlike");
			r_cc_commercial  = main.getResourceByUri(NS_CC+"CommercialUse");
			
			r_cc_permits   = main.getResourceByUri(NS_CC+"permits");
			r_cc_requires  = main.getResourceByUri(NS_CC+"requires");
			r_cc_prohibits = main.getResourceByUri(NS_CC+"prohibits");
			r_cc_license   = main.getResourceByUri(NS_CC+"license");
		} catch(DBException ex) {
			log.error(ex);
		}
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document, at.srfg.ikewiki.data.WikiResource)
	 */
	public Document apply(Document d, WikiResource context) {
		try {
			Collection<WikiResource> types = main.getStorageBackend().getTypes(context);
			
			if(types.contains(r_cc_work)) {
				Element e_view = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"view");
				e_view.setAttribute("position","after");
				e_view.setAttribute("wiklet","CreativeCommonsWiklet");

				Element e_license = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"license");
				d.getDocumentElement().appendChild(e_view);
				e_view.appendChild(e_license);
				
				for(WikiNode _license : main.getStorageBackend().getObjectsBySubjectProperty(context,r_cc_license)) {
					if(_license.isResource()) {
						WikiResource license = (WikiResource)_license;
						
						if(main.getStorageBackend().hasTriple(license,r_cc_prohibits,r_cc_commercial)) {
							Element e_noncommercial = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"noncommercial");
							e_license.appendChild(e_noncommercial);
						}
						
						if(main.getStorageBackend().hasTriple(license,r_cc_requires,r_cc_attribution)) {
							Element e_attribution = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"attribution");
							e_license.appendChild(e_attribution);							
						}
						
						if(main.getStorageBackend().hasTriple(license,r_cc_requires,r_cc_sharealike)) {
							Element e_sharealike = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"sharealike");
							e_license.appendChild(e_sharealike);							
						}

						if(!main.getStorageBackend().hasTriple(license,r_cc_permits,r_cc_derivative)) {
							Element e_derivative = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"no-derivative");
							e_license.appendChild(e_derivative);							
						}
					}
				}
				
			}
			
		} catch(DBException ex) {
			log.error(ex);
		}
		return d;
	}

}
