/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;

/**
 * Verify for each link in the document whether it already exists in the Wiki. If no, add a "create"
 * attribute to it.
 * 
 * TODO: Rename to ExistsLinkWiklet
 * 
 * @author sschaffe
 *
 */
public class ExistsLinkWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(ExistsLinkWiklet.class);
	
	/**
	 * 
	 */
	public ExistsLinkWiklet(Main main) {
		super(main);
		
	}

	/**
	 * Apply the ExistsLinkWiklet. This method will iterate through all "intlink" elements in the document,
	 * check whether the link exists, and if no, add a "create" attribute to the link.
	 * 
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document)
	 */
	public Document apply(Document d, WikiResource context) {
		NodeList nl = d.getElementsByTagNameNS(Namespace.NS_IKEWIKI_CORE,"intlink");
		
		for(int i=0; i<nl.getLength(); i++) {
			Element link = (Element)nl.item(i);
			
			// check whether the target of the link exists
			if(link.getAttribute("target") != null && !link.getAttribute("target").equals("")) {
				if(exists(link.getAttribute("target"))) {
					// the target link exists, for the moment skip
				} else {
					// the target link does not exist, add a "create" attribute and set it to true
					link.setAttribute("create","true");
				}
			}
		}
		
		return d;
	}

	
	/**
	 * Check whether a certain page exists.
	 * 
	 */
	public boolean exists(String title) {
 		String[] components = title.replace(" ","_").split(":");
 		String article_ns, article_title;
 		if(components.length == 2) {
	 		article_ns    = components[0];
	 		article_title = components[1];
	 	} else if(components.length == 1) {
	 		article_ns    = "";
	 		article_title = components[0];
	 	} else {
	 		article_ns    = "";
	 		article_title = "";	 		
	 	}
		
 		if(title.startsWith("Wikipedia:")) {
 			return true;
 		} else {
 			try {
 				boolean exists = main.getStorageBackend().existsArticleByNamespaceTitle(article_ns,article_title);
 				
 				if(!exists) {
 					exists = main.getStorageBackend().existsImageByNamespaceTitle(article_ns,article_title);
 				}
 				// log.info("check link "+title+": "+exists);
 				
 				return exists;
 			} catch(Exception ex) {
 				// some more serious error occurred, we do not handle it here
 				ex.printStackTrace();
 				return false;
 			}
 		}
	}

}
