/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.Namespace;
import at.srfg.ikewiki.data.WikiResource;
import at.srfg.ikewiki.db.DBException;

/**
 * The OWLPropertyWiklet adds functionality to the rendering of OWL/RDF properties. Currently,
 * it displays for each property the classes whose instances may be used as subject or object
 * in statements with this property.
 * 
 * @author sschaffe
 *
 */
public class OWLPropertyWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(OWLPropertyWiklet.class);

	/**
	 * @param main
	 */
	public OWLPropertyWiklet(Main main) {
		super(main);
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document, at.srfg.ikewiki.data.WikiResource)
	 */
	public Document apply(Document d, WikiResource context) {
		try {
			if(main.getStorageBackend().isObjectProperty(context)) {
				Element view = d.createElementNS(Namespace.NS_IKEWIKI_EXT,"view");
				view.setAttribute("position","after");
				view.setAttribute("wiklet","OWLPropertyWiklet");
				d.getDocumentElement().appendChild(view);
				
				
				Collection<WikiResource> subject_types = main.getStorageBackend().getSubjectTypes(context);
				Collection<WikiResource> object_types  = main.getStorageBackend().getObjectTypes(context);
				
				for(WikiResource subject_type : subject_types) {
					Element e = d.createElementNS(Namespace.NS_IKEWIKI_WIKLET_PV,"subject-type");
					e.setAttribute("uri",subject_type.getUri());
					e.setAttribute("direct",subject_type.isImplicit()?"no":"yes");
					e.appendChild(d.createTextNode(subject_type.getQualifiedTitle()));
					view.appendChild(e);
				}

				for(WikiResource object_type : object_types) {
					Element e = d.createElementNS(Namespace.NS_IKEWIKI_WIKLET_PV,"object-type");
					e.setAttribute("uri",object_type.getUri());
					e.setAttribute("direct",object_type.isImplicit()?"no":"yes");
					e.appendChild(d.createTextNode(object_type.getQualifiedTitle()));
					view.appendChild(e);
				}

			}
			// TODO: other kinds of properties
		} catch(DBException ex) {
			log.error(ex);
		}
		return d;
	}

}
