/**
 *  Copyright (C) 2005
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 */
package at.srfg.ikewiki.render.wiklet;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;

import at.srfg.ikewiki.Main;
import at.srfg.ikewiki.data.WikiResource;

/**
 * @author wastl
 *
 */
public class XSLTWiklet extends AbstractWiklet implements Wiklet {

	private static Log log = LogFactory.getLog(XSLTWiklet.class);

	private Transformer stylesheet;
	private DocumentBuilder dombuilder;
	
	/**
	 * 
	 */
	public XSLTWiklet(Main main, String template) {
		super(main);
		
		StreamSource source = new StreamSource(this.getClass().getResourceAsStream(template));
		
		try {
			stylesheet = TransformerFactory.newInstance().newTransformer(source);
		} catch(TransformerConfigurationException ex) {
			log.error("error creating XSLT wiklet "+template,ex);
			stylesheet = null;
		}
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance(); 
		factory.setNamespaceAware(true);
		
		try {
			dombuilder = factory.newDocumentBuilder();
		} catch(ParserConfigurationException ex) {
			log.error("error creating document builder for XSLT wiklet "+template,ex);
		}
	}

	/* (non-Javadoc)
	 * @see at.srfg.ikewiki.render.wiklet.Wiklet#apply(org.w3c.dom.Document)
	 */
	public Document apply(Document d, WikiResource context) {
		if(stylesheet != null && dombuilder != null) {

			DOMSource ds = new DOMSource(d);
			ds.setSystemId(d.getDocumentURI()!=null?d.getDocumentURI():"http://ikewiki.srfg.at/Internal");
			
			Document result = dombuilder.newDocument();
			result.setDocumentURI(d.getDocumentURI());
			
			DOMResult dr = new DOMResult(result);
			
			try {
				stylesheet.transform(ds,dr);

				return result;
			} catch(TransformerException ex) {
				log.error("error executing XSLT wiklet",ex);
				return d;
			}
		} else {
			return d;
		}
	}

}
