/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 *  File: Transaction.java
 */
package at.srfg.ikewiki.transactions;

import java.util.LinkedList;
import java.util.List;

import com.hp.hpl.jena.rdf.model.Statement;

/**
 * @author wastl
 *
 */
public class Transaction {

	int id, activity_count;

	List<TransactionItem> items;
	
	/**
	 * 
	 */
	public Transaction() {
		this(-1);
	}

	/**
	 * @param id
	 */
	public Transaction(int id) {
		this.id = id;
		activity_count = 1;
		items = new LinkedList<TransactionItem>();
	}
	
	
	public void increaseActivityCount() {
		activity_count++;
	}
	
	public void decreaseActivityCount() {
		activity_count--;
	}
	
	public int getActivityCount() {
		return activity_count;
	}
	
	
	public void addTransactionItem(TransactionItem ti) {
		items.add(ti);
	}
	
	
	public void addAddition(Statement stmt) {
		addTransactionItem(new TransactionItem(stmt,TransactionItem.KINDS.ADD));
	}
	
	public void addRemoval(Statement stmt) {
		addTransactionItem(new TransactionItem(stmt,TransactionItem.KINDS.REMOVE));
	}
	

	public List<TransactionItem> getTransactionItems() {
		return items;
	}
	
	public List<Statement> getStatements(TransactionItem.KINDS kind) {
		LinkedList<Statement> result = new LinkedList<Statement>();
		for(TransactionItem ti : items) {
			if(ti.getKind() == kind) {
				result.add(ti.getStatement());
			}
		}
		return result;
	}
}
