/**
 *  Copyright (C) 2006
 *  Sebastian Schaffert and Rupert Westenthaler
 *  Salzburg Research Forschungsgesellschaft m.b.H.
 *
 *  IkeWiki is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU Lesser General Public
 * 	License as published by the Free Software Foundation; either
 * 	version 2 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * 	Lesser General Public License for more details.
 *
 * 	You should have received a copy of the GNU Lesser General Public
 * 	License along with this library; if not, write to the
 * 	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * 	Boston, MA 02111-1307, USA.
 * 
 *  File: TransactionItem.java
 */
package at.srfg.ikewiki.transactions;

import java.util.Date;

import at.srfg.ikewiki.data.User;

import com.hp.hpl.jena.rdf.model.Statement;

/**
 * TransactionItem is a wrapper for a Jena statement, storing additional information about the statement
 * (currently only whether it was an addition or a removal).
 * 
 * @author wastl
 *
 */
public class TransactionItem {

	public static enum KINDS {
		ADD, REMOVE
	};
	
	private Statement stmt;
	private KINDS kind;
	private User user;
	private Date timestamp;
	
	public TransactionItem(Statement stmt, KINDS kind) {
		this(stmt,kind,null,new Date());
	}
		
	public TransactionItem(Statement stmt, KINDS kind, User user, Date timestamp) {
		this.stmt      = stmt;
		this.kind      = kind;
		this.user      = user;
		this.timestamp = timestamp;
	}

	
	
	
	public KINDS getKind() {
		return kind;
	}

	public Statement getStatement() {
		return stmt;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public User getUser() {
		return user;
	}
	
	

}
