/**
 * 
 */
package at.srfg.semweb.reasoner;

import java.util.LinkedList;
import java.util.List;

import jpl.JPL;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import com.hp.hpl.jena.graph.BulkUpdateHandler;
import com.hp.hpl.jena.graph.Capabilities;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.GraphEventManager;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Reifier;
import com.hp.hpl.jena.graph.TransactionHandler;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.graph.TripleMatch;
import com.hp.hpl.jena.graph.query.QueryHandler;
import com.hp.hpl.jena.rdf.model.AnonId;
import com.hp.hpl.jena.shared.AddDeniedException;
import com.hp.hpl.jena.shared.DeleteDeniedException;
import com.hp.hpl.jena.shared.PrefixMapping;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.util.iterator.WrappedIterator;

/**
 * @author wastl
 *
 */
public class PrologGraph implements Graph {

	private static Log log = LogFactory.getLog(PrologGraph.class);

	/**
	 * Path to the Prolog interpreter; used by InterProlog.
	 */
	private String prologPath;
	
	/**
	 * the Prolog engine used by this model.
	 */
	private JPL engine;
	
	/**
	 * 
	 */
	public PrologGraph(String prologPath) {
		super();
		this.prologPath = prologPath;
		
		initProlog();
	}

	
	private void initProlog() {
		
		//System.setProperty("java.library.path",this.prologPath);
		System.loadLibrary("jpl");

		engine.init();		
		log.info("SWI Prolog initialised");
		
		
		
//		engine.consultFromPackage("reasoner.pl",this);
//		log.info("reasoner initialised");
	}

	
	private static Node createNode(String prologAtom) {
		// anonymous nodes start with "__"
		if(prologAtom.startsWith("__")) {
			return Node.createAnon(new AnonId(prologAtom));
		} else if(prologAtom.startsWith("literal(")) {
			// TODO: literal parsing!
			return Node.createLiteral(prologAtom);
		} else {
			return Node.createURI(prologAtom);
		}
	}
	
	private static String formatNode(Node n) {
		// much TODO here
		if(n.isURI()) {
			return "'" + n.getURI() + "'";
		} else if(n.isBlank()) {
			return "'" + n.getBlankNodeId().getLabelString() + "'";
		} else {
			return n.toString();
		}
	}
	
	
	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#dependsOn(com.hp.hpl.jena.graph.Graph)
	 */
	public boolean dependsOn(Graph arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#queryHandler()
	 */
	public QueryHandler queryHandler() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#getTransactionHandler()
	 */
	public TransactionHandler getTransactionHandler() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#getBulkUpdateHandler()
	 */
	public BulkUpdateHandler getBulkUpdateHandler() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#getCapabilities()
	 */
	public Capabilities getCapabilities() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#getEventManager()
	 */
	public GraphEventManager getEventManager() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#getReifier()
	 */
	public Reifier getReifier() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#getPrefixMapping()
	 */
	public PrefixMapping getPrefixMapping() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#delete(com.hp.hpl.jena.graph.Triple)
	 */
	public void delete(Triple arg0) throws DeleteDeniedException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#find(com.hp.hpl.jena.graph.TripleMatch)
	 */
	public ExtendedIterator find(TripleMatch arg0) {
		return find(arg0.getMatchSubject(),arg0.getMatchPredicate(),arg0.getMatchObject());
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#find(com.hp.hpl.jena.graph.Node, com.hp.hpl.jena.graph.Node, com.hp.hpl.jena.graph.Node)
	 */
	public ExtendedIterator find(Node arg0, Node arg1, Node arg2) {
		
		String s,p,o;
		
		s = arg0==null?"S":formatNode(arg0);
		p = arg1==null?"P":formatNode(arg1);
		o = arg2==null?"O":formatNode(arg2);
		
		//String prologQuery = "setof(f("+s+","+p+","+o+"),triple("+s+","+p+","+o+"),L), buildTermModel(L,TM)";
		String prologQuery = "setof(f(S,P,O),triple(S,P,O),L), buildTermModel(L,TM)";
		
//		Object[] bindings = engine.deterministicGoal(prologQuery,"[TM]");
//		TermModel[] list = ((TermModel)bindings[0]).flatList();
		
		List<Triple> result = new LinkedList<Triple>();
//		for(TermModel tm : list) {
//			String s_str = tm.getChild(0).toString();
//			String p_str = tm.getChild(1).toString();
//			String o_str = tm.getChild(2).toString();
//			
//			Triple t = new Triple(createNode(s_str),createNode(p_str),createNode(o_str));
//			
//			result.add(t);
//		}
		
		return WrappedIterator.create(result.iterator());
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#isIsomorphicWith(com.hp.hpl.jena.graph.Graph)
	 */
	public boolean isIsomorphicWith(Graph arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#contains(com.hp.hpl.jena.graph.Node, com.hp.hpl.jena.graph.Node, com.hp.hpl.jena.graph.Node)
	 */
	public boolean contains(Node arg0, Node arg1, Node arg2) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#contains(com.hp.hpl.jena.graph.Triple)
	 */
	public boolean contains(Triple arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#close()
	 */
	public void close() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#isEmpty()
	 */
	public boolean isEmpty() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.Graph#size()
	 */
	public int size() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.hp.hpl.jena.graph.GraphAdd#add(com.hp.hpl.jena.graph.Triple)
	 */
	public void add(Triple arg0) throws AddDeniedException {
		// TODO Auto-generated method stub

	}

//	public List<String> myListClasses() {
//		Object[] bindings = engine.deterministicGoal("setof(X,class(X),L), buildTermModel(L,TM)","[TM]");
//		TermModel[] list = ((TermModel)bindings[0]).flatList();
//		
//		List<String> result = new LinkedList<String>();
//		for(TermModel tm : list) {
//			result.add(tm.toString());
//		}
//		
//		return result;
//	}
	
}
